      SUBROUTINE CDDFF (DD,FF,IDD,IFF)
C
C**** *CDDFF*
C
C      PURPOSE.
C     ---------
C
C           CHECK CONSISTENCY OF WIND SPEED AND DIRECTION
C
C      INTERFACE.
C     ------------
C
C           CALL CDDFF (DD,FF,IDD,IFF)
C
C       INPUT
C         DD    -  VALUE OF WIND DIRECTION
C         FF    -  VALUE OF WIND SPEED
C         IDD   -  POSITION OF CONFIDENCE FLAG FOR DD
C         IFF   -  POSITION OF CONFIDENCE FLAG FOR FF
C
C       OUTPUT
C         IFLAG (CONF) - CONFIDENCE FLAGS UPDATED
C
C      METHOD.
C     ---------
C
C          CHECK WHETHER ONE COMPONENT IS MISSING AND THE OTHER NOT
C          CHECK WHETHER ONE COMPONENT IS ZERO AND THE OTHER NOT
C          CHECK WHETHER DD HAS PERMITTED VALUE
C          IF DD IS VARIABLE, CHECK FF IS WITHIN LIMITS
C
C      EXTERNALS.
C     ------------
C
C         SETGRS  -  UPDATE CONFIDENCE FOR GROSS LIMIT CHECK
C         SETINT  -  UPDATE CONFIDENCE FOR INTERNAL CONSISTENCY CHECK
C
C      REFERENCE.
C     ------------
C
C         GUIDE ON THE GLOBAL DATA PROCESSING SYSTEM WMO N305 1982
C              CHAPTER 6 - QUALITY CONTROL PROCEDURES
C
C      AUTHOR.
C     ---------
C
C         B. NORRIS,  ECMWF,  JANUARY 1989.
C
C      MODIFICATIONS.
C     ----------------
C
C         B. NORRIS, 15/06/89,  LIGHT AND VARIABLE ASSUMED TO BE
C                               DD=0,FF>0
C
C
      INCLUDE 'const.f'
C
C    --------------------------------------------------------------------
C
C                1.  CHECKING OF DD AND FF
C               ---------------------------
C
  100 CONTINUE
      IF(DD.EQ.RMISS.AND.FF.EQ.RMISS) GO TO 200
C
C         1.1  ONE OF DD OR FF MISSING
C
      IF(DD.EQ.RMISS) CALL SETINT (IFAIL,IFF,IEND)
      IF(FF.EQ.RMISS) CALL SETINT (IFAIL,IDD,IEND)
C
      IF(DD.NE.RMISS) THEN
C
C         1.2  CHECK DD WITHIN PERMITTED VALUES
C
           IF((DD.GE.0.0.AND.DD.LE.360.0).OR.DD.EQ.RVAR) GO TO 122
           CALL SETGRS (IDD,1)
  122      CONTINUE
C
           IF(FF.NE.RMISS) THEN
C
C         1.3  CHECK WHETHER DD IS NON-ZERO AND FF IS ZERO
C
                IF(DD.NE.0.0.AND.FF.EQ.0.0)
     X            CALL SETINT (IFAIL,IDD,IFF,IEND)
C
C         1.4  VARIABLE DIRECTION, LIMIT VALUE FOR FF
C
                IF(DD.EQ.RVAR) THEN
                     IF(FF.GT.3.0.AND.FF.LE.6.0) THEN
                           CALL SETINT (IFAIL,IDD,IEND)
                           CALL SETGRS (IFF,3)
                     ELSEIF(FF.GT.6.0) THEN
                           CALL SETINT (IFAIL,IDD,IEND)
                           CALL SETGRS (IFF,2)
                     ELSE
                           CALL SETINT (IPASS,IDD,IFF,IEND)
                     ENDIF
                ENDIF
           ENDIF
      ENDIF
C
C    -------------------------------------------------------------------
C
C                     2.   EXIT
C                    -------------
C
  200 CONTINUE
      RETURN
      END
      SUBROUTINE CSLT (IERR,P,YP,IP,RLAT,RLONG)
C
C**** *CSLT*
C
C      PURPOSE.
C     ---------
C
C             CHECK SURFACE PARAMETER FOR GROSS LIMIT CHECK
C
C
C      INTERFACE.
C     ------------
C
C         CALL CSLT (IERR,P,YP,IP,RLAT,RLONG)
C
C      INPUT
C        P      -  VALUE OF PARAMETER
C        YP     -  NAME OF PARAMETER
C        IP     -  POSITION OF CONFIDENCE FLAG FOR PARAMETER
C        RLAT   -  LATITUDE VALUE FOR REPORT
C        RLONG  -  LONGITUDE VALUE FOR REPORT
C
C      OUTPUT
C        IERR   -  ERROR CODE
C        IFLAG (CONF)  -  CONFIDENCE FLAG FOR PARAMETER UPDATED
C
C      METHOD.
C     ---------
C
C        GET GROSS LIMITS FROM CORRECT TABLE
C        UPDATE CONFIDENCE ACCORDINGLY
C
C      EXTERNALS.
C     ------------
C
C         GETLIM  -  GET GROSS LIMITS FROM TABLE
C         SETSF   -  UPDATE CONFIDENCE FLAG DEPENDING UPON SEVERITY
C                      OF FAILURE
C
C      REFERENCE.
C     ------------
C
C         GUIDE ON THE GLOBAL DATA PROCSEEING SYSTEM WMO N305 1982
C           CHAPTER 6, QUALITY CONTROL PROCEDURES
C
C      AUTHOR.
C     ---------
C
C         B. NORRIS,  ECMWF,  JANUARY 1989.
C
C      MODIFICATIONS.
C     -----------------
C
C         NONE
C
      INCLUDE 'paramq.f'
      INCLUDE 'const.f'
      INCLUDE 'ctsf.f'
C
      DIMENSION VLIM(NMM)
      CHARACTER*(*) YP
      IERR=0
C
C    --------------------------------------------------------------------
C
C                 1.  GET GROSS LIMITS FROM APPROPRIATE TABLE
C                -----------------------------------------------
C
  100 CONTINUE
C
C        1.1  WIND SPEED
C
      IF(YP.EQ.'FF') THEN
           CALL GETLIM (RLAT,RLONG,SFF,VLIM)
C
C        1.2  TEMPERATURE - CONVERT TO DEG K
C
      ELSEIF(YP.EQ.'TT') THEN
           CALL GETLIM (RLAT,RLONG,STT,VLIM)
           DO 122 I=1,NMM
           VLIM(I)=VLIM(I)+RABS
  122      CONTINUE
C
C         1.3  DEW POINT - CONVERT TO DEG K
C
      ELSEIF(YP.EQ.'TD') THEN
           CALL GETLIM (RLAT,RLONG,STD,VLIM)
           DO 132 I=1,NMM
           VLIM(I)=VLIM(I)+RABS
  132      CONTINUE
C
C         1.4  STATION LEVEL PRESSURE
C
      ELSEIF(YP.EQ.'PS') THEN
           CALL GETLIM (RLAT,RLONG,SPS,VLIM)
C
C         1.5  MEAN SEA LEVEL PRESSURE
C
      ELSEIF(YP.EQ.'PPP') THEN
           CALL GETLIM (RLAT,RLONG,SPPP,VLIM)
C
C         1.6  PRESSURE TENDENCY
C
      ELSEIF(YP.EQ.'PP') THEN
           CALL GETLIM (RLAT,RLONG,SPP,VLIM)
C
C         1.7  SEA SURFACE TEMPERATURE - CONVERT TO DEG K
C
      ELSEIF(YP.EQ.'TW') THEN
           CALL GETLIM (RLAT,RLONG,STW,VLIM)
           DO 172 I=1,NMM
           VLIM(I)=VLIM(I)+RABS
  172      CONTINUE
      ELSE
C
C        1.8  PARAMETER NOT CATERED FOR
C
           IERR=2001
           GO TO 300
      ENDIF
C
C    -------------------------------------------------------------------
C
C              2.  SET APPROPRIATE FLAGS
C             ---------------------------
C
  200 CONTINUE
      CALL SETSF (P,IP,VLIM)
C
C
C    ------------------------------------------------------------------
C
C                   3.    EXIT
C                 --------------
C
  300 CONTINUE
      RETURN
      END
      SUBROUTINE CUAT (IERR,P,YP,IP,RLAT,RLONG,PP1,PP2,Z,
     X                                        IPP1,IPP2,IZ)
C
C**** *CUAT*
C
C      PURPOSE.
C     ----------
C
C             CHECK UPPER AIR PARAMETER FOR GROSS LIMIT CHECK
C
C      INTERFACE.
C     ------------
C
C         CALL CUAT (IERR,P,YP,IP,RLAT,RLONG,PP1,PP2,Z,IPP1,IPP2,IZ)
C
C        INPUT
C         P      -  VALUE OF PARAMETER
C         YP     -  NAME OF PARAMETER
C         IP     -  POSITION OF CONFIDENCE FLAG FOR PARAMETER
C         RLAT   -  LATITUDE VALUE FOR REPORT
C         RLONG  -  LONGITUDE VALUE FOR REPORT
C         PP1    -  PRESSURE LEVEL OR BASE OF LAYER
C         PP2    -  TOP OF LAYER
C         Z      -  HEIGHT LEVEL
C         IPP1   -  POSITION OF CONFIDENCE FLAG FOR PP1
C         IPP2   -  POSITION OF CONFIDENCE FLAG FOR PP2
C         IZ     -  POSITION OF CONFIDENCE FLAG FOR Z
C
C        OUTPUT
C         IERR   -  ERROR CODE
C         IFLAG (CONF)  -  CONFIDENCE FLAGS UPDATED
C
C      METHOD.
C     ---------
C
C         GET GROSS LIMITS FROM CORRECT TABLE
C         UPDATE CONFIDENCE FLAGS ACCORDINGLY
C
C       EXTERNALS.
C      ------------
C
C         GUALIM - GET GROSS LIMITS FROM TABLE
C         SETGRS - UPDATE CONFIDENCE FLAGS FOR GROSS LIMIT CHECK
C         SETINT - UPDATE CONFIDENCE FLAGS FOR INT CONSISTENCY CHECK
C
C       REFERENCE.
C      ------------
C
C         GUIDE ON THE GLOBAL DATA PROCESSING SYSTEM WMO N305 1982
C           CHAPTER 6, QUALITY CONTROL PROCEDURES
C
C       AUTHOR.
C      ---------
C
C         B. NORRIS,  ECMWF,  JANUARY 1989.
C
C       MODIFICATIONS.
C      ----------------
C
C         NONE
C
      INCLUDE 'paramq.f'
      INCLUDE 'const.f'
      INCLUDE 'ctua.f'
      INCLUDE 'conf.f'
C
      DIMENSION VLIM(NMM)
      CHARACTER*(*) YP
      LOGICAL LAYER,LSUM
      IERR=0
C
C    --------------------------------------------------------------------
C
C            1.  GET GROSS LIMITS FROM APPROPRIATE TABLE
C           ----------------------------------------------
C
  100 CONTINUE
C
C         1.1  GEOPOTENTIAL
C
      IF(YP.EQ.'Z') THEN
           LAYER=.FALSE.
           CALL GUALIM (IERR,UPZ,UZ,RLAT,RLONG,PP1,PP2,Z,
     X                  LAYER,LSUM,VLIM)
C
C           1.2  TEMPERATURE - CONVERT TO DEG K
C
      ELSEIF(YP.EQ.'TT') THEN
           LAYER=.FALSE.
           CALL GUALIM (IERR,UPZ,UTT,RLAT,RLONG,PP1,PP2,Z,
     X                  LAYER,LSUM,VLIM)
           DO 122 I=1,NMM
           VLIM(I)=VLIM(I)+RABS
  122      CONTINUE
C
C           1.3  WIND SPEED
C
      ELSEIF(YP.EQ.'FF') THEN
           LAYER=.FALSE.
           CALL GUALIM (IERR,UPZ,UFF,RLAT,RLONG,PP1,PP2,Z,
     X                  LAYER,LSUM,VLIM)
C
C           1.4  LAYER MEAN TEMPERATURE - CONVERT TO DEG K
C
      ELSEIF(YP.EQ.'TM') THEN
           LAYER=.TRUE.
           LSUM=.FALSE.
           CALL GUALIM (IERR,UPZ,UTT,RLAT,RLONG,PP1,PP2,Z,
     X                  LAYER,LSUM,VLIM)
           DO 142 I=1,NMM
           VLIM(I)=VLIM(I)+RABS
  142      CONTINUE
C
C           1.5  THICKNESS
C
      ELSEIF(YP.EQ.'TL') THEN
           LAYER=.TRUE.
           LSUM=.TRUE.
           CALL GUALIM (IERR,UPZ,UTL,RLAT,RLONG,PP1,PP2,Z,
     X                  LAYER,LSUM,VLIM)
C
C           1.6  PRECIPITABLE WATER
C
      ELSEIF(YP.EQ.'WL') THEN
           LAYER=.TRUE.
           LSUM=.TRUE.
           CALL GUALIM (IERR,UPZ,UWL,RLAT,RLONG,PP1,PP2,Z,
     X                  LAYER,LSUM,VLIM)
      ELSE
C
C          1.7   PARAMETER NOT CATERED FOR
C
           IERR=2401
      ENDIF
      IF(IERR.NE.0) GO TO 300
C
C    --------------------------------------------------------------------
C
C                2.  SET APPROPRIATE FLAGS
C               ---------------------------
C
  200 CONTINUE
      IF(P.LT.VLIM(1)) THEN
C
C          2.1  P<MIN2
C
           CALL SETINT (ICBO(2,2),IPP1,IPP2,IZ,IEND)
           CALL SETGRS (IP,2)
      ELSEIF(P.GE.VLIM(1).AND.P.LT.VLIM(2)) THEN
C
C          2.2  MIN2<P<MIN1
C
           CALL SETINT (ICBO(3,2),IPP1,IPP2,IZ,IEND)
           CALL SETGRS (IP,3)
      ELSEIF(P.GE.VLIM(2).AND.P.LE.VLIM(3)) THEN
C
C          2.3  MIN1<P<MAX1
C
           CALL SETINT (ICBO(4,2),IPP1,IPP2,IZ,IEND)
           CALL SETGRS (IP,4)
      ELSEIF(P.GT.VLIM(3).AND.P.LE.VLIM(4)) THEN
C
C          2.4  MAX1<P<MAX2
C
           CALL SETINT (ICBO(3,2),IPP1,IPP2,IZ,IEND)
           CALL SETGRS (IP,3)
      ELSEIF(P.GT.VLIM(4)) THEN
C
C          2.5  P>MAX2
C
           CALL SETINT (ICBO(2,2),IPP1,IPP2,IZ,IEND)
           CALL SETGRS (IP,2)
      ELSE
      ENDIF
C
C    ----------------------------------------------------------------------
C
C                  3.   EXIT
C                 ------------
C
  300 CONTINUE
      RETURN
      END
      SUBROUTINE DERITA (IERR,MPAR)
C
C**** *DERITA*
C
C       PURPOSE.
C      ----------
C
C              SET UP LIMIT TABLES AND INITIALISE CONFIDENCE ARRAY
C
C       INTERFACE.
C      -------------
C
C         CALL DERITA (IERR,MPAR)
C
C        INPUT
C         MPAR    -  NUMBER OF PARAMETERS IN REPORT
C
C        OUTPUT
C         IERR    -  ERROR CODE
C         GROSS LIMIT TABLES INITIALISED
C         IFLAG (CONF) - CONFIDENCE FLAGS INITIALISED
C         THICKNESS LIMIT TABLE DERIVED
C
C       METHOD.
C      ---------
C
C         SURFACE AND UPPER AIR LIMIT TABLES IN DATA STATEMENTS
C         CONFIDENCE FLAGS SET TO UNCHECKED VALUE
C         ON FIRST CALL, THICKNESS LIMIT TABLE DERIVED
C
C
C       EXTERNALS.
C      ------------
C
C         NONE
C
C       REFERENCE.
C      ------------
C
C         GUIDE ON THE GLOBAL DATA PROCESSING SYSTEM WMO N305 1982
C            CHAPTER 6, QUALITY CONTROL PROCEDURES
C
C       AUTHOR.
C      ---------
C
C         B. NORRIS,  ECMWF,  JANUARY 1989.
C
C       MODIFICATIONS.
C      ----------------
C
C         NONE
C
      INCLUDE 'paramq.f'
      INCLUDE 'ctua.f'
      INCLUDE 'const.f'
      INCLUDE 'conf.f'
      INCLUDE 'ctsf.f'
C
C    --------------------------------------------------------------------
C
C                  1.  SET CONSTANT VALUES
C                 --------------------------
C
  100 CONTINUE
C            MISSING DATA INDICATORS
      DATA RMISS /999999./,IMISS/999999/
C            GAS CONSTANT; GRAVITY; LAPSE RATE
      DATA R/287.0/,RG/9.8/,RGAMMA/0.0065/
C            ABSOLUTE TEMPERATURE; SPECIFIC HEAT OF DRY AIR
      DATA RABS/273.16/,RCP/1003.1/
C            VARIABLE WIND DIRECTION
      DATA RVAR/0.0/
C            LATITUDE LIMIT FOR DETERMINING AREA WITHIN TABLES
      DATA RLLIM/45.0/
C            PASS, FAIL, END INDICATORS
      DATA IPASS/1/,IFAIL/-1/,IEND/3HEND/
C            UNCHECKED FLAG VALUE
      DATA IUNCH/70/
C
C    -------------------------------------------------------------------
C
C               2.  SET CONFIDENCE CORRECTIONS
C              --------------------------------
C
  200 CONTINUE
C
C           2.1  GROSS LIMIT CHECKS
C
C            DIM1 = CONFIDENCE CORRECTION, DIM2 = PASS/FAIL
      DATA (ICBO(I,1),I=1,NPOSS)/
C       IMPOSS VALUE    OUTSIDE MAX2   MAX2>X>MAX1    MAX1>X>MIN1
     X     -100,            -60,           -30,            0/
      ICBO (1,2)=IFAIL
      ICBO (2,2)=IFAIL
      ICBO (3,2)=0
      ICBO (4,2)=IPASS
C
C            2.2  INTERNAL CONSISTENCY CONFIDENCE UPDATES
C
      DATA ((ICOCO(I,J),I=1,NCOCO),J=1,2)/
C        FAIL        1ST   2ND   3RD   4TH   5TH
     X               -10,  -25,  -20,  -15,  -10,
C        PASS        1ST   2ND   3RD   4TH   5TH
     X                 5,    4,    3,    2,    1/
C
C    --------------------------------------------------------------------
C
C              3.  GROSS LIMITS FOR SURFACE PARAMETERS
C            -------------------------------------------
C
  300 CONTINUE
C
C          3.1  LIMIT TABLE FOR SURFACE WIND SPEED
C
      DATA (((SFF(I,J,K),I=1,NMM),J=1,NSEASN),K=1,NAREA)/
C                WINTER                        SUMMER
C        MIN2  MIN1   MAX1  MAX2       MIN2  MIN1   MAX1  MAX2
     X   0.0,  0.0,  60.0, 125.0,      0.0,  0.0,  90.0, 150.0,
     X   0.0,  0.0,  50.0, 100.0,      0.0,  0.0,  40.0,  75.0/
C
C          3.2  LIMIT TABLE FOR SURFACE TEMPERATURE
C
      DATA (((STT(I,J,K),I=1,NMM),J=1,NSEASN),K=1,NAREA)/
C                WINTER                        SUMMER
C        MIN2  MIN1   MAX1  MAX2       MIN2  MIN1   MAX1  MAX2
     X -40.0,-30.0,  50.0,  55.0,    -30.0,-20.0,  50.0,  60.0,
     X -90.0,-80.0,  35.0,  40.0,    -40.0,-30.0,  40.0,  50.0/
C
C          3.3  LIMIT TABLE FOR SURFACE DEWPOINT
C
      DATA (((STD(I,J,K),I=1,NMM),J=1,NSEASN),K=1,NAREA)/
C                WINTER                        SUMMER
C        MIN2  MIN1   MAX1  MAX2       MIN2  MIN1   MAX1  MAX2
     X -45.0,-35.0,  35.0,  40.0,    -35.0,-25.0,  35.0,  40.0,
     X -99.0,-85.0,  30.0,  35.0,    -45.0,-35.0,  35.0,  40.0/
C
C          3.4  LIMIT TABLE FOR STATION PRESSURE
C
      DATA (((SPS(I,J,K),I=1,NMM),J=1,NSEASN),K=1,NAREA)/
C        MIN2      MIN1       MAX1      MAX2
C                WINTER   AREA 1
     X 30000.0,  40000.0,  108000.0,  110000.0,
C                SUMMER   AREA 1
     X 30000.0,  40000.0,  108000.0,  110000.0,
C                WINTER   AREA 2
     X 30000.0,  40000.0,  108000.0,  110000.0,
C                SUMMER   AREA 2
     X 30000.0,  40000.0,  108000.0,  110000.0/
C
C          3.5  LIMIT TABLE FOR MEAN SEA LEVEL PRESSURE
C
      DATA (((SPPP(I,J,K),I=1,NMM),J=1,NSEASN),K=1,NAREA)/
C        MIN2      MIN1       MAX1      MAX2
C                WINTER   AREA 1
     X 87000.0,  91000.0,  108000.0,  110000.0,
C                SUMMER   AREA 1
     X 85000.0,  90000.0,  108000.0,  110000.0,
C                WINTER   AREA 2
     X 88000.0,  91000.0,  108000.0,  110000.0,
C                SUMMER   AREA 2
     X 88000.0,  92000.0,  108000.0,  110000.0/
C
C          3.6  LIMIT TABLE FOR 3-HOUR MEAN SEA LEVEL PRESSURE TENDENCY
C
      DATA (((SPP(I,J,K),I=1,NMM),J=1,NSEASN),K=1,NAREA)/
C        MIN2      MIN1       MAX1       MAX2
C                WINTER   AREA 1
     X -5000.0,  -4000.0,    4000.0,    5000.0,
C                SUMMER   AREA 1
     X -5000.0,  -4000.0,    4000.0,    5000.0,
C                WINTER   AREA 2
     X -5000.0,  -4000.0,    4000.0,    5000.0,
C                SUMMER   AREA 2
     X -5000.0,  -4000.0,    4000.0,    5000.0/
C
C          3.7  LIMIT TABLE FOR SEA SURFACE TEMPERATURE
C
      DATA (((STW(I,J,K),I=1,NMM),J=1,NSEASN),K=1,NAREA)/
C                WINTER                        SUMMER
C        MIN2  MIN1   MAX1  MAX2       MIN2  MIN1   MAX1  MAX2
     X   0.0,  2.0,  32.0,  35.0,      0.0,  2.0,  32.0,  35.0,
     X  -2.1, -1.0,  27.0,  30.0,     -2.1, -1.0,  30.0,  35.0/
C
C    -------------------------------------------------------------------
C
C                4.  TABLE FOR UPPER AIR LEVELS
C               --------------------------------
C
  400 CONTINUE
      DATA ((UPZ(I,J),J=1,2),I=1,10)/
C              PRESSURE          HEIGHT
     X         110000.0,         -600.0,
     X         100000.0,          300.0,
     X          85000.0,         1500.0,
     X          70000.0,         3000.0,
     X          50000.0,         5500.0,
     X          40000.0,         7000.0,
     X          30000.0,         9000.0,
     X          25000.0,        10000.0,
     X          20000.0,        12000.0,
     X          15000.0,        14000.0/
      DATA ((UPZ(I,J),J=1,2),I=11,NLEV)/
C              PRESSURE          HEIGHT
     X          10000.0,        16500.0,
     X           7000.0,        18500.0,
     X           5000.0,        20000.0,
     X           3000.0,        22000.0,
     X           2000.0,        26000.0,
     X           1000.0,        30000.0,
     X            700.0,        33000.0,
     X            500.0,        36000.0,
     X            300.0,        39000.0,
     X            200.0,        42000.0,
     X            100.0,        48000.0,
     X              1.0,        99999.0/
C
C    -------------------------------------------------------------------
C
C                5.  GROSS LIMITS FOR UPPER AIR PARAMETERS
C              --------------------------------------------
C
  500 CONTINUE
C
C           5.1  LIMIT TABLE FOR UPPER AIR HEIGHTS
C
      DATA (((UZ(I,J,K),I=1,NMM),J=1,NAREA),K=1,10)/
C                  AREA1                         AREA2
C        MIN2     MIN1    MAX1    MAX2    MIN2    MIN1    MAX1    MAX2
     X -1800.0,-1600.0, -200.0,    0.0,-1500.0,-1350.0, -150.0,    0.0,
     X -1000.0, -800.0,  600.0,  800.0, -700.0, -550.0,  650.0,  800.0,
     X     0.0,  200.0, 2000.0, 2200.0,  200.0,  400.0, 2000.0, 2200.0,
     X  2200.0, 2350.0, 3450.0, 3600.0, 2300.0, 2450.0, 3450.0, 3600.0,
     X  4500.0, 4700.0, 6100.0, 6300.0, 4500.0, 4700.0, 6100.0, 6300.0,
     X  6100.0, 6300.0, 7800.0, 8000.0, 6100.0, 6300.0, 7800.0, 8000.0,
     X  7300.0, 7550.0, 9800.0,10100.0, 7300.0, 7550.0, 9800.0,10100.0,
     X  8500.0, 8800.0,11100.0,11400.0, 8500.0, 8800.0,11100.0,11400.0,
     X 10000.0,10300.0,12900.0,13200.0,10000.0,10300.0,12900.0,13200.0,
     X 12000.0,12300.0,14900.0,15200.0,12000.0,12300.0,14900.0,15200.0/
      DATA (((UZ(I,J,K),I=1,NMM),J=1,NAREA),K=11,NLEV)/
C                  AREA1                         AREA2
C        MIN2     MIN1    MAX1    MAX2    MIN2    MIN1    MAX1    MAX2
     X 14000.0,14400.0,17700.0,18100.0,14000.0,14400.0,17700.0,18100.0,
     X 15500.0,16100.0,20900.0,21500.0,15500.0,16100.0,20900.0,21500.0,
     X 17700.0,18300.0,23100.0,23700.0,17700.0,18300.0,23100.0,23700.0,
     X 20500.0,21100.0,25900.0,26500.0,20500.0,21100.0,25900.0,26500.0,
     X 23300.0,23900.0,29700.0,30300.0,23300.0,23900.0,29700.0,30300.0,
     X 26000.0,26800.0,33200.0,34000.0,26000.0,26800.0,33200.0,34000.0,
     X 30700.0,31300.0,35800.0,36400.0,30700.0,31300.0,35800.0,36400.0,
     X 33300.0,33800.0,37800.0,38300.0,33300.0,33800.0,37800.0,38300.0,
     X 36600.0,37100.0,41600.0,42100.0,36600.0,37100.0,41600.0,42100.0,
     X 39400.0,39800.0,44400.0,44900.0,39400.0,39800.0,44400.0,44900.0,
     X 44900.0,45500.0,50100.0,50700.0,44900.0,45500.0,50100.0,50700.0,
     X 99999.0,99999.0,99999.0,99999.0,99999.0,99999.0,99999.0,99999.0/
C
C           5.2  LIMIT TABLE FOR UPPER AIR TEMPERATURES
C
      DATA (((UTT(I,J,K),I=1,NMM),J=1,NAREA),K=1,10)/
C                  AREA1                         AREA2
C        MIN2     MIN1    MAX1    MAX2    MIN2    MIN1    MAX1    MAX2
     X   -50.0,  -30.0,   50.0,   60.0,  -90.0,  -70.0,   40.0,   50.0,
     X   -50.0,  -30.0,   50.0,   60.0,  -90.0,  -70.0,   40.0,   50.0,
     X   -65.0,  -50.0,   30.0,   40.0,  -90.0,  -70.0,   20.0,   30.0,
     X   -80.0,  -70.0,   20.0,   30.0,  -90.0,  -70.0,   10.0,   20.0,
     X   -95.0,  -80.0,    5.0,   10.0, -100.0,  -80.0,   -5.0,    5.0,
     X  -100.0,  -85.0,   -5.0,    0.0, -100.0,  -85.0,  -10.0,   -5.0,
     X  -100.0,  -85.0,  -10.0,   -5.0, -100.0,  -85.0,  -10.0,   -5.0,
     X  -100.0,  -85.0,  -10.0,   -5.0, -100.0,  -85.0,  -10.0,   -5.0,
     X  -100.0,  -85.0,  -10.0,   -5.0, -100.0,  -85.0,  -10.0,   -5.0,
     X  -100.0,  -85.0,  -10.0,   -5.0, -100.0,  -85.0,  -10.0,   -5.0/
         DATA (((UTT(I,J,K),I=1,NMM),J=1,NAREA),K=11,NLEV)/
C                  AREA1                         AREA2
C        MIN2     MIN1    MAX1    MAX2    MIN2    MIN1    MAX1    MAX2
     X  -100.0,  -85.0,  -10.0,   -5.0, -100.0,  -85.0,  -10.0,   -5.0,
     X  -100.0,  -85.0,   -5.0,    5.0, -100.0,  -85.0,   -5.0,    5.0,
     X  -100.0,  -85.0,   -5.0,    5.0, -100.0,  -85.0,   -5.0,    5.0,
     X  -100.0,  -85.0,   -5.0,    5.0, -100.0,  -85.0,   -5.0,    5.0,
     X  -100.0,  -85.0,   -5.0,    5.0, -100.0,  -85.0,   -5.0,    5.0,
     X  -100.0,  -85.0,   -5.0,    5.0, -100.0,  -85.0,   -5.0,    5.0,
     X   -90.0,  -80.0,   10.0,   20.0,  -90.0,  -80.0,   10.0,   20.0,
     X   -80.0,  -70.0,   15.0,   30.0,  -80.0,  -70.0,   15.0,   30.0,
     X   -70.0,  -60.0,   25.0,   35.0,  -70.0,  -60.0,   25.0,   35.0,
     X   -70.0,  -60.0,   30.0,   40.0,  -70.0,  -60.0,   30.0,   40.0,
     X   -70.0,  -60.0,   30.0,   40.0,  -70.0,  -60.0,   30.0,   40.0,
     X   -70.0,  -60.0,   30.0,   40.0,  -70.0,  -60.0,   30.0,   40.0/
C
C           5.3  LIMIT TABLE FOR UPPER AIR WIND SPEED
C
      DATA (((UFF(I,J,K),I=1,NMM),J=1,NAREA),K=1,10)/
C                  AREA1                         AREA2
C        MIN2     MIN1    MAX1    MAX2    MIN2    MIN1    MAX1    MAX2
     X     0.0,    0.0,   60.0,  100.0,    0.0,    0.0,   60.0,  100.0,
     X     0.0,    0.0,   60.0,  100.0,    0.0,    0.0,   60.0,  100.0,
     X     0.0,    0.0,   65.0,  100.0,    0.0,    0.0,   65.0,  100.0,
     X     0.0,    0.0,   70.0,  100.0,    0.0,    0.0,   70.0,  100.0,
     X     0.0,    0.0,  100.0,  120.0,    0.0,    0.0,  100.0,  120.0,
     X     0.0,    0.0,  130.0,  150.0,    0.0,    0.0,  130.0,  150.0,
     X     0.0,    0.0,  160.0,  180.0,    0.0,    0.0,  160.0,  180.0,
     X     0.0,    0.0,  160.0,  180.0,    0.0,    0.0,  160.0,  180.0,
     X     0.0,    0.0,  160.0,  180.0,    0.0,    0.0,  160.0,  180.0,
     X     0.0,    0.0,  150.0,  170.0,    0.0,    0.0,  150.0,  170.0/
         DATA (((UFF(I,J,K),I=1,NMM),J=1,NAREA),K=11,NLEV)/
C                  AREA1                         AREA2
C        MIN2     MIN1    MAX1    MAX2    MIN2    MIN1    MAX1    MAX2
     X     0.0,    0.0,  150.0,  170.0,    0.0,    0.0,  150.0,  170.0,
     X     0.0,    0.0,  150.0,  170.0,    0.0,    0.0,  150.0,  170.0,
     X     0.0,    0.0,  150.0,  170.0,    0.0,    0.0,  150.0,  170.0,
     X     0.0,    0.0,   90.0,  110.0,    0.0,    0.0,   90.0,  110.0,
     X     0.0,    0.0,   90.0,  110.0,    0.0,    0.0,   90.0,  110.0,
     X     0.0,    0.0,   75.0,   95.0,    0.0,    0.0,   75.0,   95.0,
     X     0.0,    0.0,   80.0,  100.0,    0.0,    0.0,   80.0,  100.0,
     X     0.0,    0.0,  120.0,  140.0,    0.0,    0.0,  120.0,  140.0,
     X     0.0,    0.0,  150.0,  170.0,    0.0,    0.0,  150.0,  170.0,
     X     0.0,    0.0,  200.0,  220.0,    0.0,    0.0,  200.0,  220.0,
     X     0.0,    0.0,  200.0,  220.0,    0.0,    0.0,  200.0,  220.0,
     X     0.0,    0.0,  200.0,  220.0,    0.0,    0.0,  200.0,  220.0/
C
C           5.4  LIMIT TABLE FOR UPPER AIR PRECIPITABLE WATER
C
      DATA (((UWL(I,J,K),I=1,NMM),J=1,NAREA),K=1,10)/
C                  AREA1                         AREA2
C        MIN2     MIN1    MAX1    MAX2    MIN2    MIN1    MAX1    MAX2
     X     0.0,    0.0,  0.100,  0.150,    0.0,    0.0,  0.100,  0.150,
     X     0.0,    0.0,  0.100,  0.150,    0.0,    0.0,  0.100,  0.150,
     X     0.0,    0.0,  0.040,  0.060,    0.0,    0.0,  0.040,  0.060,
     X     0.0,    0.0,  0.030,  0.045,    0.0,    0.0,  0.030,  0.045,
     X     0.0,    0.0,  0.020,  0.030,    0.0,    0.0,  0.020,  0.030,
     X     0.0,    0.0,  0.015,  0.025,    0.0,    0.0,  0.015,  0.025,
     X     0.0,    0.0,  0.010,  0.015,    0.0,    0.0,  0.010,  0.015,
     X     0.0,    0.0,  0.005,  0.008,    0.0,    0.0,  0.005,  0.008,
     X     0.0,    0.0,  0.001,  0.002,    0.0,    0.0,  0.001,  0.002,
     X     0.0,    0.0,  0.001,  0.002,    0.0,    0.0,  0.001,  0.002/
         DATA (((UWL(I,J,K),I=1,NMM),J=1,NAREA),K=11,NLEV)/
C                  AREA1                         AREA2
C        MIN2     MIN1    MAX1    MAX2    MIN2    MIN1    MAX1    MAX2
     X     0.0,    0.0,  0.001,  0.002,    0.0,    0.0,  0.001,  0.002,
     X     0.0,    0.0,  0.001,  0.002,    0.0,    0.0,  0.001,  0.002,
     X     0.0,    0.0,  0.001,  0.002,    0.0,    0.0,  0.001,  0.002,
     X     0.0,    0.0,  0.001,  0.002,    0.0,    0.0,  0.001,  0.002,
     X     0.0,    0.0,  0.001,  0.002,    0.0,    0.0,  0.001,  0.002,
     X     0.0,    0.0,  0.001,  0.002,    0.0,    0.0,  0.001,  0.002,
     X     0.0,    0.0,  0.001,  0.002,    0.0,    0.0,  0.001,  0.002,
     X     0.0,    0.0,  0.001,  0.002,    0.0,    0.0,  0.001,  0.002,
     X     0.0,    0.0,  0.001,  0.002,    0.0,    0.0,  0.001,  0.002,
     X     0.0,    0.0,  0.001,  0.002,    0.0,    0.0,  0.001,  0.002,
     X     0.0,    0.0,  0.001,  0.002,    0.0,    0.0,  0.001,  0.002,
     X     0.0,    0.0,  0.001,  0.002,    0.0,    0.0,  0.001,  0.002/
C
C    ------------------------------------------------------------------
C
C              6.   DERIVE THICKNESS LIMITS
C             -------------------------------
C
  600 CONTINUE
      IERR=0
      IF(IFIRST.NE.3H1ST) THEN
           DO 606 IAREA=1,NAREA
           DO 604 IMM=1,NMM
           DO 602 ILEV=1,NLEV-1
           UTL(IMM,IAREA,ILEV)=(R/RG)*
     X     ((UTT(IMM,IAREA,ILEV)+RABS+UTT(IMM,IAREA,ILEV+1)+RABS)/2.0)*
     X     ALOG(UPZ(ILEV,1)/UPZ(ILEV+1,1))
  602      CONTINUE
  604      CONTINUE
  606      CONTINUE
           IFIRST=3H1ST
      ENDIF
C
C    --------------------------------------------------------------------
C
C               7.  INITIALISE FLAG ARRAY
C             ------------------------------
C
  700 CONTINUE
      IF(MPAR.GT.NPARAM) THEN
           WRITE (*,'(1H ,''   FLAG ARRAY TOO SMALL, NUMBER OF ENTRIES R
     XEQUIRED:'',I7)') MPAR
           IERR=2901
           GO TO 800
      ENDIF
      DO 702 I=1,MPAR
C         CONFIDENCE = UNCHECKED VALUE
      IFLAG(1,I)=IUNCH
C         NUMBER OF FAILS
      IFLAG(2,I)=0
C         NUMBER OF PASSES
      IFLAG(3,I)=0
  702 CONTINUE
C
C    ------------------------------------------------------------------
C
C                    8.  EXIT
C                  -------------
C
  800 CONTINUE
      RETURN
      END
      SUBROUTINE DPLIM (Z,RLAT,RLONG,PLIM)
C
C**** *DPLIM*
C
C       PURPOSE.
C      ----------
C
C           OBTAIN LIMITS OF PRESSURE GIVEN HEIGHT
C
C       INTERFACE.
C      ------------
C
C         CALL DPLIM (Z,RLAT,RLONG,PLIM)
C
C       INPUT
C        Z     -  HEIGHT LEVEL
C        RLAT  -  LATITUDE VALUE FOR REPORT
C        RLONG -  LONGITUDE VALUE FOR REPORT
C
C       OUTPUT
C        PLIM  -  GROSS LIMITS FOR PRESSURE
C
C       METHOD.
C      ---------
C
C         FROM LIMIT TABLE FOR UPPER AIR HEIGHTS,
C         LINEARLY INTERPLOTATE TO OBTAIN GROSS LIMITS FOR PRESSURE
C
C       EXTERNALS.
C      ------------
C
C         NONE
C
C       REFERENCE.
C      ------------
C
C         GUIDE ON THE GLOBAL DATA PROCESSING SYSTEM WMO N305 1982
C          CHAPTER 6, QUALITY CONTROL PROCEDURES
C
C       AUTHOR.
C      ---------
C
C         B. NORRIS,  ECMWF,  JANUARY 1989.
C
C       MODIFICATIONS.
C      ----------------
C
C         NONE
C
      INCLUDE 'paramq.f'
      INCLUDE 'ctua.f'
      INCLUDE 'const.f'
C
      DIMENSION PLIM(NMM)
C
C    -------------------------------------------------------------------
C
C             1.  DETERMINE WHICH AREA REPORT FALLS INTO
C            --------------------------------------------
C
  100 CONTINUE
      IAREA=1
      IF(ABS(RLAT).GT.RLLIM) IAREA=2
C
C    --------------------------------------------------------------------
C
C             2.  LINEAR INTERPOLATION
C           -----------------------------
C
  200 CONTINUE
      DO 234 IMM=1,NMM
      IF(Z.GE.UZ(IMM,IAREA,NLEV)) THEN
C
C           2.1  Z OUTSIDE TOP OF TABLE
C
           PLIM(IMM)=UPZ(NLEV,1)
           GO TO 234
      ELSEIF(Z.LE.UZ(IMM,IAREA,1)) THEN
C
C           2.2  Z OUTSIDE BOTTOM OF TABLE
C
           PLIM(IMM)=UPZ(1,1)
           GO TO 234
      ELSE
C
C           2.3  INTERPOLATE
C
           DO 232 ILEV=1,NLEV-1
           IF(Z.GT.UZ(IMM,IAREA,ILEV).AND.Z.LE.UZ(IMM,IAREA,ILEV+1))
     X     THEN
                PLIM(IMM)=
     X          (UPZ(ILEV+1,1)*(Z-UZ(IMM,IAREA,ILEV))+
     X           UPZ(ILEV,1)  *(UZ(IMM,IAREA,ILEV+1)-Z))/
     X          (UZ(IMM,IAREA,ILEV+1)-UZ(IMM,IAREA,ILEV))
                GO TO 234
           ENDIF
  232      CONTINUE
      ENDIF
  234 CONTINUE
C
C    -------------------------------------------------------------------
C
C                   3.  EXIT
C                  ----------
C
  300 CONTINUE
      RETURN
      END
      SUBROUTINE GETLIM (RLAT,RLONG,TABLE,VLIM)
C
C**** *GETLIM*
C
C       PURPOSE.
C      ---------
C
C          GET LIMITS FOR SURFACE PARAMETER
C
C       INTERFACE.
C      -------------
C
C         CALL GETLIM (RLAT,RLONG,TABLE,VLIM)
C
C        INPUT
C         RLAT   -  LATITUDE VALUE FOR REPORT
C         RLONG  -  LONGITUDE VALUE FOR REPORT
C         TABLE  -  GROSS LIMIT TABLE FOR SURFACE PARAMETER
C
C        OUTPUT
C         VLIM   -  APPROPRIATE GROSS LIMITS
C
C       METHOD.
C      ---------
C
C         EXTRACT CORRECT SET OF LIMITS FROM TABLE
C
C       EXTERNALS.
C      ------------
C
C         NONE
C
C       REFERENCE.
C      ------------
C
C         GUIDE ON THE GLOBAL DATA PROCESSING SYSTEM WMO N305 1982
C           CHAPTER 6,  QUALITY CONTROL PROCEDURES
C
C       AUTHOR.
C      ---------
C
C         B. NORRIS,  ECMWF,  JANUARY 1989.
C
C       MODIFICATIONS.
C      ----------------
C
C         NONE
C
      INCLUDE 'paramq.f'
      INCLUDE 'const.f'
C
      DIMENSION VLIM(NMM),TABLE(NMM,NSEASN,NAREA)
C
C    ----------------------------------------------------------------------
C
C              1.  DETERMINE WHICH AREA REPORT FALLS INTO
C             --------------------------------------------
C
  100 CONTINUE
      IAREA=1
      IF(ABS(RLAT).GT.RLLIM) IAREA=2
C
C    ----------------------------------------------------------------------
C
C              2.  EXTRACT RELEVANT LIMITS FROM TABLE
C            ------------------------------------------
C
  200 CONTINUE
      DO 202 IMM=1,NMM
      VLIM(IMM)=TABLE(IMM,ISEASN,IAREA)
  202 CONTINUE
C
C    --------------------------------------------------------------------
C
C               3.  EXIT
C             ------------
C
  300 CONTINUE
      RETURN
      END
      SUBROUTINE GUALIM (IERR,UPZ,TABLE,RLAT,RLONG,PP1,PP2,Z,
     X     LAYER,LSUM,VLIM)
C
C**** *GUALIM*
C
C       PURPOSE.
C      ---------
C
C             GET PARAMETER LIMITS FROM UPPER AIR TABLE
C
C       INTERFACE.
C      -----------
C
C          CALL GUALIM (IERR,UPZ,TABLE,RLAT,RLONG,PP1,PP2,Z,
C                            LAYER,LSUM,VLIM)
C
C        INPUT
C         UPZ    -  TABLE OF STANDARD PRESSURE AND HEIGHT LEVELS
C         TABLE  -  GROSS LIMITS FOR PARAMETER
C         RLAT   -  LATITUDE VALUE FOR REPORT
C         RLONG  -  LONGITUDE VALUE FOR REPORT
C         PP1    -  PRESSURE LEVEL OR BASE OF LAYER
C         PP2    -  TOP OF LAYER
C         Z      -  HEIGHT LEVEL
C         LAYER  -  LOGICAL FOR PRESENCE OF LAYER
C         LSUM   -  LOGICAL FOR SUMMING OR MEANING
C
C        OUTPUT
C         VLIM   -  GROSS LIMITS FOR PARAMETER
C
C               POSSIBILITIES:
C
C           PP1     PP2     LAYER   Z    LSUM
C (1)        X
C (2)                               X
C (3)        X       X        X
C (4)        X       X        X           X
C
C       (1) = INDIVIDUAL PRESSURE LEVEL
C       (2) = INDIVIDUAL HEIGHT LEVEL
C       (3) = LAYER (MEAN TABLE VALUES OVER LAYER)
C       (4) = LAYER (SUM  TABLE VALUES OVER LAYER)
C
C       METHOD.
C      ---------
C
C         IF NOT LAYER, INTERPOLATE TO INDIVIDUAL LEVEL
C         IF LAYER, INTERPOLATE TO BASE AND TOP LEVELS,
C             SUM OR MEAN OVER LAYER AS REQUIRED
C
C       EXTERNALS.
C      ------------
C
C         PINTER  -  INTERPOLATE TO PRESSURE OR HEIGHT LEVEL
C
C       REFERENCE.
C      ------------
C
C         GUIDE ON THE GLOBAL DATA PROCESSING SYSTEM WMO N305 1982
C          CHAPTER 6, QUALITY CONTROL PROCEDURES
C
C       AUTHOR.
C      ---------
C
C         B. NORRIS,  ECMWF,  JANUARY 1989.
C
C       MODIFICATIONS.
C      ----------------
C
C         NONE
C
      INCLUDE 'paramq.f'
      INCLUDE 'const.f'
C
      DIMENSION UPZ(NLEV,2)
      DIMENSION TABLE(NMM,NAREA,NLEV)
      DIMENSION VLIM(NMM)
      LOGICAL LSUM,LAYER,LSTD
      DATA EPS/0.00001/
C
C    --------------------------------------------------------------------
C
C                         1.  SETUP
C                        -----------
C
  100 CONTINUE
      LSTD=.FALSE.
      IERR=0
C
C           1.1  DETERMINE WHICH AREA REPORT FALLS INTO
C
      IAREA=1
      IF(ABS(RLAT).GT.RLLIM) IAREA=2
C
      IF(LAYER) GO TO 400
C
C    ---------------------------------------------------------------------
C
C              2.  INDIVIDUAL LEVEL
C             ----------------------
C
  200 CONTINUE
C
C           2.1  HEIGHT LEVEL
C
C    (2):
C
      IF(Z.NE.RMISS)
     X         CALL PINTER (IERR,UPZ(1,2),Z,POS)
C
C           2.2  PRESSURE LEVEL
C
C    (1):
C
      IF(PP1.NE.RMISS)
     X         CALL PINTER (IERR,UPZ(1,1),PP1,POS)
C
      IF(IERR.NE.0) GO TO 500
C
C             2.3  OBTAIN GROSS LIMITS
C
      ILOW=IFIX(POS)
      IHGH=ILOW+1
      DO 235 IMM=1,NMM
      VLIM(IMM)=
     X   TABLE(IMM,IAREA,ILOW)*(FLOAT(IHGH)-POS)+
     X   TABLE(IMM,IAREA,IHGH)*(POS-FLOAT(ILOW))
  235 CONTINUE
      GO TO 500
C
C    -------------------------------------------------------------------
C
C                      4.  LAYER
C                     -----------
C
C     (3) & (4):
C
  400 CONTINUE
      IF(PP1.EQ.RMISS.OR.PP2.EQ.RMISS) THEN
           IERR=2301
           GO TO 500
      ENDIF
C
C          4.1  INTERPOLATE TO TOP AND BASE OF LAYER
C
      CALL PINTER (IERR,UPZ(1,1),PP1,POSP1)
      IF(IERR.NE.0) GO TO 500
      CALL PINTER (IERR,UPZ(1,1),PP2,POSP2)
      IF(IERR.NE.0) GO TO 500
      ILOW1=IFIX(POSP1)
      IHGH1=ILOW1+1
      ILOW2=IFIX(POSP2)
      IHGH2=ILOW2+1
      PDIFF=POSP2-POSP1
      IF(ABS(PDIFF).LT.EPS) PDIFF=SIGN(EPS,PDIFF)
C
C          4.2  CHECK WHETHER LAYER IS BOUNDED BY STANDARD LEVELS
C
      IF(ABS(FLOAT(ILOW1)-POSP1).LT.EPS.AND.
     X   ABS(FLOAT(ILOW2)-POSP2).LT.EPS) LSTD=.TRUE.
C
      DO 448 IMM=1,NMM
      SUM=0.0
      IF(LSUM) THEN
C
C          4.3   SUMMATION REQUIRED
C
           IF(LSTD.AND.ILOW2.GT.ILOW1) THEN
C
C          4.31  STANDARD LAYERS ONLY
C
                DO 431 J=ILOW1,ILOW2-1
                SUM=SUM+TABLE(IMM,IAREA,J)
  431           CONTINUE
                GO TO 435
           ENDIF
           IF(ILOW2.GT.IHGH1) THEN
C
C           4.32  ADD WHOLE STANDARD LAYERS
C
                DO 432 J=IHGH1,ILOW2-1
                SUM=SUM+TABLE(IMM,IAREA,J)
  432           CONTINUE
           ENDIF
           IF(ILOW2.LT.IHGH1) THEN
C
C           4.33  LAYER IS LESS THAN 1 STANDARD LAYER
C
                SUM=SUM+TABLE(IMM,IAREA,ILOW1)*PDIFF
           ELSE
C
C           4.34  ADD PARTIAL LAYERS AT TOP AND BOTTOM
C
                SUM=SUM+TABLE(IMM,IAREA,ILOW1)*(FLOAT(IHGH1)-POSP1)+
     X                  TABLE(IMM,IAREA,ILOW2)*(POSP2-FLOAT(ILOW2))
           ENDIF
  435      CONTINUE
           VLIM(IMM)=SUM
      ELSE
C
C           4.4  MEAN OVER LAYER REQUIRED
C
           IF(LSTD.AND.ILOW2.GT.ILOW1) THEN
C
C            4.41  STANDARD LAYERS ONLY
C
                DO 441 J=ILOW1,ILOW2-1
                SUM=SUM+(TABLE(IMM,IAREA,J)+TABLE(IMM,IAREA,J+1))/2.0
  441           CONTINUE
                GO TO 445
           ENDIF
           IF(ILOW2.GT.IHGH1) THEN
C
C            4.42  ADD WHOLE STANDARD LAYERS
C
                DO 442 J=IHGH1,ILOW2-1
                SUM=SUM+(TABLE(IMM,IAREA,J)+TABLE(IMM,IAREA,J+1))/2.0
  442           CONTINUE
           ENDIF
           PLOW=TABLE(IMM,IAREA,ILOW1)*(FLOAT(IHGH1)-POSP1)+
     X          TABLE(IMM,IAREA,IHGH1)*(POSP1-FLOAT(ILOW1))
           PHIGH=TABLE(IMM,IAREA,ILOW2)*(FLOAT(IHGH2)-POSP2)+
     X          TABLE(IMM,IAREA,IHGH2)*(POSP2-FLOAT(ILOW2))
           IF(ILOW2.LT.IHGH1) THEN
C
C           4.43  LAYER IS LESS THAN 1 STANDARD LAYER
C
                SUM=SUM+((PLOW+PHIGH)/2.0)*PDIFF
           ELSE
C
C           4.44  ADD PARTIAL LAYERS AT TOP AND BOTTOM
C
                SUM=SUM+((PLOW+TABLE(IMM,IAREA,IHGH1))/2.0)*
     X                       (FLOAT(IHGH1)-POSP1)+
     X                 ((TABLE(IMM,IAREA,ILOW2)+PHIGH)/2.0)*
     X                       (POSP2-FLOAT(ILOW2))
           ENDIF
  445      CONTINUE
           VLIM(IMM)=SUM/PDIFF
      ENDIF
  448 CONTINUE
C
C    --------------------------------------------------------------------
C
C                     5.   EXIT
C                   ------------
C
  500 CONTINUE
      RETURN
      END
      SUBROUTINE PINTER (IERR,UPZ,PP,POS)
C
C**** *PINTER*
C
C       PURPOSE.
C      ----------
C
C           INTERPOLATE POSITION OF PRESSURE OR HEIGHT
C               WITHIN TABLE OF UPPER AIR LEVELS
C
C       INTERFACE.
C      ------------
C
C         CALL PINTER (IERR,UPZ,PP,POS)
C
C        INPUT
C         UPZ    -  TABLE OF STANDARD PRESSURE OR HEIGHT LEVELS
C         PP     -  PRESSURE OR HEIGHT LEVEL
C
C        OUTPUT
C         IERR   -  ERROR CODE
C         POS    -  INTERPOLATED POSITION WITHIN TABLE
C
C       METHOD.
C      ---------
C
C           IF VALUES OF UPZ INCREASE, HEIGHT ASSUMED, LINEAR IN Z
C           IF VALUES OF UPZ DECREASE, PRESSURE ASSUMED, LINEAR IN LOGP
C
C       EXTERNALS.
C      -----------
C
C         NONE
C
C       REFERENCE.
C      ------------
C
C         NONE
C
C       AUTHOR.
C      ---------
C
C         B. NORRIS,  ECMWF,  JANUARY 1989.
C
C       MODIFICATIONS.
C      ----------------
C
C         NONE
C
      INCLUDE 'paramq.f'
      DIMENSION UPZ(NLEV)
      LOGICAL LDOWN
      DATA EPS/0.000001/
C
C    -------------------------------------------------------------------
C
C                       1.  SETUP
C                     ------------
C
  100 CONTINUE
      IERR=0
      LDOWN=.FALSE.
      IF(UPZ(1).GT.UPZ(NLEV)) LDOWN=.TRUE.
C
C    -------------------------------------------------------------------
C
C           2.  CHECK WHETHER VALUE IS OUTSIDE TABLE LIMITS
C         --------------------------------------------------
C
  200 CONTINUE
      IF(LDOWN) THEN
           IF(PP.GT.UPZ(1).OR.PP.LT.UPZ(NLEV)) THEN
                IERR=2101
                GO TO 400
           ENDIF
      ELSE
           IF(PP.LT.UPZ(1).OR.PP.GT.UPZ(NLEV)) THEN
                IERR=2101
                GO TO 400
           ENDIF
      ENDIF
C
C    -------------------------------------------------------------------------
C
C          3.   VALUE IS BETWEEN MAX AND MIN TABLE VALUES
C         ------------------------------------------------
C
  300 CONTINUE
      DO 312 JLEV=1,NLEV-1
C
C         3.1   CHECK IF VALUE  = TABLE VALUE
C
      IF(ABS(PP-UPZ(JLEV)).LT.EPS) THEN
           POS=FLOAT(JLEV)+EPS
           GO TO 400
      ENDIF
  312 CONTINUE
C
C         3.2   VALUE IS BETWEEN 2 TABLE VALUES
C
      DO 322 JLEV=1,NLEV-1
      IF(LDOWN) THEN
           IF(PP.LE.UPZ(JLEV).AND.PP.GE.UPZ(JLEV+1)) THEN
                JPOS=JLEV
                GO TO 324
           ENDIF
      ELSE
           IF(PP.GE.UPZ(JLEV).AND.PP.LE.UPZ(JLEV+1)) THEN
                JPOS=JLEV
                GO TO 324
           ENDIF
      ENDIF
  322 CONTINUE
  324 CONTINUE
C
C           3.3   INTERPOLATE
C
      IF(LDOWN) THEN
           POS=FLOAT(JPOS)+(ALOG(UPZ(JPOS))-ALOG(PP))/
     X         (ALOG(UPZ(JPOS))-ALOG(UPZ(JPOS+1))) +EPS
      ELSE
           POS=FLOAT(JPOS)+(UPZ(JPOS)-PP)/(UPZ(JPOS)-UPZ(JPOS+1))+EPS
      ENDIF
C
C    --------------------------------------------------------------------
C
C                   4.   EXIT
C                  -----------
C
  400 CONTINUE
      RETURN
      END
      SUBROUTINE PREDUC (ZRED,ZSTN,PRED,PSTN,TSTN,LPRES,
     X                  IZRED,IZSTN,IPRED,IPSTN,ITSTN)
C
C**** *PREDUC*
C
C       PURPOSE.
C      ----------
C
C         CHECK ON STATION PRESSURE REDUCTION
C
C       INTERFACE.
C      ------------
C
C         CALL PREDUC (ZRED,ZSTN,PRED,PSTN,TSTN,LPRES,
C                     IZRED,IZSTN,IPRED,IPSTN,ITSTN)
C
C        INPUT
C         ZRED   -  HEIGHT REDUCED TO STANDARD PRESSURE LEVEL
C         ZSTN   -  HEIGHT OF STATION
C         PRED   -  PRESSURE REDUCED TO STANDARD HEIGHT LEVEL
C         PSTN   -  STATION LEVEL PRESSURE
C         TSTN   -  TEMPERATURE AT STATION
C        I----   -  POSITIONS OF CONFIDENCE FLAGS OF ABOVE
C        LPRES   -  LOGICAL FOR CHECKING PRESSURE OR HEIGHT REDUCTION
C
C        OUTPUT
C         IFLAG (CONF) - CONFIDENCE FLAGS UPDATED
C
C       METHOD.
C      ---------
C
C        USE OF HYDROSTATIC EQUATION AND BY ASSUMING A LINEAR
C          TEMPERATURE VARIATION WITH RESPECT TO HEIGHT
C
C       EXTERNALS.
C      ------------
C
C         NONE
C
C       REFERENCE.
C      ------------
C
C        GUIDE ON THE GLOBAL DATA PROCESSING SYSTEM WMO N305 1982
C          CHAPTER 6, QUALITY CONTROL PROCEDURES PARA 6.3.2.1
C
C       AUTHOR.
C      ---------
C
C        B. NORRIS,  ECMWF,  JANUARY 1989.
C
C       MODIFICATIONS.
C      ----------------
C
C        NONE
C
      INCLUDE 'const.f'
      LOGICAL LPRES
      ICONF=IFAIL
C
      IF(LPRES) THEN
C
C    --------------------------------------------------------------------
C
C              1.   REDUCTION TO PRESSURE LEVEL
C            ------------------------------------
C
 100       CONTINUE
           ALPP=ALOG(PSTN/PRED)
           RRG=R/RG
           RG2=RGAMMA/2.0
           ZRED1=
     X     (ZSTN+RRG*ALPP*(TSTN+RG2*ZSTN))/
     X     (1.0+RRG*ALPP*RG2)
           IF(ABS(PRED-PSTN).LE.2000.0) THEN
                IF(ABS(ZRED-ZRED1).LE.5.0) THEN
                     ICONF=IPASS
                     GO TO 300
                ENDIF
           ELSE
                IF(ABS(ZRED-ZRED1).LE.0.0025*ABS(PRED-PSTN)) THEN
                     ICONF=IPASS
                     GO TO 300
                ENDIF
           ENDIF
      ELSE
C
C    ---------------------------------------------------------------------
C
C                 2.  REDUCTION TO HEIGHT LEVEL
C               ---------------------------------
C
  200      CONTINUE
           TM=TSTN+(RGAMMA*(ZSTN-ZRED))/2.0
           ALPHA=RG*(ZSTN-ZRED)/(R*TM)
           PRED1=PSTN*EXP(ALPHA)
           IF(ABS(ZRED-ZSTN).LE.100.0) THEN
                IF(ABS(PRED-PRED1).LE.40.0) THEN
                     ICONF=IPASS
                     GO TO 300
                ENDIF
           ELSE
                IF(ABS(PRED-PRED1).LE.0.4*ABS(ZRED-ZSTN)) THEN
                     ICONF=IPASS
                     GO TO 300
                ENDIF
           ENDIF
      ENDIF
C
C    --------------------------------------------------------------------------
C
C                   3.  UPDATE CONFIDENCE
C                  ------------------------
C
  300 CONTINUE
      CALL SETINT (ICONF,IZRED,IZSTN,IPRED,IPSTN,ITSTN,IEND)
C
C    ----------------------------------------------------------------------
C
C                    4.   EXIT
C                  ------------
C
  400 CONTINUE
      RETURN
      END
      SUBROUTINE QCAIREP (IERR)
C
C**** *QCAIREP*
C
C       PURPOSE.
C      ----------
C
C          QUALITY CONTROL FOR AIREPS
C
C       INTERFACE.
C      ------------
C
C        CALL QCAIREP (IERR)
C
C        INPUT
C         ARRAY VALUES (COMWT) - AIREP REPORTS
C
C        OUTPUT
C         ARRAY VALUES (COMWT) - AIREP REPORTS WITH CONFIDENCE FLAGS
C                                               ADDED
C         IERR   -  ERROR CODE
C
C       METHOD.
C      ---------
C
C        CHECK FLIGHT LEVEL AGAINST LIMITS, THEN CHECK TEMPERATURE
C          AND WIND SPEED FOR PARTICULAR HEIGHT;
C        CHECK CONSISTENCY OF WIND SPEED AND DIRECTION
C
C       EXTERNALS.
C      ------------
C
C        CDDFF  -  CHECK CONSISTENCY OF WIND SPEED AND DIRECTION
C        CUAT   -  GROSS LIMIT CHECK FOR UPPER AIR PARAMETERS
C        DERITA -  INITIALISE TABLES AND CONFIDENCE FLAGS
C        SEASON -  DETERMINE RELEVANT SEASON
C        SETGRS -  UPDATE CONFIDENCE FLAGS FOR GROSS LIMIT CHECK
C        UPFLAG -  ADD CONFIDENCE FLAGS TO REPORT
C        POSCHK -  COMPARE POSITION WITH PREVIOUS POSITIONS
C
C       REFERENCE.
C      ------------
C
C        GUIDE ON THE GLOBAL DATA PROCESSING SYSTEM WMO N305 1982
C          CHAPTER 6,  QUALITY CONTROL PROCEDURES
C
C       AUTHOR.
C      ---------
C
C         B. NORRIS,  ECMWF,  JANUARY 1989.
C
C       MODIFICATIONS.
C      ----------------
C
C         NONE
C
      IMPLICIT LOGICAL (L,O,G)
      INCLUDE 'parameter.f'
      INCLUDE 'paramq.f'
      INCLUDE 'paramsh.f'
      INCLUDE 'comwt.f'
      INCLUDE 'const.f'
      INCLUDE 'comkey.f'
      INCLUDE 'comkeyc.f'
      CHARACTER*35  CFNAME
      CHARACTER*9   CIDENT
      CHARACTER*9   CUNIQ(NUNIQ)
C
      LOGICAL LANSEA
      LOGICAL LSHIP
      PARAMETER (IUNIQ=5)
C
C    ---------------------------------------------------------------------------
C
C                      1.  SETUP
C                    --------------
C
  100 CONTINUE
c
      IF(IERR.NE.0) RETURN
      IOFF=0
      MISS=NINT(RMISS)
C
      LANSEA=.FALSE.
      LSHIP=.FALSE.
C
C          CFNAME = NAME OF TIME SERIES FILE
C
      CFNAME='AIREPOSITION'
C
C          1.0  CALL SIGNS BEGINNING WITH THE FOLLOWING
C                     ARE NOT UNIQUE
C
      DATA (CUNIQ(K),K=1,IUNIQ)/
     X      'XXX      ',
     X      '???      ',
     X      '///      ',
     X      'XX999    ',
     X      'XXX99    '/
C
C
C          1.1  POSITIONS OF PARAMETERS WITHIN REPORT
C
      DATA NIY/3/,NIM/4/,NID/5/,NIH/6/,NIN/7/,NRLAT/8/,NRLONG/9/
      DATA NZ/11/,NTT/12/,NDD/13/,NFF/14/
C
C
C          1.2  LOOP FOR REPORTS WITHIN ARRAY 'VALUES'
C
      DO 305 J=1,NSUB
      IERR=0
C
C          1.3  EXTRACT PARAMETER VALUES
C
      IY=NINT(VALUES(NIY,J))
      IM=NINT(VALUES(NIM,J))
      ID=NINT(VALUES(NID,J))
      IH=NINT(VALUES(NIH,J))
      IN=NINT(VALUES(NIN,J))
      RLAT =VALUES(NRLAT,J)
      RLONG=VALUES(NRLONG,J)
      Z    =VALUES(NZ,J)
      TT   =VALUES(NTT,J)
      DD   =VALUES(NDD,J)
      FF   =VALUES(NFF,J)
C
C          1.4   SET UP TABLES
C
      CALL DERITA (IERR,M)
      IF (IERR.NE.0) GO TO 400
C
C          1.5   DETERMINE SEASON
C
      CALL SEASON (RLAT,RLONG,IM)
      PP1=RMISS
      PP2=RMISS
C
C    --------------------------------------------------------------------
C
C                2.  CHECKING OF PARAMETER VALUES
C             -------------------------------------
C
  200 CONTINUE
      IF(Z.NE.RMISS) THEN
C
C            2.1  CHECK VALUE OF HEIGHT AGAINST LIMITS
C
           IF(Z.LT.10.0.OR.Z.GT.25000.0) THEN
               CALL SETGRS (NZ,2)
C
C            2.2  IF Z IS BAD, FLAG TT,DD AND FF ALSO
C
               CALL SETGRS (NTT,3)
               CALL SETGRS (NDD,3)
               CALL SETGRS (NFF,3)
           ENDIF
C
           IF(TT.NE.RMISS) THEN
C
C             2.3  CHECK TEMPERATURE DEPENDING ON HEIGHT
C
                CALL CUAT (IERR,TT,'TT',NTT,RLAT,RLONG,PP1,PP2,Z,
     X                                           IMISS,IMISS,NZ)
                IF(IERR.NE.0) WRITE (*,1698) IERR,Z,TT
           ENDIF
C
           IF(FF.NE.RMISS) THEN
C
C              2.4  CHECK WIND SPEED DEPENDING ON HEIGHT
C
                CALL CUAT (IERR,FF,'FF',NFF,RLAT,RLONG,PP1,PP2,Z,
     X                                           IMISS,IMISS,NZ)
                IF(IERR.NE.0) WRITE (*,1699) IERR,Z,FF
           ENDIF
      ENDIF
1698  format(1h,'AIREP CUAT ERROR:',I5,' Z:',F10.2,' TT:',F10.2)
1699  format(1h,'AIREP CUAT ERROR:',I5,' Z:',F10.2,' FF:',F10.2)
C
C             2.5  CHECK CONSISTENCY OF WIND SPEED AND DIRECTION
C
      CALL CDDFF (DD,FF,NDD,NFF)
C
C
C
C             2.6   POSITION CHECK (TIME CONTINUITY)
C
C
      CALL POSCHK (CFNAME,CIDENT,IUNIQ,CUNIQ,NTYPE,NSBTYPE,
     X    IY,IM,ID,IH,IN,RLAT,RLONG,LANSEA,LSHIP,IFL,IERR)
      IF(IERR.NE.0) THEN
          WRITE (*,'(1H ,'' POSPCHK ERROR:'',I5)') IERR
      ELSE
          CALL SETABS (IFL,NRLAT,NRLONG,IEND)
      ENDIF
C
C    --------------------------------------------------------------------
C
C                 3. ADD FLAGS TO BUFR RECORD
C                -------------------------------
C
  300 CONTINUE
      CALL UPFLAG (IERR,J,VALUES,JP22,JP1,M,IOFF)
      IF(IERR.NE.0) GO TO 400
C
  305 CONTINUE
C
C    --------------------------------------------------------------------
C
C                   4.  EXIT
C                  -----------
C
  400 CONTINUE
      RETURN
      END
      SUBROUTINE QCBATHY (IERR)
C
C**** *QCBATHY*
C
C       PURPOSE.
C      ----------
C
C          QUALITY CONTROL FOR BATHYS
C
C       INTERFACE.
C      ------------
C
C        CALL QCBATHY (IERR)
C
C        INPUT
C         ARRAY VALUES (COMWT) - BATHY REPORTS
C         L21  (COMWT)  -  LOGICAL FOR SURFACE REPORT
C         L22  (COMWT)  -  LOGICAL FOR SUB-SURFACE REPORT
C
C        OUTPUT
C         ARRAY VALUES (COMWT) - BATHY REPORTS WITH CONFIDENCE FLAGS
C                                               ADDED
C         IERR   -  ERROR CODE
C
C       METHOD.
C      ---------
C
C        CHECK PARAMETER VALUES AGAINST GROSS LIMITS;
C        CHECK CONSISTENCY OF WIND SPEED AND DIRECTION;
C
C       EXTERNALS.
C      ------------
C
C        CDDFF  -  CHECK CONSISTENCY OF WIND SPEED AND DIRECTION
C        CSLT   -  GROSS LIMIT CHECK FOR SURFACE PARAMETERS
C        DERITA -  INITIALISE TABLES AND CONFIDENCE FLAGS
C        SEASON -  DETERMINE RELEVANT SEASON
C        UPFLAG -  ADD CONFIDENCE FLAGS TO REPORT
C
C       REFERENCE.
C      ------------
C
C        GUIDE ON THE GLOBAL DATA PROCESSING SYSTEM WMO N305 1982
C          CHAPTER 6,  QUALITY CONTROL PROCEDURES
C
C       AUTHOR.
C      ---------
C
C         B. NORRIS,  ECMWF,  JULY 1989.
C
C       MODIFICATIONS.
C      ----------------
C
C         NONE
C
      IMPLICIT LOGICAL (L,O,G)
      INCLUDE 'parameter.f'
      INCLUDE 'paramq.f'
      INCLUDE 'comwt.f'
      INCLUDE 'const.f'
C
C    ---------------------------------------------------------------------------
C
C                      1.  SETUP
C                    --------------
C
  100 CONTINUE
      IF(IERR.NE.0) RETURN
      MISS=NINT(RMISS)
C
C          1.1  POSITIONS OF PARAMETERS WITHIN HEADER
C
      IF(L21) THEN
           NMONTH=6
           NRLAT=10
           NRLONG=11
      ENDIF
      IF(L22) THEN
           NMONTH=3
           NRLAT=7
           NRLONG=8
      ENDIF
C
C          1.2  POSITIONS OF METEOROLOGICAL PARAMETERS
C
      IF(L21) THEN
           NDD=16
           NFF=17
           NTT=18
           NTW=32
      ENDIF
      IOFF=0
C
C          1.3  LOOP FOR REPORTS WITHIN ARRAY 'VALUES'
C
      DO 405 J=1,NSUB
      IERR=0
C
C          1.4  EXTRACT HEADER VALUES
C
      MONTH=NINT(VALUES(NMONTH,J))
      RLAT =VALUES(NRLAT,J)
      RLONG=VALUES(NRLONG,J)
C
C          1.5   SET UP TABLES
C
      CALL DERITA (IERR,M)
      IF(IERR.NE.0) GO TO 500
C
C          1.6   DETERMINE SEASON
C
      CALL SEASON (RLAT,RLONG,MONTH)
C
C          1.7   EXTRACT PARAMETER VALUES
C
      IF(.NOT.L21) GO TO 400
      DD=VALUES(NDD,J)
      FF=VALUES(NFF,J)
      TT=VALUES(NTT,J)
      TW=VALUES(NTW,J)
C
C    ----------------------------------------------------------------------
C
C                 2.  GROSS LIMIT CHECKS
C                -------------------------
C
  200 CONTINUE
C
C             2.1   SURFACE WIND SPEED
C
      IF(FF.NE.RMISS) THEN
           CALL CSLT (IERR,FF,'FF',NFF,RLAT,RLONG)
           IF(IERR.NE.0) 
     x WRITE(*,'(1H ,''BATHY CSLT ERROR:'',I5,'' FF:'',F10.2)') IERR,FF
      ENDIF
C
C             2.2   SURFACE AIR TEMPERATURE
C
      IF(TT.NE.RMISS) THEN
           CALL CSLT (IERR,TT,'TT',NTT,RLAT,RLONG)
           IF(IERR.NE.0) 
     x WRITE(*,'(1H,''BATHY CSLT ERROR:'',I5,'' TT:'',F10.2)') IERR,TT
      ENDIF
C
C             2.3   SEA SURFACE TEMPERATURE
C
      IF(TW.NE.RMISS) THEN
           CALL CSLT (IERR,TW,'TW',NTW,RLAT,RLONG)
           IF(IERR.NE.0) 
     x WRITE(*,'(1H,''BATHY CSLT ERROR:'',I5,'' TW:'',F10.2)') IERR,TW
      ENDIF
C
C   ----------------------------------------------------------------------
C
C                  3.  INTERNAL CONSISTENCY CHECKS
C                ------------------------------------
C
  300 CONTINUE
C
C             3.1   CONSISTENCY OF WIND SPEED AND DIRECTION
C
      CALL CDDFF (DD,FF,NDD,NFF)
C
C    --------------------------------------------------------------------
C
C                 4. ADD FLAGS TO BUFR RECORD
C                -------------------------------
C
  400 CONTINUE
      CALL UPFLAG (IERR,J,VALUES,JP22,JP1,M,IOFF)
      IF(IERR.NE.0) GO TO 500
C
  405 CONTINUE
C
C    --------------------------------------------------------------------
C
C                   5.  EXIT
C                  -----------
C
  500 CONTINUE
      RETURN
      END
      SUBROUTINE QCDRIBU (IERR)
C
C**** *QCDRIBU*
C
C       PURPOSE.
C      ----------
C
C          QUALITY CONTROL FOR DRIBUS
C
C       INTERFACE.
C      ------------
C
C        CALL QCDRIBU (IERR)
C
C        INPUT
C         ARRAY VALUES (COMWT) - DRIBU REPORTS
C         L11  (COMWT)  -  LOGICAL FOR SURFACE REPORT
C         L12  (COMWT)  -  LOGICAL FOR SUB-SURFACE REPORT
C
C        OUTPUT
C         ARRAY VALUES (COMWT) - DRIBU REPORTS WITH CONFIDENCE FLAGS
C                                               ADDED
C         IERR   -  ERROR CODE
C
C       METHOD.
C      ---------
C
C        CHECK PARAMETER VALUES AGAINST GROSS LIMITS;
C        CHECK CONSISTENCY OF WIND SPEED AND DIRECTION;
C        CHECK CONSISTENCY OF PRESSURE TENDENCY
C
C       EXTERNALS.
C      ------------
C
C        CDDFF  -  CHECK CONSISTENCY OF WIND SPEED AND DIRECTION
C        CSLT   -  GROSS LIMIT CHECK FOR SURFACE PARAMETERS
C        DERITA -  INITIALISE TABLES AND CONFIDENCE FLAGS
C        SETINT -  UPDATE CONFIDENCE FLAGS FOR INTERNAL CONSISTENCY
C        SEASON -  DETERMINE RELEVANT SEASON
C        UPFLAG -  ADD CONFIDENCE FLAGS TO REPORT
C        POSCHK -  COMPARE POSITION WITH PREVIOUS POSITIONS
C
C       REFERENCE.
C      ------------
C
C        GUIDE ON THE GLOBAL DATA PROCESSING SYSTEM WMO N305 1982
C          CHAPTER 6,  QUALITY CONTROL PROCEDURES
C
C       AUTHOR.
C      ---------
C
C         B. NORRIS,  ECMWF,  JULY 1989.
C
C       MODIFICATIONS.
C      ----------------
C
C         NONE
C
      IMPLICIT LOGICAL (L,O,G)
      INCLUDE 'parameter.f'
      INCLUDE 'paramq.f'
      INCLUDE 'paramsh.f'
      INCLUDE 'comwt.f'
      INCLUDE 'const.f'
      INCLUDE 'comkey.f'
      INCLUDE 'comkeyc.f'
      CHARACTER*35  CFNAME
      CHARACTER*9   CIDENT
      CHARACTER*9   CUNIQ(NUNIQ)
C
      LOGICAL LANSEA
      LOGICAL LSHIP
      PARAMETER (IUNIQ=12)
C    ---------------------------------------------------------------------------
C
C                      1.  SETUP
C                    --------------
C
  100 CONTINUE
      IF(IERR.NE.0) RETURN
      MISS=NINT(RMISS)
C
      LANSEA=.TRUE.
      LSHIP=.TRUE.
C
C          CFNAME = NAME OF TIME SERIES FILE
      CFNAME='DRIBUPOSITION'
C
C
C
C          1.01  CALL SIGNS BEGINNING WITH THE FOLLOWING
C                     ARE NOT UNIQUE
C
      DATA (CUNIQ(K),K=1,IUNIQ)/
     X      'BOAT     ',
     X      'BUOY     ',
     X      'PLAT     ',
     X      'RIGG     ',
     X      'SHIP     ',
     X      'XXX      ',
     X      '???      ',
     X      '///      ',
     X      'BBXX     ',
     X      'ZSWAV    ',
     X      'CG2      ',
     X      'CG3      '/
C
C          1.1  POSITIONS OF PARAMETERS WITHIN HEADER
C
      IF(L11) THEN
C               SURFACE REPORT
           NIY=5
           NIM=6
           NID=7
           NIH=8
           NIN=9
           NRLAT=10
           NRLONG=11
      ENDIF
      IF(L12) THEN
C               SUB-SURFACE REPORT
           NIY=3
           NIM=4
           NID=5
           NIH=6
           NIN=7
           NRLAT=8
           NRLONG=9
      ENDIF
C
C
C          1.2  POSITIONS OF METEOROLOGICAL PARAMETERS
C
      IF(L11) THEN
           NPMSL=12
           NPPP=14
           NA=15
           NDD=16
           NFF=17
           NTT=18
           NTW=32
      ENDIF
      IOFF=0
C
C          1.3  LOOP FOR REPORTS WITHIN ARRAY 'VALUES'
C
      DO 405 J=1,NSUB
      IERR=0
C
C          1.4  EXTRACT HEADER VALUES
C
      IY=NINT(VALUES(NIY,J))
      IM=NINT(VALUES(NIM,J))
      ID=NINT(VALUES(NID,J))
      IH=NINT(VALUES(NIH,J))
      IN=NINT(VALUES(NIN,J))
      RLAT =VALUES(NRLAT,J)
      RLONG=VALUES(NRLONG,J)
C
C          1.5   SET UP TABLES
C
      CALL DERITA (IERR,M)
      IF(IERR.NE.0) GO TO 500
C
C          1.6   DETERMINE SEASON
C
      CALL SEASON (RLAT,RLONG,IM)
C
C          1.7   EXTRACT PARAMETER VALUES
C
      IF(.NOT.L11) GO TO 400
      PMSL=VALUES(NPMSL,J)
      PPP=VALUES(NPPP,J)
      IA=NINT(VALUES(NA,J))
      DD=VALUES(NDD,J)
      FF=VALUES(NFF,J)
      TT=VALUES(NTT,J)
      TW=VALUES(NTW,J)
C
C    ----------------------------------------------------------------------
C
C                 2.  GROSS LIMIT CHECKS
C                -------------------------
C
  200 CONTINUE
C
C             2.1   PRESSURE CHARACTERISTIC
C
      IF(IA.NE.MISS.AND.IA.NE.15) THEN
           IF(IA.LT.0.OR.IA.GT.8) CALL SETGRS (NA,1)
      ENDIF
C
C             2.2   SURFACE WIND SPEED
C
      IF(FF.NE.RMISS) THEN
           CALL CSLT (IERR,FF,'FF',NFF,RLAT,RLONG)
           IF(IERR.NE.0) 
     x WRITE (*,'(1H ,''DRIBU CSLT ERROR:'',I5,'' FF:'',F10.2)') IERR,FF
      ENDIF
C
C             2.3   MEAN SEA LEVEL PRESSURE
C
      IF(PMSL.NE.RMISS) THEN
           CALL CSLT (IERR,PMSL,'PPP',NPMSL,RLAT,RLONG)
           IF(IERR.NE.0) 
     x WRITE(*,'(1H,''DRIBU CSLT ERROR:'',I5,''PMSL:'',F10.2)')IERR,PMSL
      ENDIF
C
C             2.4   3 HOUR PRESSURE TENDENCY
C
      IF(IA.GE.0.AND.IA.LE.8.AND.PPP.NE.RMISS) THEN
           CALL CSLT (IERR,PPP,'PP',NPPP,RLAT,RLONG)
           IF(IERR.NE.0)
     x WRITE(*,'(1H,''DRIBU CSLT ERROR:'',I5,'' PPP:'',F10.2)') IERR,PPP
      ENDIF
C
C             2.5   SURFACE AIR TEMPERATURE
C
      IF(TT.NE.RMISS) THEN
           CALL CSLT (IERR,TT,'TT',NTT,RLAT,RLONG)
           IF(IERR.NE.0) 
     x WRITE(*,'(1H,''DRIBU CSLT ERROR:'',I5,'' TT:'',F10.2)') IERR,TT
      ENDIF
C
C             2.6   SEA SURFACE TEMPERATURE
C
      IF(TW.NE.RMISS) THEN
           CALL CSLT (IERR,TW,'TW',NTW,RLAT,RLONG)
           IF(IERR.NE.0) 
     x WRITE (*,'(1H,''DRIBU CSLT ERROR:'',I5,'' TW:'',F10.2)') IERR,TW
      ENDIF
C
C             2.7   POSITION CHECK (TIME CONTINUITY)
C
C
      CALL POSCHK (CFNAME,CIDENT,IUNIQ,CUNIQ,NTYPE,NSBTYPE,
     X    IY,IM,ID,IH,IN,RLAT,RLONG,LANSEA,LSHIP,IFL,IERR)
      IF(IERR.NE.0) THEN
          WRITE (*,'(1H ,'' POSCHK ERROR:'',I5)') IERR
      ELSE
          CALL SETABS (IFL,NRLAT,NRLONG,IEND)
      ENDIF
C
C   ----------------------------------------------------------------------
C
C                  3.  INTERNAL CONSISTENCY CHECKS
C                ------------------------------------
C
  300 CONTINUE
C
C             3.1   CONSISTENCY OF WIND SPEED AND DIRECTION
C
      CALL CDDFF (DD,FF,NDD,NFF)
C
C             3.2   CONSISTENCY OF PRESSURE TENDENCY
C
      IF(IA.NE.MISS.AND.IA.NE.15.AND.PPP.NE.RMISS) THEN
           ICONF=IPASS
           IF(PPP.EQ.0.0.AND.IA.NE.0.AND.IA.NE.4.AND.IA.NE.5)
     X     ICONF=IFAIL
           CALL SETINT (ICONF,NA,NPPP,IEND)
           ICONF=IPASS
           IF(IA.EQ.4.AND.PPP.NE.0.0)
     X     ICONF=IFAIL
           CALL SETINT (ICONF,NA,NPPP,IEND)
      ENDIF
C
C    --------------------------------------------------------------------
C
C                 4. ADD FLAGS TO BUFR RECORD
C                -------------------------------
C
  400 CONTINUE
      CALL UPFLAG (IERR,J,VALUES,JP22,JP1,M,IOFF)
      IF(IERR.NE.0) GO TO 500
C
  405 CONTINUE
C
C    --------------------------------------------------------------------
C
C                   5.  EXIT
C                  -----------
C
  500 CONTINUE
      RETURN
      END
      SUBROUTINE QCPILOT (KODE,IERR)
C
C**** *QCPILOT*
C
C       PURPOSE.
C      ----------
C
C          QUALITY CONTROL FOR PILOTS
C
C       INTERFACE.
C      ------------
C
C        CALL QCPILOT (KODE,IERR)
C
C        INPUT
C         ARRAY VALUES (COMWT) - PILOT REPORTS
C         KODE   -  INDICATOR FOR TYPE OF PILOT
C
C        OUTPUT
C         ARRAY VALUES (COMWT) - PILOT REPORTS WITH CONFIDENCE FLAGS
C                                               ADDED
C         IERR   -  ERROR CODE
C
C       METHOD.
C      ---------
C
C        CHECK SURFACE LEVEL AGAINST LIMITS;
C        CHECK UPPER AIR LEVELS AGAINST LIMITS;
C        CHECK CONSISTENCY OF WIND SPEED AND DIRECTION;
C        CHECK CONSECUTIVE STANDARD LEVELS FOR EXCESSIVE WIND SHEAR
C
C       EXTERNALS.
C      ------------
C
C        CDDFF  -  CHECK CONSISTENCY OF WIND SPEED AND DIRECTION
C        CUAT   -  GROSS LIMIT CHECK FOR UPPER AIR PARAMETERS
C        CSLT   -  GROSS LIMIT CHECK FOR SURFACE PARAMETERS
C        DPLIM  -  OBTAIN LIMITS FOR PRESSURE GIVEN HEIGHT
C        DERITA -  INITIALISE TABLES AND CONFIDENCE FLAGS
C        SEASON -  DETERMINE RELEVANT SEASON
C        SETSF  -  UPDATE CONFIDENCE FLAGS FOR GROSS LIMIT CHECK
C        SETGRS -  SET CONFIDENCE FLAGS FOR GROSS LIMIT CHECK
C        WSHEAR -  CHECK WIND SHEAR BETWEEN STANDARD LEVELS
C        UPFLAG -  ADD CONFIDENCE FLAGS TO REPORT
C
C       REFERENCE.
C      ------------
C
C        GUIDE ON THE GLOBAL DATA PROCESSING SYSTEM WMO N305 1982
C          CHAPTER 6,  QUALITY CONTROL PROCEDURES
C
C       AUTHOR.
C      ---------
C
C         B. NORRIS,  ECMWF,  FEBRUARY 1989.
C
C       MODIFICATIONS.
C      ----------------
C
C         NONE
C
      IMPLICIT LOGICAL (L,O,G)
      INCLUDE 'parameter.f'
      INCLUDE 'paramq.f'
      INCLUDE 'comwt.f'
      INCLUDE 'const.f'
      DIMENSION PLIM(NMM)
      DIMENSION STDPP(NLEV),STDDD(NLEV),STDFF(NLEV)
      DIMENSION ISTDDD(NLEV),ISTDFF(NLEV)
      LOGICAL LAND
C
C    ---------------------------------------------------------------------------
C
C                      1.  SETUP
C                    --------------
C
  100 CONTINUE
C
C          1.1  POSITIONS OF PARAMETERS WITHIN LEVEL
C
      DATA NPP/1/,NVS/2/,NZ/3/,NDD/4/,NFF/5/
C
C          1.2  POSITIONS OF PARAMETERS WITHIN HEADER
C
      IF(IERR.NE.0) RETURN
      LAND=.FALSE.
      IF(KODE.EQ.32) THEN
           NMONTH=6
           NRLAT=10
           NRLONG=11
           NALT=12
           LAND=.TRUE.
      ELSEIF(KODE.EQ.33) THEN
           NMONTH=7
           NRLAT=11
           NRLONG=12
      ELSE
           WRITE (*,'(1H ,''PILOT TYPE NOT RECOGNIZED:'',I5)') KODE
           GO TO 400
      ENDIF
      NILEV=13
      ILVSIZ=5
      IOFF=0
      NSTD=0
C
C          1.3  LOOP FOR REPORTS WITHIN ARRAY 'VALUES'
C
      DO 305 J=1,NSUB
      IERR=0
C
C          1.4  EXTRACT HEADER VALUES
C
      NUMLEV=NINT(VALUES(NILEV,J))
      ZSTN=0.0
      IF(LAND) ZSTN=VALUES(NALT,J)
      MONTH=NINT(VALUES(NMONTH,J))
      RLAT =VALUES(NRLAT,J)
      RLONG=VALUES(NRLONG,J)
C
C          1.5   SET UP TABLES
C
      CALL DERITA (IERR,M)
      IF(IERR.NE.0) GO TO 400
C
C          1.6   DETERMINE SEASON
C
      CALL SEASON (RLAT,RLONG,MONTH)
      Z2=RMISS
      PP2=RMISS
C
C          1.7   LOOP FOR LEVELS
C
      DO 245 ILEV=1,NUMLEV
      IBASE=(ILEV-1)*ILVSIZ+NILEV
      NNPP=IBASE+NPP
      NNZ=IBASE+NZ
      NNDD=IBASE+NDD
      NNFF=IBASE+NFF
      NNVS=IBASE+NVS
C
C          1.8  EXTRACT PARAMETER VALUES
C
      PP=VALUES(NNPP,J)
      Z =VALUES(NNZ ,J)
      IF(Z.NE.999999.) Z=Z/9.8        ! CHANGED  10/11/89.
      DD=VALUES(NNDD,J)
      FF=VALUES(NNFF,J)
      IVS=NINT(VALUES(NNVS,J))
C
C    --------------------------------------------------------------------
C
C                2.  CHECKING OF PARAMETER VALUES
C             -------------------------------------
C
  200 CONTINUE
C
C            2.1  EXTRACT STANDARD LEVELS
C
      IF(PP.NE.RMISS.AND.DD.NE.RMISS.AND.FF.NE.RMISS.AND.
     X  ((IVS.AND.32).EQ.32).AND.NSTD.LT.NLEV) THEN
           NSTD=NSTD+1
           STDPP(NSTD)=PP
           STDDD(NSTD)=DD
           STDFF(NSTD)=FF
           ISTDDD(NSTD)=NNDD
           ISTDFF(NSTD)=NNFF
      ENDIF
C
C            2.2  CHECK SURFACE LEVEL
C
      IF((IVS.AND.64).EQ.64) THEN
C
C          2.21  SURFACE PRESSURE
C
           IF(PP.NE.RMISS.AND.ZSTN.NE.RMISS) THEN
                CALL DPLIM (ZSTN,RLAT,RLONG,PLIM)
                CALL SETSF (PP,NNPP,PLIM)
           ENDIF
C
C          2.22  SURFACE WIND SPEED
C
           IF(FF.NE.RMISS) THEN
                CALL CSLT (IERR,FF,'FF',NNFF,RLAT,RLONG)
                IF(IERR.NE.0) 
     x WRITE (*,'(1H,''PILOT CSLT ERROR:'',I5,'' FF:'',F10.2)') IERR,FF
           ENDIF
           GO TO 235
      ENDIF
C
C         2.3  UPPER AIR LEVEL
C
C              2.31  GROSS LIMIT FOR PRESSURE
C
      IF(PP.NE.RMISS) THEN
           IF(PP.LT.0.0.OR.PP.GT.108000.0) CALL SETGRS (NNPP,1)
      ENDIF
C
C              2.32  CHECK HEIGHT DEPENDING ON PRESSURE
C
      IF(PP.NE.RMISS.AND.Z.NE.RMISS) THEN
                CALL CUAT (IERR,Z,'Z',NNZ,RLAT,RLONG,PP,PP2,Z2,
     X                                           NNPP,IMISS,IMISS)
                IF(IERR.NE.0)  WRITE (*,2408) IERR,PP,Z
      ENDIF
2408  format (1h,'PILOT CUAT ERROR:',I5,' PP:',F10.2,' Z:',F10.2)
C
C              2.33  CHECK WIND SPEED DEPENDING ON PRESSURE OR HEIGHT
C
      IF((PP.NE.RMISS.OR.Z.NE.RMISS).AND.FF.NE.RMISS) THEN
           IF(Z.NE.RMISS) THEN
                PAR1=RMISS
                NPAR1=IMISS
                PAR3=Z
                NPAR3=NNZ
           ELSE
                PAR1=PP
                NPAR1=NNPP
                PAR3=RMISS
                NPAR3=IMISS
           ENDIF
                CALL CUAT (IERR,FF,'FF',NNFF,RLAT,RLONG,PAR1,PP2,PAR3,
     X                                          NPAR1,IMISS,NPAR3)
                IF(IERR.NE.0) WRITE (*,2429)IERR,Z,PP,FF
      ENDIF
  235 CONTINUE
2429  format(1h,'PILOT CUAT ERROR:',I5,'Z:',F10.2,'PP:',F10.2,
     x      'FF:',F10.2)
C
C             2.4  CHECK CONSISTENCY OF WIND SPEED AND DIRECTION
C
      CALL CDDFF (DD,FF,NNDD,NNFF)
  245 CONTINUE
C
C             2.5  CHECK WIND SHEAR BETWEEN STANDARD LEVELS
C
      CALL WSHEAR (IERR,NSTD,STDPP,STDDD,STDFF,ISTDDD,ISTDFF)
      IF(IERR.NE.0) WRITE (*,'(1H ,''PILOT WSHEAR ERROR:'',I5)') IERR
C
C    --------------------------------------------------------------------
C
C                 3. ADD FLAGS TO BUFR RECORD
C                -------------------------------
C
  300 CONTINUE
      CALL UPFLAG (IERR,J,VALUES,JP22,JP1,M,IOFF)
      IF(IERR.NE.0) GO TO 400
C
  305 CONTINUE
C
C    --------------------------------------------------------------------
C
C                   4.  EXIT
C                  -----------
C
  400 CONTINUE
      RETURN
      END
      SUBROUTINE QCSATOB (KSEC,IERR)
C
C**** *QCSATOB*
C
C       PURPOSE.
C      ----------
C
C          QUALITY CONTROL FOR SATOBS
C
C       INTERFACE.
C      ------------
C
C        CALL QCSATOB (KSEC,IERR)
C
C        INPUT
C         ARRAY VALUES (COMWT) - SATOB REPORTS
C         KSEC   -  SECTION OF SATOB
C
C        OUTPUT
C         ARRAY VALUES (COMWT) - SATOB REPORTS WITH CONFIDENCE FLAGS
C                                               ADDED
C         IERR   -  ERROR CODE
C
C       METHOD.
C      ---------
C
C        CHECK SURFACE LEVEL AGAINST LIMITS;
C        CHECK UPPER AIR LEVELS AGAINST LIMITS;
C        CHECK CONSISTENCY OF WIND SPEED AND DIRECTION.
C
C       EXTERNALS.
C      ------------
C
C        CDDFF  -  CHECK CONSISTENCY OF WIND SPEED AND DIRECTION
C        CUAT   -  GROSS LIMIT CHECK FOR UPPER AIR PARAMETERS
C        CSLT   -  GROSS LIMIT CHECK FOR SURFACE PARAMETERS
C        DERITA -  INITIALISE TABLES AND CONFIDENCE FLAGS
C        SEASON -  DETERMINE RELEVANT SEASON
C        SETGRS -  UPDATE CONFIDENCE FLAGS FOR GROSS LIMIT CHECK
C        UPFLAG -  ADD CONFIDENCE FLAGS TO REPORT
C
C       REFERENCE.
C      ------------
C
C        GUIDE ON THE GLOBAL DATA PROCESSING SYSTEM WMO N305 1982
C          CHAPTER 6,  QUALITY CONTROL PROCEDURES
C
C       AUTHOR.
C      ---------
C
C         B. NORRIS,  ECMWF,  FEBRUARY 1989.
C
C       MODIFICATIONS.
C      ----------------
C
C         NONE
C
      IMPLICIT LOGICAL (L,O,G)
      INCLUDE 'parameter.f'
      INCLUDE 'paramq.f'
      INCLUDE 'comwt.f'
      INCLUDE 'const.f'
C
C    ---------------------------------------------------------------------------
C
C                      1.  SETUP
C                    --------------
C
  100 CONTINUE
      IF(IERR.NE.0) RETURN
C
C          1.1  POSITIONS OF PARAMETERS WITHIN HEADER
C
      DATA NMONTH/4/,NRLAT/9/,NRLONG/10/
C
C          1.2  POSITIONS OF PARAMETERS WITHIN LEVEL
C
      NPP=IMISS
      NTT=IMISS
      NDD=IMISS
      NFF=IMISS
      NTS=IMISS
      NTC=IMISS
      IF(KSEC.EQ.2) THEN
           NPP=12
           NTT=13
           NDD=14
           NFF=15
      ELSEIF(KSEC.EQ.3) THEN
           NPP=12
           NDD=13
           NFF=14
      ELSEIF(KSEC.EQ.4) THEN
           NTS=12
      ELSEIF(KSEC.EQ.5) THEN
           NPP=12
           NTC=13
           NTT=14
      ELSE
           WRITE (*,'(1H ,''SATOB TYPE NOT RECOGNIZED:'',I5)') KSEC
           GO TO 400
      ENDIF
      IOFF=0
C
C          1.3  LOOP FOR REPORTS WITHIN ARRAY 'VALUES'
C
      DO 305 J=1,NSUB
      IERR=0
C
C          1.4  EXTRACT HEADER VALUES
C
      MONTH=NINT(VALUES(NMONTH,J))
      RLAT =VALUES(NRLAT,J)
      RLONG=VALUES(NRLONG,J)
C
C          1.5   SET UP TABLES
C
      CALL DERITA (IERR,M)
      IF(IERR.NE.0) GO TO 400
C
C          1.6   DETERMINE SEASON
C
      CALL SEASON (RLAT,RLONG,MONTH)
      PP2=RMISS
      Z2=RMISS
C
C          1.7  EXTRACT PARAMETER VALUES
C
      IF(NPP.NE.IMISS) PP=VALUES(NPP,J)
      IF(NTT.NE.IMISS) TT=VALUES(NTT,J)
      IF(NDD.NE.IMISS) DD=VALUES(NDD,J)
      IF(NFF.NE.IMISS) FF=VALUES(NFF,J)
      IF(NTS.NE.IMISS) TS=VALUES(NTS,J)
      IF(NTC.NE.IMISS) TC=VALUES(NTC,J)
C
C    --------------------------------------------------------------------
C
C                2.  CHECKING OF PARAMETER VALUES
C             -------------------------------------
C
  200 CONTINUE
C
C          2.1  PRESSURE
C
      IF(NPP.NE.IMISS.AND.PP.NE.RMISS) THEN
           IF(PP.LT.0.0.OR.PP.GT.108000.0) CALL SETGRS (NPP,1)
      ENDIF
C
C          2.2  CLOUD AMOUNT
C
      IF(NTC.NE.IMISS.AND.TC.NE.RMISS) THEN
           IF(TC.LT.0.0.OR.TC.GT.100.0) CALL SETGRS (NTC,1)
      ENDIF
C
C          2.3  SURFACE TEMPERATURE
C
      IF(NTS.NE.IMISS.AND.TS.NE.RMISS) THEN
           CALL CSLT (IERR,TS,'TT',NTS,RLAT,RLONG)
           IF(IERR.NE.0) 
     x WRITE (*,'(1H,''SATOB CSLT ERROR:'',I5,'' TS:'',F10.2)') IERR,TS
      ENDIF
C
C          2.4  CHECK TEMPERATURE DEPENDING ON PRESSURE
C
      IF(NPP.NE.IMISS.AND.PP.NE.RMISS) THEN
           IF(NTT.NE.IMISS.AND.TT.NE.RMISS) THEN
                CALL CUAT (IERR,TT,'TT',NTT,RLAT,RLONG,PP,PP2,Z2,
     X                                           NPP,IMISS,IMISS)
                IF(IERR.NE.0) WRITE (*,2641) IERR,PP,TT
           ENDIF
C
C          2.5  CHECK WIND SPEED DEPENDING ON PRESSURE
C
           IF(NFF.NE.IMISS.AND.FF.NE.RMISS) THEN
                CALL CUAT (IERR,FF,'FF',NFF,RLAT,RLONG,PP,PP2,Z2,
     X                                           NPP,IMISS,IMISS)
                IF(IERR.NE.0) WRITE (*,2640) IERR,PP,FF
           ENDIF
      ENDIF
2640   format(1h,'SATOB CUAT ERROR:',I5, ' PP:',F10.2,' FF:',F10.2)
2641   format(1h,'SATOB CUAT ERROR:',I5, ' PP:',F10.2,' TT:',F10.2)
C
C          2.6  CHECK CONSISTENCY OF WIND SPEED AND DIRECTION
C
      IF (NDD.NE.IMISS.AND.NFF.NE.IMISS) CALL CDDFF (DD,FF,NDD,NFF)
C
C    --------------------------------------------------------------------
C
C                 3. ADD FLAGS TO BUFR RECORD
C                -------------------------------
C
  300 CONTINUE
      CALL UPFLAG (IERR,J,VALUES,JP22,JP1,M,IOFF)
      IF(IERR.NE.0) GO TO 400
C
  305 CONTINUE
C
C    --------------------------------------------------------------------
C
C                   4.  EXIT
C                  -----------
C
  400 CONTINUE
      RETURN
      END
      SUBROUTINE QCSYNOP (KSEC,KODE,KPRES,IERR)
C
C**** *QCSYNOP*
C
C       PURPOSE.
C      ----------
C
C          QUALITY CONTROL FOR SYNOPS AND SHIPS
C
C       INTERFACE.
C      ------------
C
C        CALL QCSYNOP (KSEC,KODE,KPRES,IERR)
C
C        INPUT
C         ARRAY VALUES (COMWT) - SYNOP REPORTS
C         KSEC   -  SECTION OF SYNOP
C         KODE   -  INDICATOR FOR SYNOP OR SHIP
C         KPRES  -  INDICATOR FOR HIGH OR LOW LEVEL REPORT
C
C        OUTPUT
C         ARRAY VALUES (COMWT) - SYNOP REPORTS WITH CONFIDENCE FLAGS
C                                               ADDED
C         IERR   -  ERROR CODE
C
C       METHOD.
C      ---------
C
C        COMPARE POSITION WITH PREVIOUS POSITION IF SHIP;
C        CHECK PARAMETER VALUES AGAINST GROSS LIMITS;
C        CHECK REDUCTION OF STATION LEVEL PRESSURE TO STANDARD LEVEL;
C        CHECK CONSISTENCY OF WIND SPEED AND DIRECTION;
C        CHECK INTERNAL CONSISTENCY OF COMMON PARAMETERS.
C
C       EXTERNALS.
C      ------------
C
C        CDDFF  -  CHECK CONSISTENCY OF WIND SPEED AND DIRECTION
C        CSLT   -  GROSS LIMIT CHECK FOR SURFACE PARAMETERS
C        CUAT   -  GROSS LIMIT CHECK FOR UPPER AIR PARAMETERS
C        DERITA -  INITIALISE TABLES AND CONFIDENCE FLAGS
C        DPLIM  -  OBTAIN LIMITS FOR PRESSURE GIVEN HEIGHT
C        PREDUC -  CHECK PRESSURE REDUCTION TO STANDARD LEVEL
C        SEASON -  DETERMINE RELEVANT SEASON
C        SETINT -  SET CONFIDENCE FLAGS FOR INTERNAL CONSISTENCY CHECK
C        SETGRS -  UPDATE CONFIDENCE FLAGS FOR GROSS LIMIT CHECK
C        SETSF  -  SET CONFIDENCE FLAGS FOR GROSS LIMIT CHECK
C        SETABS -  SET CONFIDENCE FLAGS TO PARTICULAR VALUE
C        POSCHK -  COMPARE POSITION WITH PREVIOUS POSITIONS
C        UPFLAG -  ADD CONFIDENCE FLAGS TO REPORT
C
C       REFERENCE.
C      ------------
C
C        GUIDE ON THE GLOBAL DATA PROCESSING SYSTEM WMO N305 1982
C          CHAPTER 6,  QUALITY CONTROL PROCEDURES
C
C       AUTHOR.
C      ---------
C
C         B. NORRIS,  ECMWF,  MARCH 1989.
C
C       MODIFICATIONS.
C      ----------------
C
C         B. NORRIS, 14/03/90, ADD SHIP POSITION CHECK
C
      IMPLICIT LOGICAL (L,O,G)
      INCLUDE 'parameter.f'
      INCLUDE 'paramq.f'
      INCLUDE 'paramsh.f'
      PARAMETER (IUNIQ=12)
      INCLUDE 'comwt.f'
      INCLUDE 'const.f'
      INCLUDE 'comkey.f'
      INCLUDE 'comkeyc.f'
      DIMENSION PLIM(NMM)
      LOGICAL LPRES,LAND,LANSEA,LSHIP
      CHARACTER*35  CFNAME
      CHARACTER*9   CIDENT
      CHARACTER*9   CUNIQ(NUNIQ)
C
C    ---------------------------------------------------------------------------
C
C                      1.  SETUP
C                    --------------
C
  100 CONTINUE
      IF(IERR.NE.0) RETURN
      MISS=NINT(RMISS)
      LANSEA=.TRUE.
      LSHIP=.TRUE.
C          CFNAME = NAME OF TIME SERIES FILE
      CFNAME='SHIPPOSITION'
C
C          1.01  CALL SIGNS BEGINNING WITH THE FOLLOWING
C                     ARE NOT UNIQUE
C
      DATA (CUNIQ(K),K=1,IUNIQ)/
     X      'BOAT     ',
     X      'BUOY     ',
     X      'PLAT     ',
     X      'RIGG     ',
     X      'SHIP     ',
     X      'XXX      ',
     X      '???      ',
     X      '///      ',
     X      'BBXX     ',
     X      'ZSWAV    ',
     X      'CG2      ',
     X      'CG3      '/
C
C          1.1  POSITIONS OF PARAMETERS WITHIN HEADER
C
      IF(KODE.LT.20) THEN
C               LAND
           NIY=4
           NIM=5
           NID=6
           NIH=7
           NIN=8
           NRLAT=9
           NRLONG=10
           NZSTN=11
           NTW=IMISS
           LAND=.TRUE.
      ELSE
C               SEA
           NIY=5
           NIM=6
           NID=7
           NIH=8
           NIN=9
           NRLAT=10
           NRLONG=11
           NZSTN=IMISS
           LAND=.FALSE.
      ENDIF
C
C          1.2  POSITIONS OF METEOROLOGICAL PARAMETERS
C

      IF(KSEC.EQ.1) THEN
C               BASIC REPORT
           IF(KPRES.LE.1.OR.(.NOT.LAND)) THEN
C                    LOW LEVEL OR SHIP
                if (kode .eq. 16) then
                NPSTN=imiss
                NPMSL=12
                NPSTD=imiss
                NZ=imiss
                NPPP=imiss
                NA=imiss
                NDD=13
                NFF=14
                NTT=15
                NTD=16
                NVV=17
                NWW=18
                NTC=19
                NNH=21
                NH=23
                NCL=24
                NCM=25
                NCH=26
              else  
                NPSTN=12
                NPMSL=13
                NPSTD=IMISS
                NZ=IMISS
                NPPP=14
                NA=15
                NDD=16
                NFF=17
                NTT=18
                NTD=19
                NVV=21
                NWW=22
                NTC=25
                NNH=27
                NH=28
                NCL=29
                NCM=30
                NCH=31
                IF(.NOT.LAND) NTW=32
             endif
           ELSE
C                    HIGH LEVEL
              
                NPSTN=12
                NPMSL=IMISS
                NPSTD=13
                NZ=14
                NPPP=15
                NA=16
                NDD=17
                NFF=18
                NTT=19
                NTD=20
                NVV=22
                NWW=23
                NTC=26
                NNH=28
                NH=29
                NCL=30
                NCM=31
                NCH=32
           ENDIF
      ELSEIF(KSEC.EQ.2) THEN
C               EXTENDED REPORT
           IF(.NOT.LAND) THEN
C                    SHIP
                NDS=2
                NVS=3
                NTMIN=12
                NTMAX=13
           ELSE
C                    LAND
                NDS=IMISS
                NVS=IMISS
                NTMIN=13
                NTMAX=14
           ENDIF
      ELSE
           WRITE (*,'(1H ,''SYNOP TYPE NOT RECOGNIZED:'',I5)') KSEC
           GO TO 500
      ENDIF
      IOFF=0
C
C          1.3  LOOP FOR REPORTS WITHIN ARRAY 'VALUES'
C
      DO 405 J=1,NSUB
      IERR=0
C
C          1.4  EXTRACT HEADER VALUES
C
      IY=NINT(VALUES(NIY,J))
      IM=NINT(VALUES(NIM,J))
      ID=NINT(VALUES(NID,J))
      IH=NINT(VALUES(NIH,J))
      IN=NINT(VALUES(NIN,J))
      RLAT =VALUES(NRLAT,J)
      RLONG=VALUES(NRLONG,J)
      ZSTN=0.0
      IF(NZSTN.NE.IMISS) ZSTN=VALUES(NZSTN,J)
      ZMSL=0.0
C
C          1.5   SET UP TABLES
C
      CALL DERITA (IERR,M)
      IF(IERR.NE.0) GO TO 500
C
C          1.6   DETERMINE SEASON
C
      CALL SEASON (RLAT,RLONG,IM)
      PP2=RMISS
      Z2=RMISS
C
C          1.7   SHIP POSITION
C
      IF(.NOT.LAND) THEN
           CALL POSCHK (CFNAME,CIDENT,IUNIQ,CUNIQ,NTYPE,NSBTYPE,
     X        IY,IM,ID,IH,IN,RLAT,RLONG,LANSEA,LSHIP,IFL,IERR)
           IF(IERR.NE.0) THEN
                WRITE (*,'(1H ,'' POSCHK ERROR:'',I5)') IERR
           ELSE
                CALL SETABS (IFL,NRLAT,NRLONG,IEND)
           ENDIF
      ENDIF
C
C    --------------------------------------------------------------------
C
C              2.  BASIC REPORT
C            --------------------
C
  200 CONTINUE
      IF(KSEC.NE.1) GO TO 300
C
C         2.1  EXTRACT PARAMETER VALUES
C
      pstn=rmiss
      PMSL=RMISS
      PSTD=RMISS
      ppp=rmiss
      Z=RMISS
      ia=imiss
      IF(npstn.NE.IMISS) PSTN=VALUES(NPSTN,J)
      IF(NPMSL.NE.IMISS) PMSL=VALUES(NPMSL,J)
c      print*,'msl',npmsl,pmsl
      IF(NPSTD.NE.IMISS) PSTD=VALUES(NPSTD,J)
      IF(NZ.NE.IMISS) Z=VALUES(NZ,J)/9.8
      if (nppp.ne.imiss) PPP=VALUES(NPPP,J)
      if (ia.ne.imiss) IA=NINT(VALUES(NA,J))
      DD=VALUES(NDD,J)
c      print*,'dd',ndd,dd
      FF=VALUES(NFF,J)
c      print*,'ff',nff,ff
      TT=VALUES(NTT,J)
c      print*,'tt',ntt,tt
      TD=VALUES(NTD,J)
c      print*,'td',ntd,td
      VV=VALUES(NVV,J)
c      print*,'vv',nvv,vv
      IWW=NINT(VALUES(NWW,J))
c      print*,'ww',nww,iww
      TC=VALUES(NTC,J)
c      print*,'tc',ntc,tc
      INH=NINT(VALUES(NNH,J))
c      print*,'nh',nnh,inh
      H=VALUES(NH,J)
      ICL=NINT(VALUES(NCL,J))
      ICM=NINT(VALUES(NCM,J))
      ICH=NINT(VALUES(NCH,J))
      TW=RMISS
      IF(NTW.NE.IMISS) TW=VALUES(NTW,J)
C
C          2.2  GROSS LIMIT CHECKS
C
C
C                   STATION LEVEL PRESSURE
C
      IF(PSTN.NE.RMISS.AND.ZSTN.NE.RMISS) THEN
           CALL DPLIM (ZSTN,RLAT,RLONG,PLIM)
           CALL SETSF (PSTN,NPSTN,PLIM)
      ENDIF
C
C                   GEOPOTENTIAL
C
      IF(PSTD.NE.RMISS.AND.Z.NE.RMISS) THEN
           CALL CUAT (IERR,Z,'Z',NZ,RLAT,RLONG,PSTD,PP2,Z2,
     X                                       NPSTD,IMISS,IMISS)
           IF(IERR.NE.0) 
     x WRITE (*,2998) IERR,PSTD,Z
      ENDIF
2998  format(1h,'SYNOP CUAT ERROR:',i5,' PP:',f10.2,' Z:',F10.2)
C
C                   TOTAL CLOUD AMOUNT
C
      IF(TC.NE.RMISS) THEN
           IF(TC.LT.0.0.OR.TC.GT.100.0) CALL SETGRS (NTC,1)
      ENDIF
C
C                   LOWEST CLOUD AMOUNT
C
      IF(INH.NE.MISS) THEN
           IF(INH.LT.0.OR.INH.GT.9) CALL SETGRS (NNH,1)
      ENDIF
C
C                   PRESSURE CHARACTERISTIC
C
      IF(IA.NE.MISS.AND.IA.NE.15) THEN
           IF(IA.LT.0.OR.IA.GT.8) CALL SETGRS (NA,1)
      ENDIF
C
C                   SURFACE WIND SPEED
C
      IF(FF.NE.RMISS) THEN
           CALL CSLT (IERR,FF,'FF',NFF,RLAT,RLONG)
           IF(IERR.NE.0) 
     x WRITE (*,'(1H ,''SYNOP CSLT ERROR:'',I5,'' FF:'',F10.2)')IERR,FF
      ENDIF
C
C                   MEAN SEA LEVEL PRESSURE
C
      IF(PMSL.NE.RMISS) THEN
           CALL CSLT (IERR,PMSL,'PPP',NPMSL,RLAT,RLONG)
           IF(IERR.NE.0) 
     x WRITE (*,'(1H ,''SYNOP CSLT ERROR:'',I5,'' PMSL:'',F10.2)')
     x IERR,PMSL
      ENDIF
C
C               3 HOUR PRESSURE TENDENCY
C
      IF(IA.GE.0.AND.IA.LE.8.AND.PPP.NE.RMISS) THEN
           CALL CSLT (IERR,PPP,'PP',NPPP,RLAT,RLONG)
           IF(IERR.NE.0) 
     x WRITE (*,'(''SYNOP CSLT ERROR:'',I5,'' PPP:'',F10.2)') 
     x IERR,PPP
      ENDIF
C
C                   SURFACE AIR TEMPERATURE
C
      IF(TT.NE.RMISS) THEN
           CALL CSLT (IERR,TT,'TT',NTT,RLAT,RLONG)
           IF(IERR.NE.0) 
     x WRITE (*,'(''SYNOP CSLT ERROR:'',I5,'' TT:'',F10.2)') IERR,TT
      ENDIF
C
C                   SURFACE DEW POINT TEMPERATURE
C
      IF(TD.NE.RMISS) THEN
           CALL CSLT (IERR,TD,'TD',NTD,RLAT,RLONG)
           IF(IERR.NE.0) 
     x WRITE (*,'(1H ,''SYNOP CSLT ERROR:'',I5,'' TD:'',F10.2)') IERR,TD
      ENDIF
C
C                   SEA SURFACE TEMPERATURE
C
      IF(TW.NE.RMISS) THEN
           CALL CSLT (IERR,TW,'TW',NTW,RLAT,RLONG)
           IF(IERR.NE.0)
     x WRITE (*,'(1H ,''SYNOP CSLT ERROR:'',I5,'' TW:'',F10.2)') IERR,TW
      ENDIF
C
C          2.3  INTERNAL CONSISTENCY CHECKS
C
C                 REDUCTION OF STATION LEVEL PRESSURE
C                   TO STANDARD LEVEL
C
      IF(ZSTN.NE.RMISS.AND.PSTN.NE.RMISS.AND.
     X   PMSL.NE.RMISS.AND.TT.NE.RMISS) THEN
C                  PRESSURE REDUCED TO MSL
           LPRES=.FALSE.
           CALL PREDUC (ZMSL,ZSTN,PMSL,PSTN,TT,LPRES,
     X               IMISS,NZSTN,NPMSL,NPSTN,NTT)
      ENDIF
      IF(Z.NE.RMISS.AND.ZSTN.NE.RMISS.AND.PSTN.NE.RMISS.AND.
     X   PSTD.NE.RMISS.AND.TT.NE.RMISS) THEN
C                  GEOPOTENTIAL OF STANDARD PRESSURE LEVEL
           LPRES=.TRUE.
           CALL PREDUC (Z,ZSTN,PSTD,PSTN,TT,LPRES,
     X               NZ,NZSTN,NPSTD,NPSTN,NTT)
      ENDIF
C
C                WIND SPEED AND DIRECTION
C
      CALL CDDFF (DD,FF,NDD,NFF)
C
C                DEW POINT DEPRESSION
C
      IF(TT.NE.RMISS.AND.TD.NE.RMISS) THEN
           ICONF=IFAIL
           IF(.NOT.LAND) THEN
                IF(TT-TD.GE.-1.0.AND.TT-TD.LE.30.0) ICONF=IPASS
           ELSE
                IF(TT-TD.GE.0.0.AND.TT-TD.LE.50.0) ICONF=IPASS
           ENDIF
           CALL SETINT (ICONF,NTT,NTD,IEND)
C
C             DEW POINT DEPRESSION AND PRESENT WEATHER
C
           IF(IWW.NE.MISS) THEN
                ICONF=IPASS
                IF(TT-TD.GT.5.0.AND.((IWW.GE.40.AND.IWW.LE.49).OR.
     X                                IWW.EQ.120.OR.
     X                               (IWW.GE.130.AND.IWW.LE.135)))
     X          ICONF=IFAIL
                CALL SETINT (ICONF,NTT,NTD,NWW,IEND)
           ENDIF
      ENDIF
C
C               TEMPERATURE AND PRESENT WEATHER
C
      IF(TT.NE.RMISS.AND.IWW.NE.MISS) THEN
           ICONF=IPASS
           IF(TT.GT.5.0+RABS.AND.((IWW.GE.68.AND.IWW.LE.79).OR.
     X                            (IWW.GE.83.AND.IWW.LE.88).OR.
     X                             IWW.EQ.111.OR.IWW.EQ.124.OR.
     X                            (IWW.GE.145.AND.IWW.LE.146).OR.
     X                            (IWW.GE.167.AND.IWW.LE.168).OR.
     X                            (IWW.GE.170.AND.IWW.LE.176).OR.
     X                            (IWW.GE.185.AND.IWW.LE.187)))
     X     ICONF=IFAIL
           CALL SETINT (ICONF,NTT,NWW,IEND)
           ICONF=IPASS
           IF(TT.LT.-2.0+RABS.AND.((IWW.GE.50.AND.IWW.LE.55).OR.
     X                             (IWW.GE.58.AND.IWW.LE.65).OR.
     X                             (IWW.GE.68.AND.IWW.LE.69).OR.
     X                             (IWW.GE.80.AND.IWW.LE.82).OR.
     X                              IWW.EQ.123.OR.
     X                             (IWW.GE.143.AND.IWW.LE.144).OR.
     X                             (IWW.GE.151.AND.IWW.LE.153).OR.
     X                             (IWW.GE.157.AND.IWW.LE.158).OR.
     X                             (IWW.GE.161.AND.IWW.LE.163).OR.
     X                             (IWW.GE.167.AND.IWW.LE.168).OR.
     X                             (IWW.GE.181.AND.IWW.LE.184)))
     X     ICONF=IFAIL
           CALL SETINT (ICONF,NTT,NWW,IEND)
      ENDIF
C
C                  PRESSURE TENDENCY
C
      IF(IA.NE.MISS.AND.IA.NE.15.AND.PPP.NE.RMISS) THEN
           ICONF=IPASS
           IF(PPP.EQ.0.0.AND.IA.NE.0.AND.IA.NE.4.AND.IA.NE.5)
     X     ICONF=IFAIL
           CALL SETINT (ICONF,NA,NPPP,IEND)
           ICONF=IPASS
           IF(IA.EQ.4.AND.PPP.NE.0.0)
     X     ICONF=IFAIL
           CALL SETINT (ICONF,NA,NPPP,IEND)
      ENDIF
C
C                 VISIBILITY AND PRESENT WEATHER
C
      IF(VV.NE.RMISS.AND.IWW.NE.MISS) THEN
           ICONF=IPASS
           IF(VV.LT.1000.0.AND.((IWW.GE.0.AND.IWW.LE.3).OR.
     X                          (IWW.GE.5.AND.IWW.LE.29).OR.
     X                           IWW.EQ.40.OR.
     X                          (IWW.GE.50.AND.IWW.LE.53).OR.
     X                          (IWW.GE.60.AND.IWW.LE.72).OR.
     X                          (IWW.GE.79.AND.IWW.LE.83).OR.
     X                          (IWW.GE.100.AND.IWW.LE.104).OR.
     X                          (IWW.GE.106.AND.IWW.LE.119).OR.
     X                          (IWW.GE.121.AND.IWW.LE.123).OR.
     X                          (IWW.GE.125.AND.IWW.LE.126).OR.
     X                          (IWW.GE.160.AND.IWW.LE.168).OR.
     X                          (IWW.GE.181.AND.IWW.LE.182)))
     X     ICONF=IFAIL
           CALL SETINT (ICONF,NVV,NWW,IEND)
           ICONF=IPASS
           IF(VV.GE.1000.0.AND.((IWW.GE.41.AND.IWW.LE.49).OR.
     X                           IWW.EQ.120.OR.
     X                          (IWW.GE.130.AND.IWW.LE.135)))
     X     ICONF=IFAIL
           CALL SETINT (ICONF,NVV,NWW,IEND)
      ENDIF
C
C                 CLOUD COVER AND PRESENT WEATHER
C
      IF(IWW.NE.MISS) THEN
           IF(TC.NE.RMISS) THEN
                ICONF=IPASS
                IF(TC.GE.0.0.AND.TC.LE.100.0.AND.
     X                     (IWW.EQ.43.OR.IWW.EQ.45.OR.
     X                      IWW.EQ.47.OR.IWW.EQ.49))  ICONF=IFAIL
                CALL SETINT (ICONF,NTC,NWW,IEND)
                ICONF=IPASS
                IF(TC.EQ.0.0.AND.(IWW.EQ.3.OR.
     X                           (IWW.GE.14.AND.IWW.LE.17).OR.
     X                           (IWW.GE.50.AND.IWW.LE.75).OR.
     X                           (IWW.GE.77.AND.IWW.LE.99).OR.
     X                            IWW.EQ.103.OR.
     X                           (IWW.GE.121.AND.IWW.LE.126).OR.
     X                           (IWW.GE.140.AND.IWW.LE.199)))
     X          ICONF=IFAIL
                CALL SETINT (ICONF,NTC,NWW,IEND)
           ENDIF
           IF(INH.NE.MISS) THEN
                ICONF=IPASS
                IF(INH.NE.9.AND.
     X                     (IWW.EQ.43.OR.IWW.EQ.45.OR.
     X                      IWW.EQ.47.OR.IWW.EQ.49))  ICONF=IFAIL
                CALL SETINT (ICONF,NNH,NWW,IEND)
           ENDIF
      ENDIF
C
C                  CLOUD COVER AND CLOUD TYPE AND CLOUD HEIGHT
C
      IF(TC.NE.RMISS.AND.INH.NE.MISS.AND.ICH.NE.MISS) THEN
           ICONF=IPASS
C  TOTAL CLOUD > 0, LOWEST LAYER = 0, NO HIGH CLOUD
           IF(TC.GT.0.0.AND.INH.EQ.0.AND.ICH.EQ.10) ICONF=IFAIL
           CALL SETINT (ICONF,NTC,NNH,NCH,IEND)
           IF(ICM.NE.MISS) THEN
                ICONF=IPASS
C  TOTAL CLOUD = 100, LOWEST LAYER NOT COMPLETE,
C  MEDIUM AND HIGH CLOUD INVISIBLE
                IF(TC.EQ.100.0.AND.INH.LT.8.AND.ICM.EQ.61.AND.ICH.EQ.60)
     X          ICONF=IFAIL
                CALL SETINT (ICONF,NTC,NNH,NCM,NCH,IEND)
           ENDIF
      ENDIF
      IF(INH.NE.MISS) THEN
C  LOWEST LAYER INVISIBLE BUT CLOUD INFORMATION REPORTED
           IF(ICL.NE.MISS) THEN
                ICONF=IPASS
                IF(INH.EQ.9.AND.ICL.GE.30.AND.ICL.LE.39) ICONF=IFAIL
                CALL SETINT (ICONF,NNH,NCL,IEND)
           ENDIF
           IF(ICM.NE.MISS) THEN
                ICONF=IPASS
                IF(INH.EQ.9.AND.ICM.GE.20.AND.ICM.LE.29) ICONF=IFAIL
                CALL SETINT (ICONF,NNH,NCM,IEND)
           ENDIF
           IF(ICH.NE.MISS) THEN
                ICONF=IPASS
                IF(INH.EQ.9.AND.ICH.GE.10.AND.ICH.LE.19) ICONF=IFAIL
                CALL SETINT (ICONF,NNH,NCH,IEND)
           ENDIF
           IF(H.NE.RMISS) THEN
                ICONF=IPASS
                IF(INH.EQ.9.AND.H.GE.0.0) ICONF=IFAIL
                CALL SETINT (ICONF,NNH,NH,IEND)
           ENDIF
C  LOWEST LAYER = 0, BUT LOW AND/OR MEDIUM CLOUD REPORTED
           IF(ICL.NE.MISS) THEN
                ICONF=IPASS
                IF(INH.EQ.0.AND.ICL.GE.31.AND.ICL.LE.39) ICONF=IFAIL
                CALL SETINT (ICONF,NNH,NCL,IEND)
           ENDIF
           IF(ICM.NE.MISS) THEN
                ICONF=IPASS
                IF(INH.EQ.0.AND.ICM.GE.21.AND.ICM.LE.29) ICONF=IFAIL
                CALL SETINT (ICONF,NNH,NCM,IEND)
           ENDIF
           IF(H.NE.RMISS) THEN
                ICONF=IPASS
                IF(INH.EQ.0.AND.H.LT.2500.0) ICONF=IFAIL
                CALL SETINT (ICONF,NNH,NH,IEND)
           ENDIF
      ENDIF
      IF(TC.NE.RMISS) THEN
C  TOTAL CLOUD = 0, BUT CLOUD TYPES OR HEIGHTS REPORTED
           IF(ICL.NE.MISS) THEN
                ICONF=IPASS
                IF(TC.EQ.0.0.AND.ICL.GE.31.AND.ICL.LE.39) ICONF=IFAIL
                CALL SETINT (ICONF,NTC,NCL,IEND)
           ENDIF
           IF(ICM.NE.MISS) THEN
                ICONF=IPASS
                IF(TC.EQ.0.0.AND.ICM.GE.21.AND.ICM.LE.29) ICONF=IFAIL
                CALL SETINT (ICONF,NTC,NCM,IEND)
           ENDIF
           IF(ICH.NE.MISS) THEN
                ICONF=IPASS
                IF(TC.EQ.0.0.AND.ICH.GE.11.AND.ICH.LE.19) ICONF=IFAIL
                CALL SETINT (ICONF,NTC,NCH,IEND)
           ENDIF
           IF(H.NE.RMISS) THEN
                ICONF=IPASS
                IF(TC.EQ.0.0.AND.H.LT.2500.0) ICONF=IFAIL
                CALL SETINT (ICONF,NTC,NH,IEND)
           ENDIF
      ENDIF
      IF(TC.NE.RMISS.AND.ICL.NE.MISS.AND.ICM.NE.MISS.AND.ICH.NE.MISS)
     X      THEN
            ICONF=IPASS
C  TOTAL CLOUD > 0, BUT NO CLOUD TYPES REPORTED
            IF(TC.GT.0.0.AND.ICL.EQ.30.AND.ICM.EQ.20.AND.
     X         ICH.EQ.10) ICONF=IFAIL
            CALL SETINT (ICONF,NTC,NCL,NCM,NCH,IEND)
      ENDIF
      IF(INH.NE.MISS) THEN
           IF(ICL.NE.MISS) THEN
                IF(ICM.NE.MISS) THEN
                     ICONF=IPASS
C  LOWEST LAYER EXISTS BUT NO LOW OR MEDIUM CLOUD
                     IF(INH.NE.0.AND.ICL.EQ.30.AND.ICM.EQ.20)
     X                    ICONF=IFAIL
                     CALL SETINT (ICONF,NNH,NCL,NCM,IEND)
                     ICONF=IPASS
C  COMPLETE LOWEST LAYER BUT MEDIUM CLOUD NOT INVISIBLE
                     IF(INH.EQ.8.AND.ICL.GE.31.AND.ICL.LE.39.AND.
     X                   ICM.NE.61) ICONF=IFAIL
                     CALL SETINT (ICONF,NNH,NCL,NCM,IEND)
                ENDIF
                ICONF=IPASS
C  LOWEST LAYER NOT COMPLETE BUT LOW CLOUD INVISIBLE
                IF(INH.NE.9.AND.ICL.EQ.62) ICONF=IFAIL
                CALL SETINT (ICONF,NNH,NCL,IEND)
           ENDIF
           IF(ICH.NE.MISS) THEN
                ICONF=IPASS
C  COMPLETE LOWEST LAYER BUT HIGH CLOUD NOT INVISIBLE
                IF(INH.EQ.8.AND.ICH.NE.60) ICONF=IFAIL
                CALL SETINT (ICONF,NNH,NCH,IEND)
           ENDIF
      ENDIF
C
C               CONSISTENCY OF CLOUD TYPES
C
      IF(ICH.NE.MISS) THEN
           IF(ICM.NE.MISS) THEN
                ICONF=IPASS
C  MEDIUM CLOUD INVISIBLE BUT HIGH CLOUD NOT INVISIBLE
                IF(ICM.EQ.61.AND.ICH.NE.60) ICONF=IFAIL
                CALL SETINT (ICONF,NCM,NCH,IEND)
                IF(ICL.NE.MISS) THEN
                     ICONF=IPASS
C  LOW CLOUD INVISIBLE BUT MEDIUM AND HIGH CLOUD NOT INVISIBLE
                     IF(ICL.EQ.62.AND.ICM.NE.61.AND.ICH.NE.60)
     X               ICONF=IFAIL
                     CALL SETINT (ICONF,NCL,NCM,NCH,IEND)
                ENDIF
           ENDIF
      ENDIF
C
C              CLOUD HEIGHT AND VISIBILITY
C
      IF(H.NE.RMISS.AND.VV.NE.RMISS) THEN
           ICONF=IPASS
           IF(H.LT.100.0.AND.VV.GT.20000.0) ICONF=IFAIL
           CALL SETINT (ICONF,NH,NVV,IEND)
      ENDIF
      GO TO 400
C
C    --------------------------------------------------------------------
C
C                3.  EXTENDED REPORT
C               ---------------------
C
  300 CONTINUE
      IF(KSEC.NE.2) GO TO 400
C
C           3.1  EXTRACT PARAMETER VALUES
C
      DS=RMISS
      VS=RMISS
      IF(NDS.NE.IMISS) DS=VALUES(NDS,J)
      IF(NVS.NE.IMISS) VS=VALUES(NVS,J)
      TMIN=VALUES(NTMIN,J)
      TMAX=VALUES(NTMAX,J)
C
C           3.2  GROSS LIMIT CHECKS
C
C
C                   MINIMUM AIR TEMPERATURE
C
      IF(TMIN.NE.RMISS) THEN
           CALL CSLT (IERR,TMIN,'TT',NTMIN,RLAT,RLONG)
           IF(IERR.NE.0)
     x WRITE (*,'(1H,''SYNOP CSLT ERROR:'',I5,'' TMIN:'',F10.2)')
     x IERR,TMIN
      ENDIF
C
C                   MAXIMUM AIR TEMPERATURE
C
      IF(TMAX.NE.RMISS) THEN
           CALL CSLT (IERR,TMAX,'TT',NTMAX,RLAT,RLONG)
           IF(IERR.NE.0) 
     x WRITE (*,'(1H ,''SYNOP CSLT ERROR:'',I5,'' TMAX:'',F10.2)')
     x IERR,TMAX
      ENDIF
C
C            3.3  INTERNAL CONSISTENCY CHECKS
C
C                  SHIP SPEED AND DIRECTION
C
      CALL CDDFF (DS,VS,NDS,NVS)
C
C    --------------------------------------------------------------------
C
C                 4. ADD FLAGS TO BUFR RECORD
C                -------------------------------
C
  400 CONTINUE
      CALL UPFLAG (IERR,J,VALUES,JP22,JP1,M,IOFF)
      IF(IERR.NE.0) GO TO 500
C
  405 CONTINUE
C
C    --------------------------------------------------------------------
C
C                   5.  EXIT
C                  -----------
C
  500 CONTINUE
      RETURN
      END
      SUBROUTINE QCTEMP (KODE,IERR)
C
C**** *QCTEMP*
C
C       PURPOSE.
C      ----------
C
C          QUALITY CONTROL FOR TEMPS
C
C       INTERFACE.
C      ------------
C
C        CALL QCTEMP (KODE,IERR)
C
C        INPUT
C         ARRAY VALUES (COMWT) - TEMP REPORTS
C         KODE   -  INDICATOR FOR TYPE OF TEMP
C
C        OUTPUT
C         ARRAY VALUES (COMWT) - TEMP REPORTS WITH CONFIDENCE FLAGS
C                                               ADDED
C         IERR   -  ERROR CODE
C
C       METHOD.
C      ---------
C
C        COMPARE POSITION WITH PREVIOUS POSITION IF SHIP;
C        CHECK SURFACE LEVEL AGAINST LIMITS;
C        CHECK UPPER AIR LEVELS AGAINST LIMITS;
C        CHECK CONSISTENCY OF WIND SPEED AND DIRECTION;
C        CHECK FOR UNREASONABLE INVERSIONS OR LAPSE RATES;
C        CHECK CONSECUTIVE STANDARD LEVELS FOR EXCESSIVE WIND SHEAR;
C        CHECK HEIGHTS OF STANDARD LEVELS WITH THOSE RECOMPUTED
C            FROM SIGNIFICANT LEVELS;
C        HYDROSTATIC CHECK OF STANDARD LEVELS;
C        INSERT COMPUTED SIGNIFICANT LEVEL HEIGHTS IN BUFR MESSAGE
C
C       EXTERNALS.
C      ------------
C
C        CDDFF  -  CHECK CONSISTENCY OF WIND SPEED AND DIRECTION
C        CUAT   -  GROSS LIMIT CHECK FOR UPPER AIR PARAMETERS
C        CSLT   -  GROSS LIMIT CHECK FOR SURFACE PARAMETERS
C        DPLIM  -  OBTAIN LIMITS FOR PRESSURE GIVEN HEIGHT
C        DERITA -  INITIALISE TABLES AND CONFIDENCE FLAGS
C        POSCHK -  COMPARE POSITION WITH PREVIOUS POSITIONS
C        INSHIZ -  DETERMINE SHIP PLATFORM HEIGHT
C        SEASON -  DETERMINE RELEVANT SEASON
C        SETSF  -  UPDATE CONFIDENCE FLAGS FOR GROSS LIMIT CHECK
C        SETGRS -  SET CONFIDENCE FLAGS FOR GROSS LIMIT CHECK
C        SETABS -  SET CONFIDENCE FLAGS TO PARTICULAR VALUE
C        LAPS   -  CHECK FOR UNREASONABLE INVERSIONS OR LAPSE RATES
C        WSHEAR -  CHECK WIND SHEAR BETWEEN STANDARD LEVELS
C        TVIRT  -  CALCULATE VIRTUAL TEMPERATURE
C        ZCOMP  -  RECOMPUTE AND CHECK STANDARD LEVEL HEIGHTS
C        HYDRO  -  HYDROSTATIC CHECK OF STANDARD LEVELS
C        UPFLAG -  ADD CONFIDENCE FLAGS TO REPORT
C
C       REFERENCE.
C      ------------
C
C        GUIDE ON THE GLOBAL DATA PROCESSING SYSTEM WMO N305 1982
C          CHAPTER 6,  QUALITY CONTROL PROCEDURES
C
C       AUTHOR.
C      ---------
C
C         B. NORRIS,  ECMWF,  FEBRUARY 1989.
C
C       MODIFICATIONS.
C      ----------------
C
C         B. NORRIS, 89/12/21, RECOMPUTE STANDARD LEVEL HEIGHTS
C         B. NORRIS, 90/02/14, INSERT COMPUTED SIGNIFICANT LEVEL
C                               HEIGHTS IN VALUES ARRAY
C         B. NORRIS, 90/03/26, LAPSE RATE CHECK
C         B. NORRIS, 90/03/27, HYDROSTATIC CHECK
C         B. NORRIS, 91/01/21, INSERT SHIP PLATFORM HEIGHT IN SURF LEVEL
C         B. NORRIS, 92/06/26, ADD SHIP POSITION CHECK
C         M. DRAGOSAVAC 93/04/06 READ IN BIAS CORRECTION TABLES ONLY ONCE
C
      IMPLICIT LOGICAL (L,O,G)
      INCLUDE 'parameter.f'
      INCLUDE 'paramq.f'
      INCLUDE 'paramsh.f'
      PARAMETER (IUNIQ=12)
      PARAMETER (JTEMP=100)
      INCLUDE 'comwt.f'
      INCLUDE 'comkey.f'
      INCLUDE 'comkeyc.f'
      INCLUDE 'const.f'
      INCLUDE 'conf.f'
      DIMENSION PLIM(NMM)
      DIMENSION STDPP(NLEV),STDDD(NLEV),STDFF(NLEV)
      DIMENSION ISTDDD(NLEV),ISTDFF(NLEV)
      DIMENSION SPP(JTEMP),STT(JTEMP),STD(JTEMP)
      DIMENSION IVSS(JTEMP),SZ(JTEMP),STV(JTEMP),SZN(JTEMP)
      DIMENSION ISPP(JTEMP),ISTT(JTEMP),ISTD(JTEMP),ISZ(JTEMP)
      CHARACTER*35 CFNAME
      CHARACTER*9 CIDENT
      CHARACTER*9 CUNIQ(NUNIQ)
      LOGICAL LAND
      LOGICAL LANSEA
      LOGICAL LSHIP
      LOGICAL LBIAS
C
      INCLUDE 'parabias.f'
      save nsobc,nstbc,LBIAS
      save YSTIDBC,YDUMMYBC,YSTYPBC,YSNAMBC,ILEVBC,RCORBC,SZNBC,LBC
C
C
C    ---------------------------------------------------------------------------
C
C                      1.  SETUP
C                    --------------
C
  100 CONTINUE
C
C          1.1  DATA INITIALIZATION
C
C       POSITIONS OF PARAMETERS WITHIN LEVEL
      DATA NPP/1/,NVS/2/,NZ/3/,NTT/4/,NTD/5/,NDD/6/,NFF/7/
C       CRITICAL CONFIDENCE VALUE
      DATA ICRIT/65/
      LANSEA=.TRUE.
      LSHIP=.TRUE.
C          CFNAME = NAME OF TIME SERIES FILE
      CFNAME='SHIPTEMPPOSI'
C
C          1.11  CALL SIGNS BEGINNING WITH THE FOLLOWING
C                     ARE NOT UNIQUE
C
      DATA (CUNIQ(K),K=1,IUNIQ)/
     X      'BOAT     ',
     X      'BUOY     ',
     X      'PLAT     ',
     X      'RIGG     ',
     X      'SHIP     ',
     X      'XXX      ',
     X      '???      ',
     X      '///      ',
     X      'BBXX     ',
     X      'ZSWAV    ',
     X      'CG2      ',
     X      'CG3      '/
C
C
C          1.2  POSITIONS OF PARAMETERS WITHIN HEADER
C
      IF(IERR.NE.0) RETURN

      IF(IERR.NE.0) THEN
          WRITE (*,'(1H ,'' BIASCOR ERROR:'',I5)') IERR
          RETURN
      ENDIF
      LAND=.FALSE.
      IF(KODE.EQ.35) THEN
           NIY=5
           NIM=6
           NID=7
           NIH=8
           NIN=9
           NRLAT=10
           NRLONG=11
           NALT=12
           LAND=.TRUE.
           NILEV=20
      ELSEIF(KODE.EQ.36.OR.KODE.EQ.135.OR.KODE.EQ.137) THEN
           NIY=6
           NIM=7
           NID=8
           NIH=9
           NIN=10
           NRLAT=11
           NRLONG=12
           NILEV=21
      ELSE
           WRITE (*,'(1H ,''TEMP TYPE NOT RECOGNIZED:'',I5)') KODE
           GO TO 400
      ENDIF
      ILVSIZ=7
      IOFF=0
      NSTD=0
      NST=0
C
C          1.3  LOOP FOR REPORTS WITHIN ARRAY 'VALUES'
C
      DO 305 J=1,NSUB
      IERR=0
C
C
C          1.4  EXTRACT HEADER VALUES
C
      NUMLEV=NINT(VALUES(NILEV,J))
      IF(LAND) THEN
           ZSTN=VALUES(NALT,J)
      ELSE
           CALL INSHIZ (IERR,CIDENT,ZSTN)
           IF(IERR.NE.0) GO TO 400
      ENDIF
      IY=NINT(VALUES(NIY,J))
      IM=NINT(VALUES(NIM,J))
      ID=NINT(VALUES(NID,J))
      IH=NINT(VALUES(NIH,J))
      IN=NINT(VALUES(NIN,J))
      RLAT =VALUES(NRLAT,J)
      RLONG=VALUES(NRLONG,J)
C
C          1.5   SET UP TABLES
C
      CALL DERITA (IERR,M)
      IF(IERR.NE.0) GO TO 400
C
C          1.6   DETERMINE SEASON
C
      CALL SEASON (RLAT,RLONG,IM)
      PP2=RMISS
      Z2=RMISS
C
C          1.65  SHIP POSITION
C
      IF(KODE.EQ.36) THEN
           CALL POSCHK (CFNAME,CIDENT,IUNIQ,CUNIQ,NTYPE,NSBTYPE,
     X        IY,IM,ID,IH,IN,RLAT,RLONG,LANSEA,LSHIP,IFL,IERR)
           IF(IERR.NE.0) THEN
                WRITE (*,'(1H ,'' POSCHK ERROR:'',I5)') IERR
           ELSE
                CALL SETABS (IFL,NRLAT,NRLONG,IEND)
           ENDIF
      ENDIF
C
C          1.7   LOOP FOR LEVELS
C
      DO 245 ILEV=1,NUMLEV
      IBASE=(ILEV-1)*ILVSIZ+NILEV
      NNPP=IBASE+NPP
      NNZ=IBASE+NZ
      NNTT=IBASE+NTT
      NNTD=IBASE+NTD
      NNDD=IBASE+NDD
      NNFF=IBASE+NFF
      NNVS=IBASE+NVS
C
C          1.8  EXTRACT PARAMETER VALUES
C
      PP=VALUES(NNPP,J)
      Z =VALUES(NNZ ,J)
      IF(Z.NE.RMISS) Z=Z/9.8        ! CHANGED  10/11/89.
      TT=VALUES(NNTT,J)
      TD=VALUES(NNTD,J)
      DD=VALUES(NNDD,J)
      FF=VALUES(NNFF,J)
      IVS=NINT(VALUES(NNVS,J))
C
C    --------------------------------------------------------------------
C
C                2.  CHECKING OF PARAMETER VALUES
C             -------------------------------------
C
  200 CONTINUE
C
C            2.1   EXTRACT PARTICULAR LEVELS
C
C         2.11  EXTRACT STANDARD LEVEL WINDS
C
      IF(PP.NE.RMISS.AND.DD.NE.RMISS.AND.FF.NE.RMISS.AND.
     X  ((IVS.AND.32).EQ.32).AND.NSTD.LT.NLEV) THEN
           NSTD=NSTD+1
           STDPP(NSTD)=PP
           STDDD(NSTD)=DD
           STDFF(NSTD)=FF
           ISTDDD(NSTD)=NNDD
           ISTDFF(NSTD)=NNFF
      ENDIF
C
C            2.12  EXTRACT TEMPERATURE LEVELS
C
      IF(PP.NE.RMISS.AND.TT.NE.RMISS.AND.NST.LT.JTEMP) THEN
           NST=NST+1
           SPP(NST) = PP
           STT(NST) = TT
           STD(NST) = TD
           IVSS(NST) = IVS
           SZ(NST) = Z
           SZN(NST) = RMISS
           ISPP(NST) = NNPP
           ISTT(NST) = NNTT
           ISTD(NST) = NNTD
           ISZ(NST) = NNZ
      ENDIF
C
C            2.2  CHECK SURFACE LEVEL
C
      IF((IVS.AND.64).EQ.64) THEN
C
C          2.21  SURFACE PRESSURE
C
           IF(PP.NE.RMISS.AND.ZSTN.NE.RMISS) THEN
                CALL DPLIM (ZSTN,RLAT,RLONG,PLIM)
                CALL SETSF (PP,NNPP,PLIM)
           ENDIF
C
C          2.22  SURFACE TEMPERATURE
C
           IF(TT.NE.RMISS) THEN
                CALL CSLT (IERR,TT,'TT',NNTT,RLAT,RLONG)
           IF(IERR.NE.0)
     x      WRITE(*,'(1H,''TEMP CSLT ERROR:'',I5,'' TT:'',F10.2)')IERR,TT
           ENDIF
C
C          2.23  SURFACE DEWPOINT
C
           IF(TD.NE.RMISS) THEN
                CALL CSLT (IERR,TD,'TD',NNTD,RLAT,RLONG)
           IF(IERR.NE.0)
     x      WRITE(*,'(1H,''TEMP CSLT ERROR:'',I5,'' TD:'',F10.2)')IERR,TD
           ENDIF
C
C          2.24  SURFACE WIND SPEED
C
           IF(FF.NE.RMISS) THEN
                CALL CSLT (IERR,FF,'FF',NNFF,RLAT,RLONG)
           IF(IERR.NE.0)
     x      WRITE(*,'(1H,''TEMP CSLT ERROR:'',I5,'' FF:'',F10.2)')IERR,ff
           ENDIF
           GO TO 235
      ENDIF
C
C         2.3  UPPER AIR LEVEL
C
C              2.31  GROSS LIMIT CHECK FOR PRESSURE
C
      IF(PP.NE.RMISS) THEN
           IF(PP.LT.0.0.OR.PP.GT.108000.0) CALL SETGRS (NNPP,1)
      ENDIF
C
C              2.32  CHECK HEIGHT DEPENDING ON PRESSURE
C
      IF(PP.NE.RMISS.AND.Z.NE.RMISS) THEN
                CALL CUAT (IERR,Z,'Z',NNZ,RLAT,RLONG,PP,PP2,Z2,
     X                                           NNPP,IMISS,IMISS)
      IF(IERR.NE.0)
     x WRITE(*,'(''TEMP CUAT ERROR:'',I5,''PP:'',F10.2,''Z:'',F10.2)')
     x IERR,PP,Z
      ENDIF
C
C              2.33  CHECK TEMPERATURE DEPENDING ON HEIGHT OR PRESSURE
C
      IF(PP.NE.RMISS.OR.Z.NE.RMISS) THEN
           IF(PP.NE.RMISS) THEN
                PAR1=PP
                NPAR1=NNPP
                PAR3=RMISS
                NPAR3=IMISS
           ELSE
                PAR1=RMISS
                NPAR1=IMISS
                PAR3=Z
                NPAR3=NNZ
           ENDIF
           IF(TT.NE.RMISS) THEN
                CALL CUAT (IERR,TT,'TT',NNTT,RLAT,RLONG,PAR1,PP2,PAR3,
     X                                          NPAR1,IMISS,NPAR3)
           IF(IERR.NE.0) WRITE(*,3789)IERR,Z,PP,TT
           ENDIF
C
C              2.34  CHECK WIND SPEED DEPENDING ON HEIGHT OR PRESSURE
C
           IF(FF.NE.RMISS) THEN
                CALL CUAT (IERR,FF,'FF',NNFF,RLAT,RLONG,PAR1,PP2,PAR3,
     X                                          NPAR1,IMISS,NPAR3)
                IF(IERR.NE.0) WRITE(*,3788) IERR,Z,PP,FF
           ENDIF
      ENDIF
c
3788  format(1h,'TEMP CUAT ERROR:',i5,' Z:',f10.2,' PP:',f10.2,
     x ' PP:',F10.2,' FF:',F10.2)
3789  format(1h,'TEMP CUAT ERROR:',i5,' Z:',f10.2,' PP:',f10.2,
     x ' PP:',F10.2,' TT:',F10.2)    
      if(ierr.ne.0) then
         print*,' *********** dados espurios'
         return
      end if
c
  235 CONTINUE
C
C             2.4  CHECK CONSISTENCY OF WIND SPEED AND DIRECTION
C
      CALL CDDFF (DD,FF,NNDD,NNFF)
C
C             2.41  CHECK DEWPOINT DEPRESSION
C
      IF(TT.NE.RMISS.AND.TD.NE.RMISS) THEN
           DEFCIT=TT-TD
           IF(DEFCIT.LT.-1.0.OR.DEFCIT.GT.50.0) THEN
                CALL SETGRS (NNTT,3)
                CALL SETGRS (NNTD,3)
           ENDIF
      ENDIF
  245 CONTINUE
C
C             2.5  CHECK INVERSION AND LAPSE RATE
C
      CALL LAPS (IERR,NST,SPP,STT,IVSS,ISPP,ISTT,RLAT,RLONG,
     X           CIDENT,IY,IM,ID,IH,IN)
      IF(IERR.NE.0) WRITE (*,'(1H ,''TEMP LAPS ERROR:'',I5)') IERR
C
C             2.6  DEWPOINT IS OBTAINED AS DEPRESSION FROM TEMPERATURE
C
      IF(NST.GT.0) THEN
           DO 265 ILEV=1,NST
           IF(STD(ILEV).EQ.RMISS) GO TO 265
           IFTD=MIN(IFLAG(1,ISTT(ILEV)),IFLAG(1,ISTD(ILEV)))
           CALL SETABS (IFTD,ISTD(ILEV),IEND)
  265      CONTINUE
      ENDIF
C
C             2.7  CHECK WIND SHEAR BETWEEN STANDARD LEVELS
C
      CALL WSHEAR (IERR,NSTD,STDPP,STDDD,STDFF,ISTDDD,ISTDFF)
      IF(IERR.NE.0) WRITE (*,'(1H ,''TEMP WSHEAR ERROR:'',I5)') IERR
C
C             2.71  CALCULATE VIRTUAL TEMPERATURE
C                    IF TD MISSING, TVIRT = TT
C
      DO 2715 ILEV=1,NST
      STV(ILEV)=RMISS
      IF(IFLAG(1,ISPP(ILEV)).GE.ICRIT.AND.
     X   IFLAG(1,ISTT(ILEV)).GE.ICRIT) THEN
           STV(ILEV)=STT(ILEV)
           IF(STD(ILEV).NE.RMISS.AND.IFLAG(1,ISTD(ILEV)).GE.ICRIT)
     X           CALL TVIRT (SPP(ILEV),STT(ILEV),STD(ILEV),STV(ILEV))
      ENDIF
 2715 CONTINUE
C
C             2.8  CHECK HEIGHTS OF STANDARD LEVELS
C
C         2.81  CHECK THAT STATION HEIGHT IS MEANINGFUL
C
      IF(ZSTN.EQ.RMISS) GO TO 288
      IF(LAND.AND.IFLAG(1,NALT).LT.ICRIT) GO TO 288
C
C         2.82  CALCULATE STANDARD LEVEL HEIGHTS AND COMPARE
C
      CALL ZCOMP (IERR,NST,SPP,STV,IVSS,SZ,ISZ,SZN,ZSTN,
     X            CIDENT,IY,IM,ID,IH,IN)
      IF(IERR.NE.0) WRITE (*,'(1H ,''TEMP ZCOMP ERROR:'',I5)') IERR
  288 CONTINUE
C
C         2.9   HYDROSTATIC CHECK OF STANDARD LEVELS
C
      CALL HYDRO (IERR,NST,SPP,STV,IVSS,SZ,ISZ,ISTT,ISTD,
     X            CIDENT,IY,IM,ID,IH,IN)
      IF(IERR.NE.0) WRITE (*,'(1H ,''TEMP HYDRO ERROR:'',I5)') IERR
C
C    --------------------------------------------------------------------
C
C                 3. ADD FLAGS AND SUBSTITUTIONS TO BUFR RECORD
C                ------------------------------------------------
C
  300 CONTINUE
C
C
C           3.01  ADD FLAGS
C
      CALL UPFLAG (IERR,J,VALUES,JP22,JP1,M,IOFF)
      IF(IERR.NE.0) GO TO 400
C
C           3.02  INSERT DERIVED HEIGHTS
C
      DO 302 ILEV=1,NST
      IF(SZ(ILEV).EQ.RMISS.AND.SZN(ILEV).NE.RMISS.AND.
     X  ((IVSS(ILEV).AND.4).EQ.4.OR.(IVSS(ILEV).AND.64).EQ.64)) THEN
C          VALUES(2*M+ISZ(ILEV),J)=SZN(ILEV)*9.8
C*****
C*****      REMOVE NEXT 2 LINES WHEN BUFR CAN HANDLE SUBSTITUTIONS
C*****
          VALUES(ISZ(ILEV),J)=SZN(ILEV)*9.8
          VALUES(M+ISZ(ILEV),J)=70.0
      ENDIF
  302 CONTINUE
C
C          DO THE BIAS CORRECTION
C          READ BIAS CORRECTION FILES
C
      LBC=.TRUE.
C
      IF(.NOT.LBIAS) THEN
      print*,' ***************** RBCFILES ROUTINE COMMENTED OUT'
c     CALL RBCFILES (IERR,YSTIDBC,YSTYPBC,NSTBC,YSNAMBC,RCORBC,NSOBC)
      LBIAS=.TRUE.
      END IF
C
C          1.7   LOOP FOR LEVELS
C
      NST=0
      DO 303 ILEV=1,NUMLEV
      IBASE=(ILEV-1)*ILVSIZ+NILEV
      NNPP=IBASE+NPP
      NNZ=IBASE+NZ
c      NNTT=IBASE+NTT
c      NNTD=IBASE+NTD
c      NNDD=IBASE+NDD
c      NNFF=IBASE+NFF
c      NNVS=IBASE+NVS
c
C          1.8  EXTRACT PARAMETER VALUES
C
      PP=VALUES(NNPP,J)
      Z =VALUES(NNZ ,J)
      IF(Z.NE.RMISS) Z=Z/9.8        ! CHANGED  10/11/89.
c      TT=VALUES(NNTT,J)
c      TD=VALUES(NNTD,J)
c      DD=VALUES(NNDD,J)
c      FF=VALUES(NNFF,J)
c      IVS=NINT(VALUES(NNVS,J))
C
C            2.12  EXTRACT TEMPERATURE LEVELS
C
      IF(PP.NE.RMISS.AND.Z.NE.RMISS.AND.NST.LT.JTEMP) THEN
           NST=NST+1
           SPP(NST) = PP
c           STT(NST) = TT
c           STD(NST) = TD
c           IVSS(NST) = IVS
           SZ(NST) = Z
           SZNBC(NST) = RMISS
           ISPP(NST) = NNPP
c           ISTT(NST) = NNTT
c           ISTD(NST) = NNTD
           ISZ(NST) = NNZ
      ENDIF
C

  303 CONTINUE
C
C        2.10   BIASCORRECTION
C
      CALL BIASCOR (LBC,NST,SPP,SZ,SZNBC,
     X              YSTIDBC,YSTYPBC,NSTBC,YSNAMBC,RCORBC,NSOBC, 
     X              CIDENT,IM,ID,IH,RLAT,RLONG)
C
      DO 304 ILEV=1,NST
      IF( LBC.AND.SZNBC(ILEV).NE.RMISS) THEN
C     X  .AND.((IVSS(ILEV).AND.4).EQ.4.OR.(IVSS(ILEV).AND.64).EQ.64)) THEN
          VALUES(2*M+ISZ(ILEV),J)=SZNBC(ILEV)*9.8
      ENDIF
  304 CONTINUE
C END OF BIASCOR
C
C
  305 CONTINUE
C
C    --------------------------------------------------------------------
C
C                   4.  EXIT
C                  -----------
C
  400 CONTINUE
      RETURN
      END
      SUBROUTINE QCTESAC (IERR)
C
C**** *QCTESAC*
C
C       PURPOSE.
C      ----------
C
C          QUALITY CONTROL FOR TESACS
C
C       INTERFACE.
C      ------------
C
C        CALL QCTESAC (IERR)
C
C        INPUT
C         ARRAY VALUES (COMWT) - TESAC REPORTS
C         L31  (COMWT)  -  LOGICAL FOR SURFACE REPORT
C         L32  (COMWT)  -  LOGICAL FOR SUB-SURFACE REPORT
C
C        OUTPUT
C         ARRAY VALUES (COMWT) - TESAC REPORTS WITH CONFIDENCE FLAGS
C                                               ADDED
C         IERR   -  ERROR CODE
C
C       METHOD.
C      ---------
C
C        CHECK PARAMETER VALUES AGAINST GROSS LIMITS;
C        CHECK CONSISTENCY OF WIND SPEED AND DIRECTION;
C
C       EXTERNALS.
C      ------------
C
C        CDDFF  -  CHECK CONSISTENCY OF WIND SPEED AND DIRECTION
C        CSLT   -  GROSS LIMIT CHECK FOR SURFACE PARAMETERS
C        DERITA -  INITIALISE TABLES AND CONFIDENCE FLAGS
C        SEASON -  DETERMINE RELEVANT SEASON
C        UPFLAG -  ADD CONFIDENCE FLAGS TO REPORT
C
C       REFERENCE.
C      ------------
C
C        GUIDE ON THE GLOBAL DATA PROCESSING SYSTEM WMO N305 1982
C          CHAPTER 6,  QUALITY CONTROL PROCEDURES
C
C       AUTHOR.
C      ---------
C
C         B. NORRIS,  ECMWF,  JULY 1989.
C
C       MODIFICATIONS.
C      ----------------
C
C         NONE
C
      IMPLICIT LOGICAL (L,O,G)
      INCLUDE 'parameter.f'
      INCLUDE 'paramq.f'
      INCLUDE 'comwt.f'
      INCLUDE 'const.f'
C
C    ---------------------------------------------------------------------------
C
C                      1.  SETUP
C                    --------------
C
  100 CONTINUE
      IF(IERR.NE.0) RETURN
      MISS=NINT(RMISS)
C
C          1.1  POSITIONS OF PARAMETERS WITHIN HEADER
C
      IF(L31) THEN
           NMONTH=6
           NRLAT=10
           NRLONG=11
      ENDIF
      IF(L32) THEN
           NMONTH=3
           NRLAT=7
           NRLONG=8
      ENDIF
C
C          1.2  POSITIONS OF METEOROLOGICAL PARAMETERS
C
      IF(L31) THEN
           NDD=16
           NFF=17
           NTT=18
           NTW=32
      ENDIF
      IOFF=0
C
C          1.3  LOOP FOR REPORTS WITHIN ARRAY 'VALUES'
C
      DO 405 J=1,NSUB
      IERR=0
C
C          1.4  EXTRACT HEADER VALUES
C
      MONTH=NINT(VALUES(NMONTH,J))
      RLAT =VALUES(NRLAT,J)
      RLONG=VALUES(NRLONG,J)
C
C          1.5   SET UP TABLES
C
      CALL DERITA (IERR,M)
      IF(IERR.NE.0) GO TO 500
C
C          1.6   DETERMINE SEASON
C
      CALL SEASON (RLAT,RLONG,MONTH)
C
C          1.7   EXTRACT PARAMETER VALUES
C
      IF(.NOT.L31) GO TO 400
      DD=VALUES(NDD,J)
      FF=VALUES(NFF,J)
      TT=VALUES(NTT,J)
      TW=VALUES(NTW,J)
C
C    ----------------------------------------------------------------------
C
C                 2.  GROSS LIMIT CHECKS
C                -------------------------
C
  200 CONTINUE
C
C             2.1   SURFACE WIND SPEED
C
      IF(FF.NE.RMISS) THEN
           CALL CSLT (IERR,FF,'FF',NFF,RLAT,RLONG)
           IF(IERR.NE.0)
     x WRITE (*,'(1H ,''TESAC CSLT ERROR:'',I5,'' FF:'',F10.2)')IERR,FF
      ENDIF
C
C             2.2   SURFACE AIR TEMPERATURE
C
      IF(TT.NE.RMISS) THEN
           CALL CSLT (IERR,TT,'TT',NTT,RLAT,RLONG)
           IF(IERR.NE.0) 
     x WRITE (*,'(1H ,''TESAC CSLT ERROR:'',I5,'' TT:'',F10.2)') IERR,TT
      ENDIF
C
C             2.3   SEA SURFACE TEMPERATURE
C
      IF(TW.NE.RMISS) THEN
           CALL CSLT (IERR,TW,'TW',NTW,RLAT,RLONG)
           IF(IERR.NE.0) 
     x WRITE (*,'(1H ,''TESAC CSLT ERROR:'',I5,'' TW:'',F10.2)') IERR,TW
      ENDIF
C
C   ----------------------------------------------------------------------
C
C                  3.  INTERNAL CONSISTENCY CHECKS
C                ------------------------------------
C
  300 CONTINUE
C
C             3.1   CONSISTENCY OF WIND SPEED AND DIRECTION
C
      CALL CDDFF (DD,FF,NDD,NFF)
C
C    --------------------------------------------------------------------
C
C                 4. ADD FLAGS TO BUFR RECORD
C                -------------------------------
C
  400 CONTINUE
      CALL UPFLAG (IERR,J,VALUES,JP22,JP1,M,IOFF)
      IF(IERR.NE.0) GO TO 500
C
  405 CONTINUE
C
C    --------------------------------------------------------------------
C
C                   5.  EXIT
C                  -----------
C
  500 CONTINUE
      RETURN
      END
      SUBROUTINE QCTOVS (K,IERR)
C
C**** *QCTOVS*
C
C       PURPOSE.
C      ----------
C
C          QUALITY CONTROL FOR TOVS
C
C       INTERFACE.
C      ------------
C
C        CALL QCTOVS (K,IERR)
C
C        INPUT
C         ARRAY VALUES (COMWT) - TOVS REPORTS
C         K      -  INDICATOR FOR TYPE OF TOVS
C
C        OUTPUT
C         ARRAY VALUES (COMWT) - TOVS REPORTS WITH CONFIDENCE FLAGS
C                                               ADDED
C         IERR   -  ERROR CODE
C
C       METHOD.
C      ---------
C
C        DETERMINE WHETHER LAYER MEAN TEMPERATURE OR PRECIPITABLE
C          WATER IS REPORTED;
C        CHECK UPPER AIR LEVELS AGAINST LIMITS.
C
C       EXTERNALS.
C      ------------
C
C        CUAT   -  GROSS LIMIT CHECK FOR UPPER AIR PARAMETERS
C        DERITA -  INITIALISE TABLES AND CONFIDENCE FLAGS
C        SEASON -  DETERMINE RELEVANT SEASON
C        SETGRS -  SET CONFIDENCE FLAGS FOR GROSS LIMIT CHECK
C        UPFLAG -  ADD CONFIDENCE FLAGS TO REPORT
C
C       REFERENCE.
C      ------------
C
C        GUIDE ON THE GLOBAL DATA PROCESSING SYSTEM WMO N305 1982
C          CHAPTER 6,  QUALITY CONTROL PROCEDURES
C
C       AUTHOR.
C      ---------
C
C         B. NORRIS,  ECMWF,  JUNE 1989.
C
C       MODIFICATIONS.
C      ----------------
C
C         NONE
C
      IMPLICIT LOGICAL (L,O,G)
      INCLUDE 'parameter.f'
      INCLUDE 'paramq.f'
      INCLUDE 'comwt.f'
      INCLUDE 'const.f'
      LOGICAL LTEMP
C
C    ---------------------------------------------------------------------------
C
C                      1.  SETUP
C                    --------------
C
  100 CONTINUE
C
C          1.1  POSITIONS OF PARAMETERS WITHIN LEVEL
C
      DATA NPP1/1/,NPP2/2/,NTM/5/,NWL/5/
C
C          1.2  POSITIONS OF PARAMETERS WITHIN HEADER
C
      IF(IERR.NE.0) RETURN
      IOFF=0
      NMONTH=5
      NRLAT=9
      NRLONG=10
C
C          1.3  SET STARTING POSITION OF LEVELS AND LENGTH OF LEVELS
C
      NILEV=18
      ILVSIZ=5
      NUMLEV=(M-NILEV)/ILVSIZ
C
C          1.4  DETERMINE WHETHER LAYER MEAN TEMPERATURE OR
C                    PRECIPITABLE WATER IS REPORTED
C
      IF(K.EQ.0.OR.K.EQ.1) THEN
           LTEMP=.TRUE.
      ELSEIF(K.EQ.2) THEN
           LTEMP=.FALSE.
      ELSE
           WRITE (*,'(1H ,''TOVS TYPE NOT RECOGNIZED:'',I5)') K
           GO TO 400
      ENDIF
C
C          1.5  LOOP FOR REPORTS WITHIN ARRAY 'VALUES'
C
      DO 305 J=1,NSUB
      IERR=0
C
C          1.6  EXTRACT HEADER VALUES
C
      MONTH=NINT(VALUES(NMONTH,J))
      RLAT =VALUES(NRLAT,J)
      RLONG=VALUES(NRLONG,J)
C
C          1.7   SET UP TABLES
C
      CALL DERITA (IERR,M)
      IF(IERR.NE.0) GO TO 400
C
C          1.8   DETERMINE SEASON
C
      CALL SEASON (RLAT,RLONG,MONTH)
      Z2=RMISS
C
C          1.9   LOOP FOR LEVELS
C
      DO 245 ILEV=1,NUMLEV
      IBASE=(ILEV-1)*ILVSIZ+NILEV
      NNPP1=IBASE+NPP1
      NNPP2=IBASE+NPP2
      IF(LTEMP) THEN
           NNTM=IBASE+NTM
      ELSE
           NNWL=IBASE+NWL
      ENDIF
C
C          1.10  EXTRACT PARAMETER VALUES
C
      PP1=VALUES(NNPP1,J)
      PP2=VALUES(NNPP2,J)
      IF(LTEMP) THEN
           TM=VALUES(NNTM,J)
      ELSE
           WL=VALUES(NNWL,J)
      ENDIF
C
C    --------------------------------------------------------------------
C
C                2.  CHECKING OF PARAMETER VALUES
C             -------------------------------------
C
  200 CONTINUE
C
C         2.1  UPPER AIR LEVEL
C
C              2.11  GROSS LIMIT FOR PRESSURE
C
      IF(PP1.NE.RMISS.AND.PP2.NE.RMISS) THEN
           IF(PP1.LT.0.0.OR.PP1.GT.108000.0) CALL SETGRS (NNPP1,1)
           IF(PP2.LT.0.0.OR.PP2.GT.108000.0) CALL SETGRS (NNPP2,1)
           IF(LTEMP) THEN
C
C              2.12  CHECK LAYER MEAN TEMPERATURE DEPENDING ON PRESSURE
C
                IF(TM.NE.RMISS) THEN
                     CALL CUAT (IERR,TM,'TM',NNTM,RLAT,RLONG,PP1,PP2,Z2,
     X                                           NNPP1,NNPP2,IMISS)
                     IF(IERR.NE.0) WRITE (*,4313) IERR,PP1,PP2,TM
                ENDIF
           ELSE
C
C              2.13  CHECK PRECIPITABLE WATER DEPENDING ON PRESSURE
C
                IF(WL.NE.RMISS) THEN
                     CALL CUAT (IERR,WL,'WL',NNWL,RLAT,RLONG,PP1,PP2,Z2,
     X                                           NNPP1,NNPP2,IMISS)
                     IF(IERR.NE.0) 
     x                 WRITE (*,4325) IERR,PP1,PP2,WL
                ENDIF
           ENDIF
      ENDIF
  245 CONTINUE
 4325 format(1h,'TOVS CUAT ERROR:',i5,' PP1:',f10.2,' PP2:',
     x       F10.2,' WL:',F10.2)
 4313 format(1h,'TOVS CUAT ERROR:',i5,' PP1:',f10.2,' PP2:',
     x       F10.2,' TM:',F10.2)    
C
C    --------------------------------------------------------------------
C
C                 3. ADD FLAGS TO BUFR RECORD
C                -------------------------------
C
  300 CONTINUE
      CALL UPFLAG (IERR,J,VALUES,JP22,JP1,M,IOFF)
      IF(IERR.NE.0) GO TO 400
C
  305 CONTINUE
C
C    --------------------------------------------------------------------
C
C                   4.  EXIT
C                  -----------
C
  400 CONTINUE
      RETURN
      END
      SUBROUTINE SEASON (RLAT,RLONG,MONTH)
C
C**** *SEASON*
C
C       PURPOSE.
C      ----------
C
C           DETERMINE SEASON
C
C       INTERFACE.
C      ------------
C
C           CALL SEASON (RLAT,RLONG,MONTH)
C
C        INPUT
C         RLAT   -  LATITUDE VALUE FOR REPORT
C         RLONG  -  LONGITUDE VALUE FOR REPORT
C         MONTH  -  MONTH FOR REPORT
C
C        OUTPUT
C         ISEASN (CONST) - SEASON (1=WINTER,2=SUMMER)
C
C       METHOD.
C      ---------
C
C        SUMMER IS ASSUMED IF NORTH OF EQUATOR, APRIL TO SEPTEMBER OR
C                             SOUTH OF EQUATOR, OCTOBER TO MARCH,
C           OTHERWISE WINTER.
C
C       EXTERNALS.
C      ------------
C
C        NONE
C
C       REFERENCE.
C      ------------
C
C        NONE
C
C       AUTHOR.
C      ---------
C
C        B. NORRIS,  ECMWF,  JANUARY 1989.
C
C       MODIFICATIONS.
C      ----------------
C
C        NONE
C
      INCLUDE 'const.f'
C
      ISEASN=1
      IF(RLAT.GT.0.0.AND.MONTH.GE.4.AND.MONTH.LE.9) ISEASN=2
      IF(RLAT.LT.0.0.AND.(MONTH.GE.10.OR.MONTH.LE.3)) ISEASN=2
C
      RETURN
      END
      SUBROUTINE SETFLG (IF,IP)
C
C**** *SETFLG*
C
C       PURPOSE.
C      ----------
C
C         UPDATE FLAGS AND FLAG COUNTERS (INTERNAL CONSISTENCY CHECK)
C
C       INTERFACE.
C      ------------
C
C        CALL SETFLG (IF,IP)
C
C        INPUT
C         IF  -  'PASS' OR 'FAIL' INDICATOR
C         IP  -  POSITION OF CONFIDENCE FLAG
C
C        OUTPUT
C         IFLAG (CONF) - COUNTERS AND CONFIDENCE FLAGS UPDATED:
C           IFLAG(1,--) = CONFIDENCE FLAG
C           IFLAG(2,--) = NUMBER OF FAILS
C           IFLAG(3,--) = NUMBER OF PASSES
C
C       METHOD.
C      ---------
C
C        INCREMENT RELEVANT COUNTER;
C        UPDATE CONFIDENCE FLAG WITH A VALUE DEPENDING ON NUMBER OF
C        PREVIOUS FAILS AND PASSES.
C
C       EXTERNALS.
C      ------------
C
C        NONE.
C
C       REFERENCE.
C      ------------
C
C        NONE
C
C       AUTHOR.
C      ---------
C
C        B. NORRIS,  ECMWF,  JANUARY 1989.
C
C       MODIFICATIONS.
C      ----------------
C
C        NONE
C
      INCLUDE 'paramq.f'
      INCLUDE 'conf.f'
      INCLUDE 'const.f'
C
C    -------------------------------------------------------------------
C
C                1.  UPDATE CONFIDENCE AND COUNTERS.
C               -------------------------------------
C
  100 CONTINUE
      IF(IP.EQ.IMISS) GO TO 200
C
C          1.1  PASS
C
      IF(IF.EQ.IPASS) THEN
          IFLAG(3,IP)=IFLAG(3,IP)+1
          IF(IFLAG(3,IP).LE.NCOCO) IFLAG(1,IP)=IFLAG(1,IP)
     X      +ICOCO(IFLAG(3,IP),2)
      ENDIF
C
C          1.2  FAIL
C
      IF(IF.EQ.IFAIL) THEN
          IFLAG(2,IP)=IFLAG(2,IP)+1
          IF(IFLAG(2,IP).LE.NCOCO) IFLAG(1,IP)=IFLAG(1,IP)
     X      +ICOCO(IFLAG(2,IP),1)
      ENDIF
C
C    ----------------------------------------------------------------------
C
C                   2.  EXIT.
C                  -----------
C
  200 CONTINUE
      RETURN
      END
      SUBROUTINE SETGRS (IP,N)
C
C**** *SETGRS*
C
C       PURPOSE.
C      ----------
C
C         SET FLAGS FOR GROSS LIMIT CHECK
C
C       INTERFACE.
C      ------------
C
C        CALL SETGRS (IP,N)
C
C        INPUT
C         IP  -  POSITION OF CONFIDENCE FLAG
C         N   -  SEVERITY OF FAILURE (1 = IMPOSSIBLE VALUE,
C                                     2 = OUTSIDE MIN2/MAX2,
C                                     3 = BETWEEN MAX1 AND MAX2 ETC,
C                                     4 = BETWEEN MIN1 AND MAX1)
C
C        OUTPUT
C         IFLAG (CONF) - COUNTER AND CONFIDENCE VALUE UPDATED
C
C       METHOD.
C      ---------
C
C        UPDATE RELAVANT COUNTER AND CONFIDENCE DEPENDING ON
C          DEGREE OF PASS/FAIL
C
C       EXTERNALS.
C      ------------
C
C        NONE.
C
C       AUTHOR.
C      ---------
C
C        B. NORRIS,  ECMWF,  JANUARY 1989.
C
C       MODIFICATIONS.
C      ----------------
C
C        NONE.
C
      INCLUDE 'paramq.f'
      INCLUDE 'conf.f'
      INCLUDE 'const.f'
C
C    ---------------------------------------------------------------------
C
C                 1.  UPDATE COUNTERS AND CONFIDENCE
C                ------------------------------------
C
  100 CONTINUE
      IF(ICBO(N,2).EQ.IPASS) THEN
C
C           1.1  PASS COUNTER
C
           IFLAG(3,IP)=IFLAG(3,IP)+1
      ELSEIF (ICBO(N,2).EQ.IFAIL) THEN
C
C           1.2  FAIL COUNTER
C
           IFLAG(2,IP)=IFLAG(2,IP)+1
      ELSE
C
      ENDIF
C
C           1.3  CONFIDENCE VALUE
C
      IFLAG(1,IP)=IFLAG(1,IP)+ICBO(N,1)
C
C    -------------------------------------------------------------------
C
C                   2.  EXIT
C                  -----------
C
  200 CONTINUE
      RETURN
      END
      SUBROUTINE SETINT (IF,IP1,IP2,IP3,IP4,IP5,IP6,IP7,IP8,IP9,IP10)
C
C**** *SETINT*
C
C       PURPOSE.
C      ---------
C
C          SET FLAGS FOR A NUMBER OF PARAMETERS,
C               INTERNAL CONSISTENCY CHECKS
C
C       INTERFACE.
C      ------------
C
C        CALL SETINT (IF,IP1,IP2,IP3,IP4,IP5,IP6,IP7,IP8,IP9,IP10)
C
C        INPUT
C         IF  -  'PASS' OR 'FAIL' INDICATOR
C         IP- -  POSITIONS OF CONFIDENCE FLAGS
C                IF IP- = 'END' MARKER, NO FURTHER ARGUMENTS ARE
C                 CONSIDERED
C
C        OUTPUT
C         IFLAG (CONF) - COUNTERS AND CONFIDENCE FLAGS UPDATED
C
C       METHOD.
C      ---------
C
C        FOR EACH POSITION, CALL SETFLG UNTIL 'END' MARKER IS REACHED.
C
C       EXTERNALS.
C      ------------
C
C        SETFLG  -  UPDATE COUNTERS AND CONFIDENCE FLAGS
C
C       REFERENCE.
C      ------------
C
C        NONE
C
C       AUTHOR.
C      ---------
C
C        B. NORRIS,  ECMWF,  JANUARY 1989.
C
C       MODIFICATIONS.
C      ----------------
C
C        NONE
C
      INCLUDE 'paramq.f'
      INCLUDE 'conf.f'
      INCLUDE 'const.f'
C
C    --------------------------------------------------------------------
C
C                 1.  SET FLAGS FOR EACH PARAMETER
C                -----------------------------------
C
  100 CONTINUE
      IF(IP1.EQ.IEND) GO TO 200
      CALL SETFLG (IF,IP1)
      IF(IP2.EQ.IEND) GO TO 200
      CALL SETFLG (IF,IP2)
      IF(IP3.EQ.IEND) GO TO 200
      CALL SETFLG (IF,IP3)
      IF(IP4.EQ.IEND) GO TO 200
      CALL SETFLG (IF,IP4)
      IF(IP5.EQ.IEND) GO TO 200
      CALL SETFLG (IF,IP5)
      IF(IP6.EQ.IEND) GO TO 200
      CALL SETFLG (IF,IP6)
      IF(IP7.EQ.IEND) GO TO 200
      CALL SETFLG (IF,IP7)
      IF(IP8.EQ.IEND) GO TO 200
      CALL SETFLG (IF,IP8)
      IF(IP9.EQ.IEND) GO TO 200
      CALL SETFLG (IF,IP9)
      IF(IP10.EQ.IEND) GO TO 200
      CALL SETFLG (IF,IP10)
C
C    --------------------------------------------------------------------
C
C                      2.  EXIT
C                    ------------
C
  200 CONTINUE
      RETURN
      END
      SUBROUTINE SETABS (IF,IP1,IP2,IP3,IP4,IP5,IP6,IP7,IP8,IP9,IP10)
C
C**** *SETABS*
C
C       PURPOSE.
C      ---------
C
C          SET CONFIDENCE FOR A NUMBER OF PARAMETERS
C
C       INTERFACE.
C      ------------
C
C        CALL SETABS (IF,IP1,IP2,IP3,IP4,IP5,IP6,IP7,IP8,IP9,IP10)
C
C        INPUT
C         IF  -  CONFIDENCE VALUE TO BE APPLIED
C         IP- -  POSITIONS OF CONFIDENCE FLAGS
C                IF IP- = 'END' MARKER, NO FURTHER ARGUMENTS ARE
C                 CONSIDERED
C
C        OUTPUT
C         IFLAG (CONF) - CONFIDENCE FLAGS UPDATED
C
C       METHOD.
C      ---------
C
C        FOR EACH POSITION, SET CONFIDENCE UNTIL 'END' MARKER IS REACHED.
C
C       EXTERNALS.
C      ------------
C
C        NONE
C
C       REFERENCE.
C      ------------
C
C        NONE
C
C       AUTHOR.
C      ---------
C
C        B. NORRIS,  ECMWF,  MARCH 1990.
C
C       MODIFICATIONS.
C      ----------------
C
C        NONE
C
      INCLUDE 'paramq.f'
      INCLUDE 'conf.f'
      INCLUDE 'const.f'
C
C    --------------------------------------------------------------------
C
C                 1.  SET FLAGS FOR EACH PARAMETER
C                -----------------------------------
C
  100 CONTINUE
      IF(IP1.EQ.IEND) GO TO 200
      IFLAG(1,IP1)=IF
      IF(IP2.EQ.IEND) GO TO 200
      IFLAG(1,IP2)=IF
      IF(IP3.EQ.IEND) GO TO 200
      IFLAG(1,IP3)=IF
      IF(IP4.EQ.IEND) GO TO 200
      IFLAG(1,IP4)=IF
      IF(IP5.EQ.IEND) GO TO 200
      IFLAG(1,IP5)=IF
      IF(IP6.EQ.IEND) GO TO 200
      IFLAG(1,IP6)=IF
      IF(IP7.EQ.IEND) GO TO 200
      IFLAG(1,IP7)=IF
      IF(IP8.EQ.IEND) GO TO 200
      IFLAG(1,IP8)=IF
      IF(IP9.EQ.IEND) GO TO 200
      IFLAG(1,IP9)=IF
      IF(IP10.EQ.IEND) GO TO 200
      IFLAG(1,IP10)=IF
C
C    --------------------------------------------------------------------
C
C                      2.  EXIT
C                    ------------
C
  200 CONTINUE
      RETURN
      END
      SUBROUTINE SETSF (P,IP,VLIM)
C
C**** *SETSF*
C
C      PURPOSE.
C     ---------
C
C           SET CONFIDENCE FLAG FOR GROSS LIMIT CHECK DEPENDING
C                 UPON SEVERITY OF FAILURE
C
C
C      INTERFACE.
C     ------------
C
C         CALL SETSF (P,IP,VLIM)
C
C      INPUT
C        P      -  VALUE OF PARAMETER
C        IP     -  POSITION OF CONFIDENCE FLAG FOR PARAMETER
C        VLIM   -  GROSS LIMITS
C
C      OUTPUT
C        IFLAG (CONF)  -  CONFIDENCE FLAG FOR PARAMETER UPDATED
C
C      METHOD.
C     ---------
C
C        DETERMINE WHERE PARAMETER VALUE FALLS WITHIN GROSS LIMITS
C        UPDATE CONFIDENCE ACCORDINGLY
C
C      EXTERNALS.
C     ------------
C
C         SETGRS  -  UPDATE CONFIDENCE FLAG FOR GROSS LIMIT CHECK
C
C      REFERENCE.
C     ------------
C
C         GUIDE ON THE GLOBAL DATA PROCESSING SYSTEM WMO N305 1982
C           CHAPTER 6, QUALITY CONTROL PROCEDURES
C
C      AUTHOR.
C     ---------
C
C         B. NORRIS,  ECMWF,  FEBRUARY 1989.
C
C      MODIFICATIONS.
C     -----------------
C
C         NONE
C
      INCLUDE 'paramq.f'
C
      DIMENSION VLIM(NMM)
C
C    -------------------------------------------------------------------
C
C              1.  SET APPROPRIATE FLAGS
C             ---------------------------
C
  100 CONTINUE
      IF(P.LT.VLIM(1)) THEN
C
C          1.1  P<MIN2
C
           CALL SETGRS (IP,2)
      ELSEIF(P.GE.VLIM(1).AND.P.LT.VLIM(2)) THEN
C
C          1.2  MIN2<P<MIN1
C
           CALL SETGRS (IP,3)
      ELSEIF(P.GE.VLIM(2).AND.P.LE.VLIM(3)) THEN
C
C          1.3  MIN1<P<MAX1
C
           CALL SETGRS (IP,4)
      ELSEIF(P.GT.VLIM(3).AND.P.LE.VLIM(4)) THEN
C
C          1.4  MAX1<P<MAX2
C
           CALL SETGRS (IP,3)
      ELSEIF(P.GT.VLIM(4)) THEN
C
C          1.5  P>MAX2
C
           CALL SETGRS (IP,2)
      ELSE
      ENDIF
C
C    ------------------------------------------------------------------
C
C                   2.    EXIT
C                 --------------
C
  200 CONTINUE
      RETURN
      END
      SUBROUTINE UPFLAG (IERR,J,VALUE,IDIM1,IDIM2,M,IOFF)
C
C**** *UPFLAG*
C
C       PURPOSE.
C      ----------
C
C        INSERT CONFIDENCE VALUES IN BUFR MESSAGE
C
C       INTERFACE.
C      ------------
C
C        CALL UPFLAG (IERR,J,VALUE,IDIM1,IDIM2,M,IOFF)
C
C        INPUT
C         J     - NUMBER OF REPORT WITHIN ARRAY 'VALUE'
C         VALUE - ARRAY OF BUFR MESSAGES
C         IDIM1,IDIM2 -  DIMENSIONS OF ARRAY VALUE
C         M     - NUMBER OF ENTRIES IN EACH REPORT
C         IOFF  - OFFSET WITHIN CONFIDENCE FLAG ARRAY 'IFLAG'
C
C        OUTPUT
C         IERR  - ERROR CODE
C         VALUE - CONFIDENCE FLAGS ADDED, AREA FOR SUBSTITUTIONS
C                     SET TO MISSING
C
C       METHOD.
C      ---------
C
C        ADD CONFIDENCE VALUES TO END OF REPORT;
C        INITIALISE AREA FOR SUBSTITUTIONS TO MISSING
C
C       EXTERNALS.
C      ------------
C
C        NONE
C
C       REFERENCE.
C      ------------
C
C        NONE
C
C       AUTHOR.
C      ---------
C
C        B. NORRIS,  ECMWF,  JANUARY 1989.
C
C       MODIFICATIONS.
C      ----------------
C
C        B. NORRIS, 90/02/20, SET AREA FOR SUBSTITUTIONS TO MISSING
C
      INCLUDE 'paramq.f'
      INCLUDE 'const.f'
      INCLUDE 'conf.f'
      DIMENSION VALUE (IDIM1,IDIM2)
C
      IERR=0
C
C    --------------------------------------------------------------------
C
C             1.  CHECK WHETHER ARRAY IS LARGE ENOUGH
C            -----------------------------------------
C
  100 CONTINUE
      IF(M*3.GT.IDIM1) THEN
           WRITE (*,'(1H ,'' VALUES ARRAY TOO SMALL, NUMBER OF ENTRIES R
     XEQUIRED:'',I7)') M*3
           IERR=2501
           GO TO 300
      ENDIF
C
C    ------------------------------------------------------------------------
C
C             2.  ADD CONFIDENCE VALUES TO RECORD, SUBJECT TO LIMITS
C            --------------------------------------------------------
C
  200 CONTINUE
      DO 202 I=1,M
      VALUE(M+I,J)=IFLAG(1,IOFF+I)
      IF(VALUE(M+I,J).LT.0.0)   VALUE(M+I,J)=0.0
      IF(VALUE(M+I,J).GT.100.0) VALUE(M+I,J)=100.0
      IF(VALUE(I,J).EQ.RMISS)   VALUE(M+I,J)=RMISS
      VALUE(2*M+I,J)=RMISS
  202 CONTINUE
C
C    -----------------------------------------------------------------------------
C
C                     3.  EXIT
C                   ------------
C
  300 CONTINUE
      RETURN
      END
      SUBROUTINE WSHEAR (IERR,NSTD,STDPP,STDDD,STDFF,
     X                                   ISTDDD,ISTDFF)
C
C**** *WSHEAR*
C
C       PURPOSE.
C      ----------
C
C           CHECK CONSECUTIVE STANDARD LEVELS OF UPPER AIR
C             REPORT FOR EXCESSIVE WIND SHEAR
C
C       INTERFACE.
C      ------------
C
C         CALL WSHEAR (IERR,NSTD,STDPP,STDDD,STDFF,ISTDDD,ISTDFF)
C
C        INPUT
C         NSTD   -  NUMBER OF STANDARD LEVELS
C         STDPP  -  STANDARD LEVEL PRESSURE VALUES
C         STDDD  -  CORRESPONDING STANDARD LEVEL WIND DIRECTION VALUES
C         STDFF  -  CORRESPONDING STANDARD LEVEL WIND SPEED VALUES
C         ISTDDD -  CORRESPONDING STANDARD LEVEL WIND DIR FLAG POSITIONS
C         ISTDFF -  CORRESPONDING STANDARD LEVEL WIND SPD FLAG POSITIONS
C
C        OUTPUT
C         IERR   -  ERROR CODE
C         IFLAG (CONF)  -  CONFIDENCE FLAGS UPDATED
C
C       METHOD.
C      ---------
C
C           CHECKS THE SPEED SHEAR AND DIRECTIONAL SHEAR OF WIND
C           BETWEEN 2 STANDARD LEVELS.
C           MAXIMUM SPEED SHEAR PERMITTED IS 20.6+0.275(F1+F2) M/S
C           MAXIMUM PERMITTED SUM OF SPEEDS FOR PARTICULAR DIRECTIONAL
C           SHEAR IS GIVEN BY A TABLE.
C
C       EXTERNALS.
C      -----------
C
C         SETINT  -  SET CONFIDENCE FLAGS FOR INTERNAL CONSISTENCY CHECK
C         PINTER  -  INTERPOLATE TO PRESSURE OR HEIGHT LEVEL
C
C       REFERENCE.
C      ------------
C
C         NONE
C
C       AUTHOR.
C      ---------
C
C         B. NORRIS,  ECMWF,  FEBRUARY 1989.
C
C       MODIFICATIONS.
C      ----------------
C
C         NONE
C
      INCLUDE 'paramq.f'
      PARAMETER (NSCOL=7)
      INCLUDE 'ctua.f'
      INCLUDE 'const.f'
      DIMENSION STDPP(*),STDDD(*),STDFF(*)
      DIMENSION ISTDDD(*),ISTDFF(*)
      DIMENSION IPOS(NLEV),USH(NSCOL,NLEV)
      LOGICAL LDIR,LSPD
C
C    -------------------------------------------------------------------
C
C                       1.  SETUP
C                     ------------
C
  100 CONTINUE
C
C           1.1  LIMIT TABLE FOR SUM OF SPEEDS FOR CERTAIN
C                      DIRECTIONAL SHEAR
C
      DATA ((USH(I,J),I=1,NSCOL),J=1,10)/
C                   DIRECTIONAL SHEAR (DEGREES)
C         >30      >40      >50      >60      >70      >80      >90
     X   72.0,    61.0,    57.0,    53.0,    49.0,    46.0,    41.0,
     X   72.0,    61.0,    57.0,    53.0,    49.0,    46.0,    41.0,
     X   72.0,    61.0,    57.0,    53.0,    49.0,    46.0,    41.0,
     X  110.0,    84.0,    77.0,    70.0,    63.0,    52.0,    50.0,
     X  110.0,    84.0,    77.0,    70.0,    63.0,    52.0,    50.0,
     X  110.0,    84.0,    77.0,    70.0,    63.0,    52.0,    50.0,
     X  110.0,    84.0,    77.0,    70.0,    63.0,    52.0,    50.0,
     X  110.0,    84.0,    77.0,    70.0,    63.0,    52.0,    50.0,
     X  110.0,    84.0,    77.0,    70.0,    63.0,    52.0,    50.0,
     X   72.0,    61.0,    57.0,    53.0,    49.0,    46.0,    41.0/
      DATA ((USH(I,J),I=1,NSCOL),J=11,NLEV)/
     X   72.0,    61.0,    57.0,    53.0,    49.0,    46.0,    41.0,
     X   72.0,    61.0,    57.0,    53.0,    49.0,    46.0,    41.0,
     X   72.0,    61.0,    57.0,    53.0,    49.0,    46.0,    41.0,
     X   72.0,    61.0,    57.0,    53.0,    49.0,    46.0,    41.0,
     X   72.0,    61.0,    57.0,    53.0,    49.0,    46.0,    41.0,
     X   72.0,    61.0,    57.0,    53.0,    49.0,    46.0,    41.0,
     X   72.0,    61.0,    57.0,    53.0,    49.0,    46.0,    41.0,
     X   72.0,    61.0,    57.0,    53.0,    49.0,    46.0,    41.0,
     X   72.0,    61.0,    57.0,    53.0,    49.0,    46.0,    41.0,
     X   72.0,    61.0,    57.0,    53.0,    49.0,    46.0,    41.0,
     X   72.0,    61.0,    57.0,    53.0,    49.0,    46.0,    41.0,
     X   72.0,    61.0,    57.0,    53.0,    49.0,    46.0,    41.0/
C
C        1.2  OBTAIN POSITION OF EACH LEVEL WITHIN TABLE OF
C                       STANDARD LEVELS
C
      DO 122 ILEV=1,NSTD
      CALL PINTER (IERR,UPZ(1,1),STDPP(ILEV),POS)
      IF(IERR.NE.0) THEN
           IPOS(ILEV)=1000
      ELSE
           IPOS(ILEV)=NINT(POS)
      ENDIF
  122 CONTINUE
C
C    -------------------------------------------------------------------
C
C           2.  CALCULATE LIMITS FOR SHEAR
C         ----------------------------------
C
  200 CONTINUE
      IERR=0
C
C         2.1  LOOP FOR STANDARD LEVELS
C
      DO 245 ILEV=1,NSTD-1
      LDIR=.FALSE.
      LSPD=.FALSE.
      IF(IABS(IPOS(ILEV)-IPOS(ILEV+1)).LE.1) THEN
C
C         2.2  SPEED SHEAR
C
           SUMWIN=STDFF(ILEV)+STDFF(ILEV+1)
           SHEWIN=ABS(STDFF(ILEV)-STDFF(ILEV+1))
           SHEMAX=20.6+0.275*SUMWIN
           IF(SHEWIN.GT.SHEMAX) LSPD=.TRUE.
C
C         2.3  DIRECTIONAL SHEAR
C
           IF(STDDD(ILEV).EQ.0.0.OR.STDDD(ILEV+1).EQ.0.0) GO TO 235
           DIRSHE=ABS(STDDD(ILEV)-STDDD(ILEV+1))
           IF(DIRSHE.GT.180.0) DIRSHE=360.0-DIRSHE
           IND=(DIRSHE-20.0)/10.0
           IF(IND.LT.1) GO TO 235
           IF(IND.GT.NSCOL) IND=NSCOL
           SUMPER=USH(IND,IPOS(ILEV))
           IF(SUMWIN.GT.SUMPER) LDIR=.TRUE.
  235      CONTINUE
C
C         2.4  UPDATE CONFIDENCE FLAGS
C
           IF(LDIR.OR.LSPD) CALL SETINT
     X           (IFAIL,ISTDDD(ILEV),ISTDFF(ILEV),
     X                 ISTDDD(ILEV+1),ISTDFF(ILEV+1),IEND)
      ENDIF
  245 CONTINUE
C
C    --------------------------------------------------------------------
C
C                   3.   EXIT
C                  -----------
C
  300 CONTINUE
      RETURN
      END
      SUBROUTINE TVIRT (PP,TT,TD,TV)
C
C**** *TVIRT*
C
C     PURPOSE.
C     --------
C
C        CALCULATE VIRTUAL TEMPERATURE
C
C     INTERFACE.
C     ----------
C
C      CALL TVIRT (PP,TT,TD,TV)
C
C     INPUT
C            PP  -  PRESSURE (PA)
C            TT  -  TEMPERATURE  (DEG K)
C            TD  -  DEW-POINT TEMPERATURE  (DEG K)
C     OUTPUT
C            TV  -  VIRTUAL TEMPERATURE  (DEG K)
C
C     METHOD.
C     -------
C
C         CALCULATE VIRTUAL TEMPERATURE GIVEN PRESSURE, TEMPERATURE
C         AND DEWPOINT.
C
C     EXTERNALS.
C     ----------
C
C        NONE
C
C     REFERENCE.
C     ----------
C
C        NONE
C
C     AUTHOR.
C     -------
C
C        B. NORRIS,  ECMWF,  DECEMBER 1989.
C
C     MODIFICATIONS.
C     --------------
C
C        NONE
C
C
C    ------------------------------------------------------------------
C
C                     1.  PREPARATION
C                     ---------------
C
  100 CONTINUE
      DATA CO/231.82582/
      DATA EL/5418.118466/
      DATA TO/3.660322E-3/
C
C    -------------------------------------------------------------------
C
C                        2.  CALCULATION
C                       -----------------
C
  200 CONTINUE
      TV=TT*(1.+CO*EXP(EL*(TO-1./TD))/PP)
C
C    --------------------------------------------------------------------
C
C                        3.  EXIT
C                        --------
C
  300 CONTINUE
      RETURN
      END
      SUBROUTINE ZCOMP (IERR,NST,PP,TV,IVS,Z,NNZ,ZN,ZSTN,
     X                  CALLSN,IY,IM,ID,IH,IN)
C
C**** *ZCOMP*
C
C     PURPOSE.
C     --------
C
C        COMPARE STANDARD LEVEL HEIGHTS WITH HEIGHTS RECOMPUTED
C         FROM SIGNIFICANT TEMPERATURE LEVELS
C
C     INTERFACE.
C     ----------
C
C      CALL ZCOMP (IERR,NST,PP,TV,IVS,Z,NNZ,ZN,ZSTN,
C                  CALLSN,IY,IM,ID,IH,IN)
C
C     INPUT
C            NST -  NUMBER OF TEMPERATURE LEVELS
C            PP  -  PRESSURE (PA)
C            TV  -  VIRTUAL TEMPERATURE  (DEG K)
C            IVS -  VERTICAL SIGNIFICANCE
C            Z   -  HEIGHT (M)
C            NNZ -  FLAG POSITIONS FOR HEIGHT
C            ZSTN-  HEIGHT OF STATION
C            CALLSN  -  STATION NUMBER OR CALL SIGN
C            IY,IM,ID,IH,IN - YEAR,MONTH,DAY,HOUR,MINUTE OF REPORT
C     OUTPUT
C            IERR-  ERROR CODE
C            ZN  -  COMPUTED HEIGHT (M)
C
C     METHOD.
C     -------
C
C         LOOK FOR SURFACE LEVEL;
C         FOR ALL OTHER LEVELS, CALCULATE HEIGHT FROM NEXT LOWER
C         SIGNIFICANT LEVEL. IF STANDARD LEVEL, COMPARE OBSERVED
C         HEIGHT WITH RECOMPUTED HEIGHT.
C         CALCULATION OF HEIGHTS TERMINATES IF VIRTUAL TEMPERATURE
C         IS MISSING, OR LAYER IS TOO DEEP, OR PART B OR PART D
C         IS MISSING.
C
C     EXTERNALS.
C     ----------
C
C        NONE
C
C     REFERENCE.
C     ----------
C
C        GUIDE ON THE GLOBAL DATA PROCESSING SYSTEM WMO N305 1982
C           CHAPTER 6, QUALITY CONTROL PROCEDURES
C
C     AUTHOR.
C     -------
C
C        B. NORRIS,  ECMWF,  DECEMBER 1989.
C
C     MODIFICATIONS.
C     --------------
C
C        NONE
C
      INCLUDE 'paramq.f'
      INCLUDE 'const.f'
      INCLUDE 'conf.f'
      DIMENSION PP(*),TV(*),IVS(*),Z(*),NNZ(*),ZN(*)
      CHARACTER*(*) CALLSN
      LOGICAL LB,LD
C
C    ------------------------------------------------------------------
C
C                     1.  INITIALIZATION
C                    --------------------
C
  100 CONTINUE
      IERR=0
      LB=.FALSE.
      LD=.FALSE.
C
C          1.1  CHECK THERE IS MORE THAN 1 LEVEL;
C                LOOK FOR SURFACE LEVEL
C
      IF(NST.LE.1) GO TO 300
      DO 112 ILEV=1,NST
      IF((IVS(ILEV).AND.64).EQ.64) THEN
           NLSIG=ILEV
           ZN(NLSIG)=ZSTN
           GO TO 114
      ENDIF
  112 CONTINUE
      GO TO 300
  114 CONTINUE
      IF(NLSIG.EQ.NST) GO TO 300
      ILOW=NLSIG+1
C
C           1.2  LOOK FOR PART B AND PART D
C
      DO 122 ILEV=1,NST
      IF((IVS(ILEV).AND. 4).EQ.4.AND.
     X   (IVS(ILEV).AND.64).EQ.0.AND.
     X   (IVS(ILEV).AND.16).EQ.0.AND.
     X   PP(ILEV).GE.10000.0)  LB=.TRUE.
      IF((IVS(ILEV).AND. 4).EQ.4.AND.
     X   (IVS(ILEV).AND.16).EQ.0.AND.
     X   PP(ILEV).LT.10000.0)  LD=.TRUE.
  122 CONTINUE
      IF(.NOT.LB) GO TO 300
C
C    -------------------------------------------------------------------
C
C                  2.  CALCULATE HEIGHTS
C                 -----------------------
C
  200 CONTINUE
      DO 245 ILEV=ILOW,NST
C
C          2.01  CHECK THAT PART D EXISTS
C
      IF(.NOT.LD.AND.PP(ILEV).LT.10000.0) GO TO 300
C
C          2.1  CHECK THAT VIRTUAL TEMPERATURE EXISTS
C
      IF(TV(NLSIG).EQ.RMISS.OR.TV(ILEV).EQ.RMISS) GO TO 300
C
C          2.2  CHECK DEPTH OF LAYER
C
      DP=PP(NLSIG)-PP(ILEV)
      DLIM=PP(NLSIG)*0.3+2000.0
      IF(DP.GT.DLIM) GO TO 300
C
C         2.3  CALCULATE HEIGHT FOR LEVEL
C
      DZ=(R/RG)*((TV(NLSIG)+TV(ILEV))/2.0)*ALOG(PP(NLSIG)/PP(ILEV))
      ZN(ILEV)=ZN(NLSIG)+DZ
      IF((IVS(ILEV).AND.4).EQ.4) NLSIG=ILEV
C
C         2.4  COMPARE OBSERVED HEIGHT WITH RECOMPUTED HEIGHT
C
C     WRITE (*,'(1H ,''PP;Z;ZN;TV: '',4F12.2)')
C    X         PP(ILEV),Z(ILEV),ZN(ILEV),TV(ILEV)
      IF(Z(ILEV).NE.RMISS) THEN
           TOL=20.0
           IF(ZN(ILEV).GE.6000.0) TOL=30.0
           IF(ZN(ILEV).GE.15000.0) TOL=40.0
           IF(ABS(Z(ILEV)-ZN(ILEV)).GT.TOL) THEN
                CALL SETGRS (NNZ(ILEV),2)
C                WRITE (*,'(1H ,'' A COMPARED TO B     '',
C     X               A,2X,I4,4I2.2,3(A,F10.2))')
C     X               CALLSN,IY,IM,ID,IH,IN,
C     X               ' PP:',PP(ILEV),', ZA:',Z(ILEV),', ZB:',ZN(ILEV)
           ELSE
                CALL SETINT (IPASS,NNZ(ILEV),IEND)
           ENDIF
      ENDIF
  245 CONTINUE
C
C    --------------------------------------------------------------------
C
C                        3.  EXIT
C                        --------
C
  300 CONTINUE
      RETURN
      END
      SUBROUTINE POSCHK (CFNAME,CALLSN,IUNIQ,CUNIQ,JTYPE,JSBTYPE,
     X     IY,IM,ID,IH,IN,RLAT,RLON,LANSEA,LSHIP,IFLAG,IRET)
C
C****  *SHIPCHK*
C
C       PURPOSE.
C      ----------
C
C           MASTER ROUTINE FOR TIME CONTINUITY CHECKING
C
C       INTERFACE.
C      ------------
C
C           CALL POSCHK (CFNAME,CALLSN,IUNIQ,CUNIQ,JTYPE,JSBTYPE,
C                IY,IM,ID,IH,IN,RLAT,RLON,LANSEA,LSHIP,IFLAG,IRET)
C
C        INPUT
C          CFNAME  -  NAME OF TIME CONTINUITY FILE
C          CALLSN  -  CALL SIGN OF CURRENT REPORT
C          IUNIQ   -  NUMBER OF NON-UNIQUE CALL SIGNS
C          CUNIQ   -  LIST OF NON-UNIQUE CALL SIGNS
C          JTYPE   -  BUFR TYPE FOR CURRENT REPORT
C          JSBTYPE -  BUFR SUBTYPE FOR CURRENT REPORT
C          IY,IM,ID,IH,IN  -  YEAR, MONTH, DAY, HOUR, MINUTE OF
C                     CURRENT REPORT
C          RLAT,RLON       -  LATITUDE AND LONGITUDE OF CURRENT REPORT
C          LANSEA  -  LOGICAL FOR LAND SEA MASK CHECK
C          LSHIP   -  
C
C        OUTPUT
C          IFLAG   -  CONFIDENCE APPLIED TO POSITION
C          IRET    -  RETURN CODE
C
C       METHOD.
C      ---------
C
C         CHECK WHETHER CALL SIGN IS UNIQUE;
C         CHECK WHETHER POSITION IS OVER LAND;
C         COMPARE CURRENT POSITION WITH PREVIOUS POSITION;
C         WRITE UPDATED RECORD TO TIME SERIES FILE.
C
C       EXTERNALS.
C      ------------
C
C         ISTIME   -  GET NUMBER OF MINUTES SINCE 1 JAN 1978
C         SHIPUP   -  UNPACK TIME SERIES RECORD
C         SHIPPK   -  PACK TIME SERIES RECORD
C         SHIPAD   -  ADD CURRENT INFORMATION TO TIME SERIES RECORD
C         COMPOS   -  COMPARE CURRENT POSITION WITH PREVIOUS POSITIONS
C         LANSEA   -  CHECK WHETHER POSITION IS OVER LAND
C
C       REFERENCE.
C      ------------
C
C         NONE
C
C       AUTHOR.
C      ---------
C
C         B. NORRIS,  ECMWF,  MARCH 1990
C         BASED ON CODE WRITTEN FOR NOS/BE IN 1982
C
C       MODIFICATIONS.
C      ----------------
C
C         NONE
C
      IMPLICIT LOGICAL (L,O,G)
      INCLUDE 'paramsh.f'
      INCLUDE 'comship.f'
      INCLUDE 'compref.f'
      INCLUDE 'comlsm.f'
      INCLUDE 'const.f'
      CHARACTER*3 CMONTH(12)
      CHARACTER*23 CT
      CHARACTER*(*) CALLSN,CFNAME
      CHARACTER*(*)   CUNIQ(IUNIQ)
      DIMENSION IREC(512)
      LOGICAL LCHECK,LREP,LANSEA,LSHIP,LUNIQ,OVERLND
c
      structure/track/
      character*9 yident
      integer*4   kslen
      integer*4   ksent
      integer*4   kspar
      integer*4   kinnt
      integer*4   krec(512)
      end structure
      record /track/ record
c
      character*9 aux_key
c
C    --------------------------------------------------------------------
C
C                      1.  INITIALISATION.
C                     ---------------------
C
  100 CONTINUE
C
C          1.1  SET REFERENCE VALUES AND SCALING FACTORS
C
C       JREF = REFERENCE VALUE FOR PACKING LAT AND LONG
      DATA JREF/-9000000,-18000000/
C       JSCAL = SCALING FACTOR FOR PACKING LAT AND LONG
      DATA JSCAL/5,5/
      DATA CMONTH/'JAN','FEB','MAR','APR','MAY','JUN',
     X            'JUL','AUG','SEP','OCT','NOV','DEC'/
      IF(IRET.NE.0) RETURN
      IF(IY.EQ.IMISS.OR.IM.EQ.IMISS.OR.ID.EQ.IMISS.OR.
     X   IH.EQ.IMISS.OR.IN.EQ.IMISS.OR.
     X   RLAT.EQ.RMISS.OR.RLON.EQ.RMISS) GO TO 500
C
C          1.2  OPEN AND READ LAND SEA MASK
C
      IF(ISFIR.NE.3H1ST.AND.LANSEA) THEN
           OPEN (81,IOSTAT=IOS,ERR=122,
     X           FILE='land_sea_mask.dat',
     X           STATUS='OLD',FORM='UNFORMATTED')
           READ (81,IOSTAT=IOS,ERR=124) MASK
           CLOSE (81)
           ISFIR=3H1ST
           GO TO 130
  122      CONTINUE
           WRITE (*,5470)ios
           IRET=1001
           GO TO 500
  124      CONTINUE
           WRITE (*,5475)ios
           IRET=1002
           GO TO 500
      ENDIF
5470  format(1h,' ERROR OPENING LAND SEA MASK FILE.IOSTAT = ',i5)
5475  format(1h,' ERROR READING LAND SEA MASK FILE.IOSTAT = ',i5)
C
C          1.3  CONVERT TIME OF REPORT TO MINUTES
C
  130 CONTINUE
      WRITE(CT(1:2),'(I2.2)') ID
      CT(3:3)='-'
      CT(4:6)=CMONTH(IM)
      CT(7:7)='-'
      WRITE(CT(8:11),'(I4)') IY
      CT(12:12)=' '
      WRITE(CT(13:14),'(I2.2)') IH
      CT(15:15)=':'
      WRITE(CT(16:17),'(I2.2)') IN
      CT(18:18)=':'
      WRITE(CT(19:20),'(I2.2)') 0
      CT(21:21)='.'
      WRITE(CT(22:23),'(I2.2)') 0
c      CALL ISTIME (CT,JMINSIN,IRET)
C     WRITE (*,'(1X,A5,2X,I4,4I2.2,I10,2F8.2)')
C    X         CALLSN,IY,IM,ID,IH,IN,JMINSIN,RLAT,RLON
C
C          1.4  CHECK WHETHER CALL SIGN IS UNIQUE
C
      LUNIQ=.TRUE.
      DO 142 KUNIQ=1,IUNIQ
      KP=INDEX(CUNIQ(KUNIQ),' ')
      KP=KP-1
      IF(KP.LT.1) KP=1
      IF(CALLSN(1:KP).EQ.CUNIQ(KUNIQ)(1:KP)) THEN
           LUNIQ=.FALSE.
           GO TO 144
      ENDIF
  142 CONTINUE
  144 CONTINUE
C
C    ----------------------------------------------------------------------
C
C                2.  MANAGE TIME SERIES FILE
C               -----------------------------
C
  200 CONTINUE
      IF(.NOT.LUNIQ) GO TO 300
C
C          2.1  MAKE SURE FILE IS OPEN
C
C
c     if(i1st.ne.999) then
      open(unit=76,iostat=ios,err=201,file=CFNAME,
c    1            access='keyed',
c    4            organization='indexed',
     1            organization='relative',
     2            form='unformatted',
     4            recl=520,
c    5            key=(1:9:character),
     5            status='unknown')
c       i1st=999
c     end if
      go to 220
c
 201  continue
c
      print*,'Open error on ',cfname,' is ',ios
      call exit(2)
c
C
C            2.2  Read record
C
 220  continue
c
      record.yident(1:9)=callsn
      aux_key = callsn
      ios = 0
      ierr = 0
      irecnum = 0
      call readseq(76,ierr,ios,irecnum,aux_key,cfname,record)
      if(ierr.eq.-1) then
        print*,' ************ error reading file 76'
        iret = 1000
        return
      end if
      if(ierr.eq.1) then
        ierr = 0
        go to 230
      end if
c     read(unit=76,key=record.yident,iostat=ios,err=230) record
c
      nslen=record.kslen
      nsent=record.ksent
      nspar=record.kspar
      minnt=record.kinnt
      do 221 kk=1,512
      irec(kk)=record.krec(kk)
 221  continue
      go to 250
c
C
C            2.3  RECORD DOES NOT EXIST, BUILD EMPTY RECORD
C
 230  continue
c
      if(ios.eq.142) then
           NSENT=0
           NSLEN=0
           NSPAR=2
           do 231 kk=1,512
           irec(kk)=0
 231       continue
           go to 260
      ELSE
         print*,'Read error ',ios,' on  ',cfname 
         call exit(2)
      end if
C
C            2.5  UNPACK RECORD
C
 250  continue
c
      CALL SHIPUP (IREC)
C
C            2.6  ADD LATEST INFORMATION TO RECORD
C
 260  continue
c
      CALL SHIPAD (JTYPE,JSBTYPE,JMINSIN,RLAT,RLON)
C     WRITE (*,'(1H ,5I10)') NCREP,NSLEN,NSENT,NSPAR,MINNT
C     DO 800 KENT=1,NSENT
C     WRITE (*,'(1H ,8I8)')
C    X     (NSTYPE(KSUB,KENT),NSSTYP(KSUB,KENT),KSUB=1,NTSUBT)
C     WRITE (*,'(1H ,I10)') MINSIN(KENT)
C     WRITE (*,'(1H ,2(F10.2,I8,F10.2))')
C    X     (SPARAM(KPAR,KENT),ISCONF(KPAR,KENT),SSUBST(KPAR,KENT),
C    X       KPAR=1,NSPAR)
C 800 CONTINUE
C
C    ------------------------------------------------------------------
C
C                 3.  POSITION CHECKING
C                -----------------------
C
  300 CONTINUE
C
C          3.1  INITIALISE FLAG VALUES
C
      IFLAG=IUNCH
      IFLAG0=0
      IFLAG1=0
      IFLAG2=0
      IFLAG3=0
      LCHECK=.FALSE.
      NCHECK=0
C
C            3.2  CHECK WHETHER POSITION OVER LAND
C
      OVERLND=.FALSE.
      IF(LANSEA) THEN
           CALL LANDSEA (RLAT,RLON,ISUM)
           IF(ISUM.EQ.4) THEN
                IFLAG=10
                OVERLND=.TRUE.
      WRITE (*,'(1H ,''POSITION OVER LAND:'',A,2X,I4,4I2.2,2F10.2)')
     X                     CALLSN,IY,IM,ID,IH,IN,RLAT,RLON
           ENDIF
      ENDIF
      IF(.NOT.LUNIQ) GO TO 500
      J=NCREP
      IF(J.LE.0) GO TO 500
      IF(OVERLND) GO TO 400
C
C            3.3  CHECK WITH PREVIOUS UNFLAGGED POSITIONS
C
      K0=J
      K1=J-1
  332 CONTINUE
      IF(K1.GT.0) THEN
           IF(ISCONF(1,K1).GE.60) THEN
                K2=K1-1
  334           CONTINUE
                IF(K2.GT.0) THEN
                     IF(ISCONF(1,K2).GE.60) THEN
C
C            COMPARE WITH N-1, N-2
C
                          CALL COMPOS (SPARAM(1,K2),SPARAM(2,K2),
     X                                 MINSIN(K2),
     X                                 SPARAM(1,K1),SPARAM(2,K1),
     X                                 MINSIN(K1),
     X                                 SPARAM(1,K0),SPARAM(2,K0),
     X                                 MINSIN(K0),
     X                                 LSHIP,LCHECK,IFLAG0)
                          IF(LCHECK) NCHECK=NCHECK+1
                          K3=K2-1
  336                     CONTINUE
                          IF(K3.GT.0) THEN
                             IF(ISCONF(1,K3).GE.60) THEN
C
C              COMPARE WITH N-2, N-3
C
                                CALL COMPOS (SPARAM(1,K3),SPARAM(2,K3),
     X                                       MINSIN(K3),
     X                                       SPARAM(1,K2),SPARAM(2,K2),
     X                                       MINSIN(K2),
     X                                       SPARAM(1,K0),SPARAM(2,K0),
     X                                       MINSIN(K0),
     X                                       LSHIP,LCHECK,IFLAG1)
                                IF(LCHECK) NCHECK=NCHECK+1
C
C              COMPARE WITH N-1, N-3
C
                                CALL COMPOS (SPARAM(1,K3),SPARAM(2,K3),
     X                                       MINSIN(K3),
     X                                       SPARAM(1,K1),SPARAM(2,K1),
     X                                       MINSIN(K1),
     X                                       SPARAM(1,K0),SPARAM(2,K0),
     X                                       MINSIN(K0),
     X                                       LSHIP,LCHECK,IFLAG2)
                                IF(LCHECK) NCHECK=NCHECK+1
                                GO TO 340
                             ELSE
                                K3=K3-1
                                GO TO 336
                             ENDIF
                          ENDIF
                     ELSE
                          K2=K2-1
                          GO TO 334
                     ENDIF
                ENDIF
           ELSE
                K1=K1-1
                GO TO 332
           ENDIF
      ENDIF
C
C            3.4  CHECK 3 LATEST POSITIONS REGARDLESS OF CONFIDENCE
C
  340 CONTINUE
      J2=J-2
      IF(J2.GT.0.AND.J2.NE.K2) THEN
           J1=J-1
           CALL COMPOS (SPARAM(1,J2),SPARAM(2,J2),MINSIN(J2),
     X                  SPARAM(1,J1),SPARAM(2,J1),MINSIN(J1),
     X                  SPARAM(1,J),SPARAM(2,J),MINSIN(J),
     X                  LSHIP,LCHECK,IFLAG3)
           IF(LCHECK) NCHECK=NCHECK+1
      ENDIF
C
C             3.5  ASSIGN CONFIDENCE
C
      IF(MAX(IFLAG0,IFLAG1,IFLAG2).GE.70) THEN
         IF(NCHECK.GT.1) IFLAG=MAX(IFLAG0,IFLAG1,IFLAG2,IFLAG3)
      ELSE
         IF(NCHECK.GT.1) IFLAG=MIN(IFLAG0,IFLAG1,IFLAG2,IFLAG3)
      END IF
C 
C     WRITE (*,'(1H ,11I4)') K0,K1,K2,K3,J,J1,J2,
C    X                      IFLAG0,IFLAG1,IFLAG2,IFLAG3
      IF(IFLAG.LT.IUNCH)
     X WRITE (*,5738)
     X CALLSN,IY,IM,ID,IH,IN,RLAT,RLON,IFLAG,
     X (SPARAM(1,K),SPARAM(2,K),K=1,NCREP-1)
5738  format(1h,' POSITION SUSPECT:',A,2X,I4,4I2.2,2F10.2,
     x ' CONFIDENCE: ',i4,'; PREVIOUS POSITIONS: ',4(2X,2F10.2))   
C
C    -------------------------------------------------------------------
C
C                  4.  UPDATE RECORD
C                 -------------------
C
  400 CONTINUE
      ISCONF(1,NCREP)=IFLAG
      ISCONF(2,NCREP)=IFLAG
C
C           4.1  PACK TIME SERIES RECORD
C
      CALL SHIPPK (IREC)
C
C           4.2  WRITE TIME SERIES RECORD
C
      record.kslen=nslen
      record.ksent=nsent
      record.kspar=nspar
      record.kinnt=minnt
      do 421 kk=1,512
      record.krec(kk)=irec(kk)
 421  continue
c
c     IF(IOS.EQ.0) THEN
c        WRITE(UNIT=76,rec=irecnum,IOSTAT=IOS,ERR=410)  record
cc       REWRITE(UNIT=76,IOSTAT=IOS,ERR=410) record
c     ELSE
c        WRITE(UNIT=76,IOSTAT=IOS,ERR=410)  record
c     END IF
c
      WRITE(UNIT=76,rec=irecnum,IOSTAT=IOS,ERR=410)  record
      close(76)
c
      go to 500
c
 410  continue
c
      print*,'Error during write on file ',cfname
      iret=ios
      return
c
C
C    -------------------------------------------------------------------
C
C                       5.  EXIT
C                      -----------
C
  500 CONTINUE
      RETURN
      END
      SUBROUTINE COMPOS (RLAT1,RLON1,ICM1,
     X                   RLAT2,RLON2,ICM2,
     X                   RLAT3,RLON3,ICM3,LSHIP,LCHECK,IFLAG)
C
C****  *COMPOS*
C
C       PURPOSE.
C      ----------
C
C        COMPARE A POSITION REPORT WITH 2 PREVIOUS POSITIONS
C
C       INTERFACE.
C      ------------
C
C           CALL COMPOS (RLAT1,RLON1,ICM1,
C                        RLAT2,RLON2,ICM2,
C                        RLAT3,RLON3,ICM3,LSHIP,LCHECK,IFLAG)
C
C        INPUT
C          RLAT1,RLON1,ICM1 -
C                  -  LATITUDE, LONGITUDE AND TIME IN MINUTES
C                     OF OLDEST REPORT
C          RLAT2,RLON2,ICM2 -
C                  -  LATITUDE, LONGITUDE AND TIME IN MINUTES
C                     OF MIDDLE REPORT
C          RLAT3,RLON3,ICM3 -
C                  -  LATITUDE, LONGITUDE AND TIME IN MINUTES
C                     OF CURRENT REPORT
C          LSHIP   - LOGICAL : TRUE IF SHIP
C
C        OUTPUT
C          LCHECK  -  LOGICAL WHETHER CHECKING POSSIBLE
C          IFLAG   -  CONFIDENCE APPLIED TO CURRENT POSITION
C
C       METHOD.
C      ---------
C
C         LINEARLY EXTRAPOLATE POSITIONS 1 AND 2 TO GET AN ESTIMATE (E)
C         OF POSITION 3.  POSITION 3 SHOULD FALL WITHIN A CIRCLE CENTRED
C         AT E AND WITH RADIUS BASED ON THE DISTANCE 2 TO E AND THE TIME
C         DIFFERENCE 2 TO 3.  CONFIDENCE DEPENDS ON THE DISTANCE OF 3
C         FROM THE CENTRE.
C
C       EXTERNALS.
C      ------------
C
C         D2PTS    -  DISTANCE BETWEEN 2 POINTS ON EARTH'S SURFACE
C
C       REFERENCE.
C      ------------
C
C         NONE
C
C       AUTHOR.
C      ---------
C
C         B. NORRIS,  ECMWF,  MARCH 1990
C         BASED ON CODE WRITTEN FOR NOS/BE IN 1982
C
C       MODIFICATIONS.
C      ----------------
C
C         NONE
C
      LOGICAL LCHECK,LSHIP
C
C    ----------------------------------------------------------------------
C
C                   1.  INITIALISATION
C                  --------------------
C
  100 CONTINUE
C              ZSCALE = RADIUS OF EARTH IN KM
      DATA ZSCALE/6371./
      LCHECK=.FALSE.
C
C    ---------------------------------------------------------------------
C
C                   2.  CHECK POSITION
C                  --------------------
C
  200 CONTINUE
C
C           2.1  CHECK TIME DIFFERENCE
C
      ICMD=ICM3-ICM2
      ICME=ICM2-ICM1
      IF(LSHIP) THEN
         ITOOLD=2160
      ELSE
         ITOOLD=360
      ENDIF
C
      IF(ICMD.GT.ITOOLD.OR.ICME.GT.ITOOLD) GO TO 300
      IF(LSHIP.AND.ICME.LT.60) ICME=60
      IF(ICME.LT.1) ICME=1
      RCMF=FLOAT(ICMD)/FLOAT(ICME)
C
C           2.2  OBTAIN ESTIMATE OF POSITION IN RLATE & RLONE
C
      RLATE=RCMF*(RLAT2-RLAT1)+RLAT2
      RLO1=RLON1
      RLO2=RLON2
      IF(RLO1.LT.-90.0.AND.RLO2.GT.0.0) RLO1=RLO1+360.0
      IF(RLO1.GT. 90.0.AND.RLO2.LT.0.0) RLO1=RLO1-360.0
      RLONE=RCMF*(RLO2-RLO1)+RLO2
      IF(ABS(RLATE).GT.90.0) THEN
           RLATE=SIGN(180.0,RLATE)-RLATE
           RLONE=RLONE+180.0
      ENDIF
      IF(RLONE.GT.180.0) RLONE=RLONE-360.0
      IF(RLONE.GT.180.0) RLONE=RLONE-360.0
      IF(RLONE.LT.-180.0) RLONE=RLONE+360.0
C
C          2.3  OBTAIN DISTANCE BETWEEN ESTIMATED AND SECOND POSITION
C
      CALL D2PTS (RLAT2,RLON2,RLATE,RLONE,ZSCALE,DIST2E)
C
C          2.4  OBTAIN DISTANCE BETWEEN ESTIMATED AND THIRD POSITION
C
      CALL D2PTS (RLAT3,RLON3,RLATE,RLONE,ZSCALE,DIST3E)
C
C          2.5  ASSIGN CONFIDENCE DEPENDING ON RADIUS ALLOWED
C
      IF(LSHIP) THEN
         RAVGDIST=30.0
      ELSE
         RAVGDIST=200.0
      ENDIF
C
      RAD=DIST2E+RAVGDIST+FLOAT(ICMD)/2.0
      IFLAG=90.0-(40.0*DIST3E)/RAD
C
      IF(IFLAG.LT.0) IFLAG=0
      LCHECK=.TRUE.
C     WRITE (*,'(1H ,2I6,2F10.2,3F8.1)')
C    X     ICMD,ICME,RLATE,RLONE,DIST2E,DIST3E,RAD
C
C    ---------------------------------------------------------------------
C
C                       3.  EXIT
C                      ----------
C
  300 CONTINUE
      RETURN
      END
      SUBROUTINE D2PTS (RLAT1,RLON1,RLAT2,RLON2,ZSCALE,DIST)
C
C****  *D2PTS*
C
C       PURPOSE.
C      ----------
C
C         CALCULATE THE DISTANCE BETWEEN 2 POINTS ON THE
C                   EARTH'S SURFACE
C
C       INTERFACE.
C      ------------
C
C           CALL D2PTS (RLAT1,RLON1,RLAT2,RLON2,ZSCALE,DIST)
C
C        INPUT
C          RLAT1,RLON1 -  LATITUDE AND LONGITUDE OF FIRST POSITION
C          RLAT2,RLON2 -  LATITUDE AND LONGITUDE OF SECOND POSITION
C          ZSCALE  -  SCALING FACTOR
C                   ZSCALE=1.      GIVES DISTANCE IN EARTH RADII
C                   ZSCALE=3960.   GIVES DISTANCE IN MILES
C                   ZSCALE=6371.   GIVES DISTANCE IN KM
C
C        OUTPUT
C          DIST  -  DISTANCE IN UNITS GIVEN BY ZSCALE
C
C       METHOD.
C      ---------
C
C         DELTAX = COS(LAT1)*COS(LON1) - COS(LAT2)*COS(LON2)
C         DELTAY = COS(LAT1)*SIN(LON1) - COS(LAT2)*SIN(LON2)
C         DELTAX = SIN(LAT1) - SIN(LAT2)
C         DIST = SQRT (DELTAX**2 + DELTAY**2 + DELTAZ**2)
C
C       EXTERNALS.
C      ------------
C
C         NONE
C
C       REFERENCE.
C      ------------
C
C         NONE
C
C       AUTHOR.
C      ---------
C
C         B. NORRIS,  ECMWF,  MARCH 1990
C         BASED ON CODE WRITTEN BY DATA ASSIMILATION
C
C       MODIFICATIONS.
C      ----------------
C
C         NONE
C
C
C    -------------------------------------------------------------------
C
C                     1.  INITIALISATION
C                   ----------------------
C
  100 CONTINUE
C              PIF = (PI/180)
      DATA PIF/0.017453/
C
C    --------------------------------------------------------------------
C
C                     2.  CALCULATION
C                   -------------------
C
  200 CONTINUE
      ALA1=RLAT1*PIF
      ALO1=RLON1*PIF
      ALA2=RLAT2*PIF
      ALO2=RLON2*PIF
      CALA1=COS(ALA1)
      CALA2=COS(ALA2)
      DELX=CALA1*COS(ALO1)-CALA2*COS(ALO2)
      DELY=CALA1*SIN(ALO1)-CALA2*SIN(ALO2)
      DELZ=SIN(ALA1)-SIN(ALA2)
      DIST=SQRT(DELX*DELX+DELY*DELY+DELZ*DELZ)*ZSCALE
C
C    ----------------------------------------------------------------------
C
C                         3.  EXIT
C                       ------------
C
  300 CONTINUE
      RETURN
      END
      SUBROUTINE SHIPUP (IREC)
C
C****  *SHIPUP*
C
C       PURPOSE.
C      ----------
C
C         UNPACK TIME SERIES RECORD
C
C       INTERFACE.
C      ------------
C
C           CALL SHIPUP (IREC)
C
C        INPUT
C          IREC -  PACKED RECORD
C
C        OUTPUT
C          UNPACKED RECORD IN COMMON COMSHIP
C
C       METHOD.
C      ---------
C
C         UNPACK HOUSEKEEPING ITEMS FROM KEY;
C         UNPACK TYPE, SUBTYPE, TIME, PARAMETER, CONFIDENCE AND
C         SUBSTITUTIONS FROM RECORD, APPLYING REFERENCE VALUES
C         AND SCALING FACTORS WHERE APPROPRIATE
C
C       EXTERNALS.
C      ------------
C
C         UNPACK  -  EXTRACT BIT FIELDS FROM ARRAY
C
C       REFERENCE.
C      ------------
C
C         NONE
C
C       AUTHOR.
C      ---------
C
C         B. NORRIS,  ECMWF,  MARCH 1990
C         BASED ON CODE WRITTEN FOR NOS/BE IN 1982
C
C       MODIFICATIONS.
C      ----------------
C
C         NONE
C
      INCLUDE 'paramsh.f'
      INCLUDE 'comship.f'
      INCLUDE 'compref.f'
      INCLUDE 'const.f'
      DIMENSION IREC(*)
C
C    -------------------------------------------------------------------
C
C                     1.  UNPACK KEY
C                    ----------------
C
  100 CONTINUE
C    ---------------------------------------------------------------------
C
C                     2.  UNPACK RECORD
C                   ---------------------
C
  200 CONTINUE
      IW=1
      IBT=0
C
C            2.1  LOOP FOR NUMBER OF ENTRIES
C
      DO 234 JENT=1,NSENT
C
C            2.2  LOOP FOR NUMBER OF TYPES AND SUBTYPES
C
      DO 222 JTYP=1,NTSUBT
      CALL UNPACK (NSBPW,IREC(IW),NSTYPE(JTYP,JENT),IW,IBT,8,IRET)
      IF(NSTYPE(JTYP,JENT).EQ.2**8-1) NSTYPE(JTYP,JENT)=IMISS
      CALL UNPACK (NSBPW,IREC(IW),NSSTYP(JTYP,JENT),IW,IBT,8,IRET)
      IF(NSSTYP(JTYP,JENT).EQ.2**8-1) NSSTYP(JTYP,JENT)=IMISS
  222 CONTINUE
      CALL UNPACK (NSBPW,IREC(IW),MINSIN(JENT),IW,IBT,32,IRET)
C
C           2.3  LOOP FOR NUMBER OF PARAMETERS
C
      DO 232 JPAR=1,NSPAR
      CALL UNPACK (NSBPW,IREC(IW),KPARAM,IW,IBT,28,IRET)
      IF(KPARAM.EQ.2**28-1) THEN
           SPARAM(JPAR,JENT)=RMISS
      ELSE
           SPARAM(JPAR,JENT)=FLOAT(KPARAM+JREF(JPAR))/10.**JSCAL(JPAR)
      ENDIF
      CALL UNPACK (NSBPW,IREC(IW),ISCONF(JPAR,JENT),IW,IBT,8,IRET)
      IF(ISCONF(JPAR,JENT).EQ.2**8-1) ISCONF(JPAR,JENT)=IMISS
      CALL UNPACK (NSBPW,IREC(IW),KPARAM,IW,IBT,28,IRET)
      IF(KPARAM.EQ.2**28-1) THEN
           SSUBST(JPAR,JENT)=RMISS
      ELSE
           SSUBST(JPAR,JENT)=FLOAT(KPARAM+JREF(JPAR))/10.**JSCAL(JPAR)
      ENDIF
  232 CONTINUE
  234 CONTINUE
C
C    ---------------------------------------------------------------------
C
C                         3.  EXIT
C                        -----------
C
  300 CONTINUE
      RETURN
      END
      SUBROUTINE SHIPPK (IREC)
C
C****  *SHIPPK*
C
C       PURPOSE.
C      ----------
C
C         PACK TIME SERIES RECORD
C
C       INTERFACE.
C      ------------
C
C           CALL SHIPPK (IREC)
C
C        INPUT
C          UNPACKED RECORD IN COMMON COMSHIP
C
C        OUTPUT
C          IKEY -  FULL KEY OF RECORD
C          IREC -  PACKED RECORD
C
C       METHOD.
C      ---------
C
C         PACK TYPE, SUBTYPE, TIME, PARAMETER, CONFIDENCE AND
C         SUBSTITUTIONS INTO RECORD, APPLYING REFERENCE VALUES
C         AND SCALING FACTORS WHERE APPROPRIATE;
C         PACK HOUSEKEEPING ITEMS INTO KEY.
C
C       EXTERNALS.
C      ------------
C
C         PACK  -  WRITE BIT FIELDS INTO ARRAY
C
C       REFERENCE.
C      ------------
C
C         NONE
C
C       AUTHOR.
C      ---------
C
C         B. NORRIS,  ECMWF,  MARCH 1990
C         BASED ON CODE WRITTEN FOR NOS/BE IN 1982
C
C       MODIFICATIONS.
C      ----------------
C
C         NONE
C
      INCLUDE 'paramsh.f'
      INCLUDE 'comship.f'
      INCLUDE 'compref.f'
      INCLUDE 'const.f'
      DIMENSION IREC(*)
C
C    ---------------------------------------------------------------------
C
C                     1.  PACK RECORD
C                   -------------------
C
  100 CONTINUE
      IW=1
      IBT=0
C
C           1.1  LOOP FOR NUMBER OF ENTRIES
C
      DO 134 JENT=1,NSENT
C
C           1.2  LOOP FOR NUMBER OF TYPES AND SUBTYPES
C
      DO 122 JTYP=1,NTSUBT
      KSTYPE=NSTYPE(JTYP,JENT)
      IF(KSTYPE.EQ.IMISS) KSTYPE=2**8-1
      CALL PACK (NSBPW,IREC(IW),KSTYPE,IW,IBT,8,IRET)
      KSSTYP=NSSTYP(JTYP,JENT)
      IF(KSSTYP.EQ.IMISS) KSSTYP=2**8-1
      CALL PACK (NSBPW,IREC(IW),KSSTYP,IW,IBT,8,IRET)
  122 CONTINUE
      CALL PACK (NSBPW,IREC(IW),MINSIN(JENT),IW,IBT,32,IRET)
C
C            1.3  LOOP FOR NUMBER OF PARAMETERS
C
      DO 132 JPAR=1,NSPAR
      IF(SPARAM(JPAR,JENT).EQ.RMISS) THEN
           KPARAM=2**28-1
      ELSE
           KPARAM=NINT(SPARAM(JPAR,JENT)*10.**JSCAL(JPAR))-JREF(JPAR)
      ENDIF
      CALL PACK (NSBPW,IREC(IW),KPARAM,IW,IBT,28,IRET)
      KSCONF=ISCONF(JPAR,JENT)
      IF(KSCONF.EQ.IMISS) KSCONF=2**8-1
      CALL PACK (NSBPW,IREC(IW),KSCONF,IW,IBT,8,IRET)
      IF(SSUBST(JPAR,JENT).EQ.RMISS) THEN
           KPARAM=2**28-1
      ELSE
           KPARAM=NINT(SSUBST(JPAR,JENT)*10.**JSCAL(JPAR))-JREF(JPAR)
      ENDIF
      CALL PACK (NSBPW,IREC(IW),KPARAM,IW,IBT,28,IRET)
  132 CONTINUE
  134 CONTINUE
C
C    ---------------------------------------------------------------------
C
C                     2.  PACK KEY
C                   -----------------
C
  200 CONTINUE
      NSLEN=(IW-1)*4+IBT/8
C
C    -----------------------------------------------------------------------
C
C                        3.  EXIT
C                       ----------
C
  300 CONTINUE
      RETURN
      END
      SUBROUTINE SHIPAD (JTYPE,JSBTYPE,JMINSIN,RLAT,RLON)
C
C****  *SHIPAD*
C
C       PURPOSE.
C      ----------
C
C         ADD NEW REPORT TO TIME SERIES RECORD
C
C       INTERFACE.
C      ------------
C
C           CALL SHIPAD (JTYPE,JSBTYPE,JMINSIN,RLAT,RLON)
C
C        INPUT
C          JTYPE    -  BUFR TYPE FOR NEW REPORT
C          JSBTYPE  -  BUFR SUBTYPE FOR NEW REPORT
C          JMINSIN  -  MINUTE OF NEW REPORT
C          RLAT,RLON - LATITUDE AND LONGITUDE OF NEW REPORT
C          TIME SERIES RECORD IN COMMON COMSHIP
C
C        OUTPUT
C          TIME SERIES RECORD UPDATED SUCH THAT REPORTS ARE ORDERED
C          CHRONOLOGICALLY
C
C       METHOD.
C      ---------
C
C         IF LAT, LONG AND MINUTE MATCH A PREVIOUS REPORT, THEN ADD
C         TYPE AND SUBTYPE;
C         IF NUMBER OF ENTRIES IS LESS THAN MAXIMUM, ADD NEW REPORT
C         IN CORRECT POSITION DEPENDING ON TIME;
C         IF NUMBER OF ENTRIES IS ALREADY AT MAXIMUM, ADD NEW REPORT
C         IN CORRECT POSITION AND EXCLUDE OLDEST ENTRY.
C
C       EXTERNALS.
C      ------------
C
C         NONE
C
C       REFERENCE.
C      ------------
C
C         NONE
C
C       AUTHOR.
C      ---------
C
C         B. NORRIS,  ECMWF,  MARCH 1990
C         BASED ON CODE WRITTEN FOR NOS/BE IN 1982
C
C       MODIFICATIONS.
C      ----------------
C
C         NONE
C
      INCLUDE 'paramsh.f'
      INCLUDE 'comship.f'
      INCLUDE 'const.f'
C
C    ----------------------------------------------------------------------
C
C                        1.  INITIALISATION
C                      ---------------------
C
  100 CONTINUE
C
C           1.1  NCREP = ENTRY NUMBER OF CURRENT REPORT
C
      NCREP=0
      EPS=0.001
C
C    ----------------------------------------------------------------------
C
C               2.  LOOK FOR MATCH IN POSITION AND TIME
C             ------------------------------------------
C
  200 CONTINUE
      IF(NSENT.EQ.0) GO TO 324
C
C         2.1  LOOP FOR EXISTING ENTRIES
C
      DO 224 JENT=1,NSENT
      IF(JMINSIN.EQ.MINSIN(JENT).AND.
     X   ABS(RLAT-SPARAM(1,JENT)).LT.EPS.AND.
     X   ABS(RLON-SPARAM(2,JENT)).LT.EPS) THEN
C
C         2.2  MATCH FOUND, ADD TYPE AND SUBTYPE
C
           NCREP=JENT
           DO 222 JSUB=1,NTSUBT
           IF(JTYPE.EQ.NSTYPE(JSUB,JENT).AND.
     X        JSBTYPE.EQ.NSSTYP(JSUB,JENT)) GO TO 600
           IF(NSTYPE(JSUB,JENT).EQ.IMISS) THEN
                NSTYPE(JSUB,JENT)=JTYPE
                NSSTYP(JSUB,JENT)=JSBTYPE
                GO TO 600
           ENDIF
  222      CONTINUE
      ENDIF
  224 CONTINUE
C
C    ----------------------------------------------------------------------
C
C              3.  FIND POSITION FOR NEW ENTRY
C             ---------------------------------
C
  300 CONTINUE
      NSE=NSENT
      DO 322 JENT=1,NSE
      IF(JMINSIN.LE.MINSIN(JENT)) THEN
           IF(NSENT.LT.NENTMAX) THEN
C
C        3.1  ENTRIES < MAXIMUM, TIME < LATEST TIME
C
                IFF=NSENT+1
                IFL=JENT+1
                K1=-1
                IF(IFL.GT.IFF) K1=0
                NCREP=JENT
                NSENT=NSENT+1
           ELSE
C
C        3.2  ENTRIES = MAXIMUM, TIME < LATEST TIME
C
                IFF=1
                IFL=JENT-2
                K1=1
                IF(IFL.LT.IFF) K1=0
                NCREP=JENT-1
           ENDIF
           GO TO 345
      ENDIF
  322 CONTINUE
  324 CONTINUE
C
      IF(NSENT.LT.NENTMAX) THEN
C
C        3.3  ENTRIES < MAXIMUM, TIME > LATEST TIME
C
           K1=0
           NSENT=NSENT+1
           NCREP=NSENT
      ELSE
C
C        3.4  ENTRIES = MAXIMUM, TIME > LATEST TIME
C
           IFF=1
           IFL=NENTMAX-1
           K1=1
           IF(IFL.LT.IFF) K1=0
           NCREP=NENTMAX
      ENDIF
  345 CONTINUE
C
C    ----------------------------------------------------------------------
C
C                4.  MOVE OLD ENTRIES
C               ----------------------
C
  400 CONTINUE
      IF(K1.NE.0) THEN
           DO 406 I=IFF,IFL,K1
           DO 402 JTYP=1,NTSUBT
           NSTYPE(JTYP,I)=NSTYPE(JTYP,I+K1)
           NSSTYP(JTYP,I)=NSSTYP(JTYP,I+K1)
  402      CONTINUE
           MINSIN(I)=MINSIN(I+K1)
           DO 404 JPAR=1,NSPAR
           SPARAM(JPAR,I)=SPARAM(JPAR,I+K1)
           ISCONF(JPAR,I)=ISCONF(JPAR,I+K1)
           SSUBST(JPAR,I)=SSUBST(JPAR,I+K1)
  404      CONTINUE
  406      CONTINUE
      ENDIF
C
C    --------------------------------------------------------------------------
C
C                   5.  ADD NEW ENTRY
C                 --------------------
C
  500 CONTINUE
      IF(NCREP.GT.0) THEN
           NSTYPE(1,NCREP)=JTYPE
           NSSTYP(1,NCREP)=JSBTYPE
           DO 502 JTYP=2,NTSUBT
           NSTYPE(JTYP,NCREP)=IMISS
           NSSTYP(JTYP,NCREP)=IMISS
  502      CONTINUE
           MINSIN(NCREP)=JMINSIN
           SPARAM(1,NCREP)=RLAT
           ISCONF(1,NCREP)=IMISS
           SSUBST(1,NCREP)=RMISS
           SPARAM(2,NCREP)=RLON
           ISCONF(2,NCREP)=IMISS
           SSUBST(2,NCREP)=RMISS
      ENDIF
C
C    -----------------------------------------------------------------------
C
C                          6.  EXIT
C                        -----------
C
  600 CONTINUE
      MINNT=MINSIN(NSENT)
      RETURN
      END
      SUBROUTINE LANDSEA (RLAT,RLON,ISUM)
C
C****  *LANDSEA*
C
C       PURPOSE.
C      ----------
C
C         CHECK WHETHER POSITION IS OVER LAND
C
C       INTERFACE.
C      ------------
C
C           CALL LANDSEA (RLAT,RLON,ISUM)
C
C        INPUT
C           RLAT,RLON  -  POSITION
C           MASK (COMMON COMLSM)
C                      -  LAND/SEA MASK RECORD
C                         (97 LATITUDE ROWS * 192 LONGITUDE COLUMNS,
C                         EACH ROW PACKED INTO 6 WORDS)
C
C        OUTPUT
C          ISUM  -  NUMBER OF SURROUNDING LAND POINTS
C
C       METHOD.
C      ---------
C
C         CALCULATE POSITIONS OF 4 SURROUNDING GRID POINTS OF MASK;
C         SUM THE NUMBER OF LAND POINTS.
C
C       EXTERNALS.
C      ------------
C
C         GBYTE  -  EXTRACT BIT FIELD FROM ARRAY
C
C       REFERENCE.
C      ------------
C
C         NONE
C
C       AUTHOR.
C      ---------
C
C         B. NORRIS,  ECMWF,  MARCH 1990
C         BASED ON CODE WRITTEN FOR NOS/BE IN 1982
C
C       MODIFICATIONS.
C      ----------------
C
C         NONE
C
      INCLUDE 'paramsh.f'
      INCLUDE 'comlsm.f'
      DIMENSION IPTS(2,4)
C
C    ----------------------------------------------------------------------
C
C                       1.  INITIALISATION
C                      ---------------------
C
  100 CONTINUE
C
C          1.1  GRID = GRID OF MASK FIELD;
C                      MASK FIELD HAS DIMENSION IDIM1 * IDIM2
C
      DATA GRID/1.875/,IDIM1/97/,IDIM2/192/
      ISUM=0
C
C          1.2  CHECK FOR REASONABLE LATITUDE AND LONGITUDE
C
      IF(ABS(RLAT).GT.90.0.OR.ABS(RLON).GT.180.0) THEN
           ISUM=4
           GO TO 400
      ENDIF
C
C    ----------------------------------------------------------------------
C
C                  2.  FIND 4 SURROUNDING GRID POINTS
C                --------------------------------------
C
  200 CONTINUE
C
C           2.1  SCALE THE POSITION TO BE CONSISTENT WITH MASK ARRAY
C
      SCLAT=90.0-RLAT
      SCLON=RLON
      IF(SCLON.LT.0.0) SCLON=SCLON+360.0
C
C         2.2  IPTS CONTAINS POSITIONS WITHIN MASK ARRAY
C              OF 4 SURROUNDING POINTS
C              POINT 1 = NW
C              POINT 2 = NE
C              POINT 3 = SW
C              POINT 4 = SE
C
      IPTS(1,1)=IFIX(SCLAT/GRID)+1
      IPTS(2,1)=IFIX(SCLON/GRID)+1
      IF(IPTS(1,1).EQ.IDIM1)   IPTS(1,1)=IDIM1-1
      IF(IPTS(2,1).EQ.IDIM2+1) IPTS(2,1)=IDIM2
      NLONG=IPTS(2,1)+1
      IF(NLONG.EQ.IDIM2+1) NLONG=1
      IPTS(1,2)=IPTS(1,1)
      IPTS(2,2)=NLONG
      IPTS(1,3)=IPTS(1,1)+1
      IPTS(2,3)=IPTS(2,1)
      IPTS(1,4)=IPTS(1,3)
      IPTS(2,4)=NLONG
C
C    ------------------------------------------------------------------------
C
C                 3.  SUM THE LAND POINTS
C                -------------------------
C
  300 CONTINUE
      ISUM=0
      DO 312 I=1,4
C
C           3.1  UNPACK APPROPRIATE WORDS OF MASK
C
      IWRD=(IPTS(2,I)-1)/NSBPW+1
      IPOS=MOD(IPTS(2,I)-1,NSBPW)
      CALL GBYTE (MASK(IWRD,IPTS(1,I)),ISMASK,IPOS,1)
C     WRITE(*,'(1H ,3I4,1X,O11)') IWRD,IPOS,ISMASK,MASK(IWRD,IPTS(1,I))
      ISUM=ISUM+ISMASK
  312 CONTINUE
C
C    -----------------------------------------------------------------------
C
C                         4.  EXIT
C                       ------------
C
  400 CONTINUE
C     WRITE (*,'(1H ,2F10.2,9I4)') RLAT,RLON,IPTS,ISUM
      RETURN
      END
      SUBROUTINE LAPS (IERR,NST,PP,TT,IVS,NNPP,NNTT,RLAT,RLONG,
     X                 CALLSN,IY,IM,ID,IH,IN)
C
C****  *LAPS*
C
C       PURPOSE.
C      ----------
C
C           LAPSE RATE AND INVERSION CHECK FOR VERTICAL
C            TEMPERATURE PROFILES
C
C       INTERFACE.
C      ------------
C
C           CALL LAPS (IERR,NST,PP,TT,IVS,NNPP,NNTT,RLAT,RLONG,
C                      CALLSN,IY,IM,ID,IH,IN)
C
C        INPUT
C          NST     -  NUMBER OF TEMPERATURE LEVELS
C          PP      -  PRESSURE (PA)
C          TT      -  TEMPERATURE (DEG K)
C          IVS     -  VERTICAL SIGNIFICANCE
C          NNPP    -  FLAG POSITIONS FOR PRESSURE
C          NNTT    -  FLAG POSITIONS FOR TEMPERATURE
C          RLAT    -  LATITUDE VALUE OF REPORT
C          RLONG   -  LONGITUDE VALUE OF REPORT
C          CALLSN  -  STATION NUMBER OR CALL SIGN
C          IY,IM,ID,IH,IN - YEAR,MONTH,DAY,HOUR,MINUTE FOR REPORT
C
C        OUTPUT
C          IERR    -  ERROR CODE
C          CONFIDENCE VALUES UPDATED
C
C       METHOD.
C      ---------
C
C         CHECK FOR UNREASONABLE INVERSION;
C         CHECK FOR SUPERADIABATIC LAYER.
C
C       EXTERNALS.
C      ------------
C
C         PINTER   -  INTERPOLATE TO PRESSURE OR HEIGHT LEVEL
C         SETINT   -  SET CONFIDENCE FLAGS FOR INTERNAL CONSISTENCY
C                      CHECK
C         SETGRS   -  SET CONFIDENCE FLAGS FOR GROSS LIMIT CHECK
C
C       REFERENCE.
C      ------------
C
C         GUIDE ON THE GLOBAL DATA PROCESSING SYSTEM WMO N305 1982
C            CHAPTER 6, QUALITY CONTROL PROCEDURES
C
C       AUTHOR.
C      ---------
C
C         B. NORRIS,  ECMWF,  MARCH 1990
C
C       MODIFICATIONS.
C      ----------------
C
C         NONE
C
      INCLUDE 'paramq.f'
      PARAMETER (NZONE=3)
      INCLUDE 'const.f'
      INCLUDE 'ctua.f'
      DIMENSION PP(*),TT(*),IVS(*),NNPP(*),NNTT(*)
      DIMENSION ULAYT(NLEV),ULAPS(NLEV)
      DIMENSION UINL(NSEASN,NZONE,NLEV)
      DIMENSION UICL(NSEASN,NZONE,NLEV)
      CHARACTER*(*) CALLSN
      LOGICAL LFLAG
C
C    --------------------------------------------------------------------
C
C                     1. SET UP
C                   ------------
C
  100 CONTINUE
C
C          1.1  TABLE (IN PA) FOR BOUNDARY BETWEEN
C                             THIN AND THICK LAYER
C
      DATA (ULAYT(I),I=1,10)/
     X       2000.0,
     X       2000.0,
     X       1000.0,
     X       1000.0,
     X       1000.0,
     X       1000.0,
     X       1000.0,
     X       1000.0,
     X       1000.0,
     X       1000.0/
      DATA (ULAYT(I),I=11,NLEV)/
     X       1000.0,
     X       1000.0,
     X       1000.0,
     X       1000.0,
     X       1000.0,
     X       1000.0,
     X       1000.0,
     X       1000.0,
     X       1000.0,
     X       1000.0,
     X       1000.0,
     X       1000.0/
C
C          1.2  TABLE (IN DEG K PER HPA) FOR MAXIMUM INVERSION
C                   OVER THIN LAYER
C
      DATA (((UINL(I,J,K),I=1,NSEASN),J=1,NZONE),K=1,10)/
C            0-30 DEG            30-60 DEG           60-90 DEG LATITUDE
C       WINTER    SUMMER    WINTER    SUMMER    WINTER    SUMMER
     X    1.5,      1.2,      2.0,      1.5,      2.5,      1.8,
     X    1.5,      1.2,      2.0,      1.5,      2.5,      1.8,
     X    1.4,      1.2,      1.8,      1.5,      2.2,      1.8,
     X    1.2,      1.2,      1.5,      1.5,      1.8,      1.8,
     X    1.2,      1.2,      1.5,      1.5,      1.8,      1.8,
     X    1.2,      1.2,      1.5,      1.5,      1.8,      1.8,
     X    1.4,      1.4,      1.5,      1.5,      1.8,      1.8,
     X    1.8,      1.8,      1.8,      1.8,      1.8,      1.8,
     X    2.2,      2.2,      2.2,      2.2,      2.2,      2.2,
     X    2.6,      2.6,      2.6,      2.6,      2.6,      2.6/
      DATA (((UINL(I,J,K),I=1,NSEASN),J=1,NZONE),K=11,NLEV)/
C            0-30 DEG            30-60 DEG           60-90 DEG LATITUDE
C       WINTER    SUMMER    WINTER    SUMMER    WINTER    SUMMER
     X    3.0,      3.0,      3.0,      3.0,      3.0,      3.0,
     X    3.5,      3.5,      3.5,      3.5,      3.5,      3.5,
     X    4.5,      4.5,      4.5,      4.5,      4.5,      4.5,
     X    5.5,      5.5,      5.5,      5.5,      5.5,      5.5,
     X    6.5,      6.5,      6.5,      6.5,      6.5,      6.5,
     X    7.5,      7.5,      7.5,      7.5,      7.5,      7.5,
     X    8.5,      8.5,      8.5,      8.5,      8.5,      8.5,
     X    8.5,      8.5,      8.5,      8.5,      8.5,      8.5,
     X    8.5,      8.5,      8.5,      8.5,      8.5,      8.5,
     X    8.5,      8.5,      8.5,      8.5,      8.5,      8.5,
     X    8.5,      8.5,      8.5,      8.5,      8.5,      8.5,
     X    8.5,      8.5,      8.5,      8.5,      8.5,      8.5/
C
C          1.3  TABLE (IN DEG K) FOR MAXIMUM INVERSION
C                   OVER THICK LAYER
C
      DATA (((UICL(I,J,K),I=1,NSEASN),J=1,NZONE),K=1,10)/
C            0-30 DEG            30-60 DEG           60-90 DEG LATITUDE
C       WINTER    SUMMER    WINTER    SUMMER    WINTER    SUMMER
     X   30.0,     24.0,     40.0,     30.0,     50.0,     36.0,
     X   30.0,     24.0,     40.0,     30.0,     50.0,     36.0,
     X   14.0,     12.0,     18.0,     15.0,     22.0,     18.0,
     X   12.0,     12.0,     15.0,     15.0,     18.0,     18.0,
     X   12.0,     12.0,     15.0,     15.0,     18.0,     18.0,
     X   12.0,     12.0,     15.0,     15.0,     18.0,     18.0,
     X   14.0,     14.0,     15.0,     15.0,     18.0,     18.0,
     X   18.0,     18.0,     18.0,     18.0,     18.0,     18.0,
     X   26.0,     22.0,     26.0,     22.0,     26.0,     22.0,
     X   29.0,     26.0,     29.0,     26.0,     29.0,     26.0/
      DATA (((UICL(I,J,K),I=1,NSEASN),J=1,NZONE),K=11,NLEV)/
C            0-30 DEG            30-60 DEG           60-90 DEG LATITUDE
C       WINTER    SUMMER    WINTER    SUMMER    WINTER    SUMMER
     X   32.0,     30.0,     32.0,     30.0,     32.0,     30.0,
     X   35.0,     35.0,     35.0,     35.0,     35.0,     35.0,
     X   35.0,     35.0,     35.0,     35.0,     35.0,     35.0,
     X   35.0,     35.0,     35.0,     35.0,     35.0,     35.0,
     X   35.0,     35.0,     35.0,     35.0,     35.0,     35.0,
     X   35.0,     35.0,     35.0,     35.0,     35.0,     35.0,
     X   35.0,     35.0,     35.0,     35.0,     35.0,     35.0,
     X   35.0,     35.0,     35.0,     35.0,     35.0,     35.0,
     X   35.0,     35.0,     35.0,     35.0,     35.0,     35.0,
     X   35.0,     35.0,     35.0,     35.0,     35.0,     35.0,
     X   35.0,     35.0,     35.0,     35.0,     35.0,     35.0,
     X   35.0,     35.0,     35.0,     35.0,     35.0,     35.0/
C
C          1.4  TABLE (IN DEG K) FOR SUPERADIABATIC CORRECTION
C
      DATA (ULAPS(I),I=1,10)/
     X          4.5,
     X          3.5,
     X          2.5,
     X          1.5,
     X          1.0,
     X          0.5,
     X          0.5,
     X          0.5,
     X          0.5,
     X          0.5/
      DATA (ULAPS(I),I=11,NLEV)/
     X          0.5,
     X          0.5,
     X          0.5,
     X          0.5,
     X          0.5,
     X          0.5,
     X          0.5,
     X          0.5,
     X          0.5,
     X          0.5,
     X          0.5,
     X          0.5/
C
      IERR=0
      U=R/RCP
      IF(NST.LE.1) GO TO 400
C
C           1.5  ZONE DEPENDS ON LATITUDE
C
      IZONE=INT(ABS(RLAT)/30.0)+1
      IF(IZONE.GT.3) IZONE=3
C
C    -------------------------------------------------------------------
C
C                    2.  INVERSION CHECK
C                   ---------------------
C
  200 CONTINUE
      DO 355 ILEV=1,NST-1
C
C           2.1  INTERPOLATE POSITION IN TABLES
C
      CALL PINTER (IERR,UPZ(1,1),PP(ILEV),POS)
      IF(IERR.NE.0) THEN
           IERR=0
           GO TO 355
      ENDIF
      IPOS=INT(POS)
C
C           2.2  DETERMINE MAXIMUM INVERSION ALLOWED
C
      PDIFF=PP(ILEV)-PP(ILEV+1)
      TDIFF=TT(ILEV+1)-TT(ILEV)
      IF(PDIFF.GT.ULAYT(IPOS)) THEN
           TLIM=UICL(ISEASN,IZONE,IPOS)
      ELSEIF(PDIFF.EQ.0) THEN
           TLIM=0.1
      ELSE
           TLIM=UINL(ISEASN,IZONE,IPOS)*PDIFF/100.0
      ENDIF
C
C           2.3  SET CONFIDENCE
C
      IF(TDIFF.GT.TLIM) THEN
           CALL SETINT (IFAIL,NNTT(ILEV),NNTT(ILEV+1),IEND)
C           WRITE (*,'(1H ,'' INVERSION CHECK     '',
C     X               A,2X,I4,4I2.2,4(A,F10.2))')
C     X               CALLSN,IY,IM,ID,IH,IN,
C     X               ' PP:',PP(ILEV),  ', TT:',TT(ILEV),
C     X              ', PP:',PP(ILEV+1),', TT:',TT(ILEV+1)
      ENDIF
C
C    -------------------------------------------------------------------
C
C                  3.  LAPSE RATE CHECK
C                -----------------------
C
  300 CONTINUE
C
C        3.1  LOOK FOR SUPERADIABATIC LAYER
C
c	print*,'LAPS',IPOS,PP(ILEV),PP(ILEV+1),TT(ILEV),
c     x TT(ILEV+1),PDIFF,TDIFF,TLIM,TTI

      TTI=TT(ILEV)*((PP(ILEV+1)/PP(ILEV))**U)-ULAPS(IPOS)
      
      IF(TT(ILEV+1).LT.TTI)
     X THEN
           LFLAG=.FALSE.
           IF(ILEV+2.LE.NST.AND.ILEV-1.GE.1) THEN
C
C        3.2  USE SURROUNDING LEVELS TO DETERMINE ERRONEOUS LEVEL
C
                CALL PINTER (IERR,UPZ(1,1),PP(ILEV-1),PI1)
                IF(IERR.NE.0) THEN
                     IERR=0
                     GO TO 355
                ENDIF
                IPI1=INT(PI1)
                TTI1=TT(ILEV-1)*((PP(ILEV+1)/PP(ILEV-1))**U)
                TTI2=TT(ILEV)  *((PP(ILEV+2)/PP(ILEV))  **U)
C               WRITE (*,'(1H ,''LAPS FAIL'',I3,10F10.2)')
C    X            IPI1,PP(ILEV-1),PP(ILEV),PP(ILEV+1),PP(ILEV+2),
C    X                 TT(ILEV-1),TT(ILEV),TT(ILEV+1),TT(ILEV+2),
C    X                 TTI1,TTI2
                IF(TT(ILEV+1).LT.TTI1-ULAPS(IPI1).AND.
     X             TT(ILEV+2).GE.TTI2-ULAPS(IPOS)) THEN
C
C        3.3  LEVEL I+1 IN ERROR
C
                     CALL SETGRS(NNTT(ILEV+1),2)
C                     WRITE (*,'(1H ,'' LAPSE RATE CHECK    '',
C     X               A,2X,I4,4I2.2,2(A,F10.2))')
C     X               CALLSN,IY,IM,ID,IH,IN,
C     X               ' PP:',PP(ILEV+1),', TT:',TT(ILEV+1)
                     LFLAG=.TRUE.
                ENDIF
                IF(TT(ILEV+1).GE.TTI1-ULAPS(IPI1).AND.
     X             TT(ILEV+2).LT.TTI2-ULAPS(IPOS)) THEN
C
C        3.4  LEVEL I IN ERROR
C
                     CALL SETGRS(NNTT(ILEV),2)
C                     WRITE (*,'(1H ,'' LAPSE RATE CHECK    '',
C     X               A,2X,I4,4I2.2,2(A,F10.2))')
C     X               CALLSN,IY,IM,ID,IH,IN,
C     X               ' PP:',PP(ILEV),  ', TT:',TT(ILEV)
                     LFLAG=.TRUE.
                ENDIF
           ENDIF
C
C        3.5  EITHER I OR I+1 IN ERROR
C
           IF(.NOT.LFLAG) THEN
                CALL SETINT (IFAIL,NNTT(ILEV),NNTT(ILEV+1),IEND)
C                WRITE (*,'(1H ,'' LAPSE RATE CHECK    '',
C     X               A,2X,I4,4I2.2,4(A,F10.2))')
C     X               CALLSN,IY,IM,ID,IH,IN,
C     X               ' PP:',PP(ILEV),  ', TT:',TT(ILEV),
C     X              ', PP:',PP(ILEV+1),', TT:',TT(ILEV+1)
           ENDIF
      ENDIF
  355 CONTINUE
C
C    ---------------------------------------------------------------------
C
C                       4.  EXIT
C                     -----------
C
  400 CONTINUE
      RETURN
      END
      SUBROUTINE HYDRO (IERR,NST,PP,TV,IVS,Z,NNZ,NNTT,NNTD,
     X                  CALLSN,IY,IM,ID,IH,IN)
C
C****  *HYDRO*
C
C       PURPOSE.
C      ----------
C
C           HYDROSTATIC CHECK OF STANDARD LEVEL HEIGHTS
C            AND TEMPERATURES
C
C       INTERFACE.
C      ------------
C
C           CALL HYDRO (IERR,NST,PP,TV,IVS,Z,NNZ,NNTT,NNTD,
C                       CALLSN,IY,IM,ID,IH,IN)
C
C        INPUT
C          NST     -  NUMBER OF TEMPERATURE LEVELS
C          PP      -  PRESSURE (PA)
C          TV      -  VIRTUAL TEMPERATURE (DEG K)
C          IVS     -  VERTICAL SIGNIFICANCE
C          Z       -  GEOPOTENTIAL (M)
C          NNZ     -  FLAG POSITIONS FOR GEOPOTENTIAL
C          NNTT    -  FLAG POSITIONS FOR TEMPERATURE
C          NNTD    -  FLAG POSITIONS FOR DEW POINT
C          CALLSN  -  STATION NUMBER OR CALL SIGN
C          IY,IM,ID,IH,IN - YEAR,MONTH,DAY,HOUR,MINUTE OF REPORT
C
C        OUTPUT
C          IERR    -  ERROR CODE
C          CONFIDENCE VALUES UPDATED
C
C       METHOD.
C      ---------
C
C         EXTRACT STANDARD LEVELS;
C         ADJUST TEMPERATURE IF TROPOPAUSE OCCURS;
C         CALCULATE EXTREME VALUES FOR THICKNESS;
C         SET CONFIDENCE VALUES DEPENDING ON RATIO OF THICKNESS
C         DEPARTURES IN NEIGHBOURING LAYERS.
C
C       EXTERNALS.
C      ------------
C
C         PINTER   -  INTERPOLATE TO PRESSURE OR HEIGHT LEVEL
C         SETINT   -  SET CONFIDENCE FLAGS FOR INTERNAL CONSISTENCY
C                      CHECK
C         SETGRS   -  SET CONFIDENCE FLAGS FOR GROSS LIMIT CHECK
C
C       REFERENCE.
C      ------------
C
C         GUIDE ON THE GLOBAL DATA PROCESSING SYSTEM WMO N305 1982
C            CHAPTER 6, QUALITY CONTROL PROCEDURES
C
C       AUTHOR.
C      ---------
C
C         B. NORRIS,  ECMWF,  MARCH 1990
C
C       MODIFICATIONS.
C      ----------------
C
C         NONE
C
      INCLUDE 'paramq.f'
      INCLUDE 'const.f'
      INCLUDE 'conf.f'
      INCLUDE 'ctua.f'
      DIMENSION PP(*),TV(*),IVS(*),Z(*),NNZ(*),NNTT(*),NNTD(*)
      DIMENSION STDPP(NLEV),STDTV(NLEV),STDZ(NLEV),STDTC(NLEV)
      DIMENSION ISTDZ(NLEV),ISTDTT(NLEV),ISTDTD(NLEV)
      DIMENSION E(NLEV),D(NLEV),DA(NLEV),DB(NLEV),IPOS(NLEV)
      CHARACTER*(*) CALLSN
      LOGICAL LFLAG,LTROP
C
C    --------------------------------------------------------------------
C
C                     1. SET UP
C                   ------------
C
  100 CONTINUE
C
C         CRITICAL CONFIDENCE VALUE
      DATA ICRIT/65/
      IERR=0
      U=R/RCP
      IF(NST.LE.1) GO TO 400
C
C           1.1  EXTRACT STANDARD AND TROPOPAUSE LEVELS
C
      LTROP=.FALSE.
      NSTD=0
      DO 112 ILEV=1,NST
      IF(((IVS(ILEV).AND.32).EQ.32).AND.NSTD.LT.NLEV) THEN
           NSTD=NSTD+1
           STDPP (NSTD)=  PP(ILEV)
           STDTV (NSTD)=  TV(ILEV)
           STDTC (NSTD)=STDTV(NSTD)
           STDZ  (NSTD)=  Z (ILEV)
           ISTDZ (NSTD)= NNZ(ILEV)
           ISTDTT(NSTD)=NNTT(ILEV)
           ISTDTD(NSTD)=NNTD(ILEV)
C
C       1.11  OBTAIN POSITION OF EACH LEVEL WITHIN TABLE OF
C                 STANDARD LEVELS
C
           CALL PINTER (IERR,UPZ(1,1),STDPP(NSTD),POS)
           IF(IERR.NE.0) THEN
                IERR=0
                IPOS(NSTD)=1000
           ELSE
                IPOS(NSTD)=NINT(POS)
           ENDIF
C
C       1.12  ADJUST TEMPERATURE AT TOP OF LAYER IF TROPOPAUSE
C                 OCCURS WITHIN LAYER
C
           IF(NSTD.GT.1) THEN
           IF(LTROP.AND.IPOS(NSTD)-IPOS(NSTD-1).EQ.1.AND.
     X        TTROP.NE.RMISS.AND.PTROP.NE.RMISS.AND.
     X        STDTV(NSTD).NE.RMISS.AND.STDTV(NSTD-1).NE.RMISS)
     X            STDTC(NSTD)=STDTV(NSTD)+TTROP-STDTV(NSTD-1)-
     X               (STDTV(NSTD)-STDTV(NSTD-1))*
     X               (ALOG(STDPP(NSTD-1)/PTROP)/
     X                ALOG(STDPP(NSTD-1)/STDPP(NSTD)))
           END IF
           LTROP=.FALSE.
      ENDIF
C
C          1.13  EXTRACT TROPOPAUSE
C
      IF((IVS(ILEV).AND.16).EQ.16) THEN
           PTROP=PP(ILEV)
           TTROP=TV(ILEV)
           LTROP=.TRUE.
      ENDIF
  112 CONTINUE
      IF(NSTD.LE.1) GO TO 400
C
C          1.2  INITIALIZE ARRAYS
C
      DO 122 ILEV=1,NLEV
      E (ILEV)=RMISS
      D (ILEV)=RMISS
      DA(ILEV)=RMISS
      DB(ILEV)=RMISS
  122 CONTINUE
C
C    -------------------------------------------------------------------
C
C                2.  CALCULATE EXTREME THICKNESSES
C               -----------------------------------
C
  200 CONTINUE
      DO 265 ILEV=1,NSTD-1
C
C              2.0  ENSURE STANDARD LEVELS ARE ADJACENT
C
      IF(IABS(IPOS(ILEV)-IPOS(ILEV+1)).GT.1) GO TO 265
      IF(STDTV(ILEV).NE.RMISS.AND.STDTC(ILEV+1).NE.RMISS) THEN
           RAL=(R/(2.0*RG))*ALOG(STDPP(ILEV)/STDPP(ILEV+1))
           SU=(STDPP(ILEV+1)/STDPP(ILEV))**U
C
C              2.1  CALCULATED THICKNESS
C
           D(ILEV)=RAL*(STDTV(ILEV)+STDTC(ILEV+1))
C
C              2.2  TEMPERATURE AT ILEV+1 EXTRAPOLATED FROM ILEV
C
           TTI1=STDTV(ILEV)*SU
C
C              2.3  TEMPERATURE AT ILEV EXTRAPOLATED FROM ILEV+1
C
           TTI=STDTC(ILEV+1)/SU
C
C              2.4  MAXIMUM POSSIBLE THICKNESS
C
           DA(ILEV)=RAL*(TTI+STDTC(ILEV+1))
C
C              2.5  MINIMUM POSSIBLE THICKNESS
C
           DB(ILEV)=RAL*(STDTV(ILEV)+TTI1)
C
C              2.6  THICKNESS DEPARTURE
C
           IF(STDZ(ILEV).NE.RMISS.AND.STDZ(ILEV+1).NE.RMISS.AND.
     X        IFLAG(1,ISTDZ(ILEV)).GE.ICRIT.AND.
     X        IFLAG(1,ISTDZ(ILEV+1)).GE.ICRIT)
     X     E(ILEV)=STDZ(ILEV+1)-STDZ(ILEV)-D(ILEV)
C          WRITE (*,'(1H ,''HYDRO'',2I3,10F10.2)')
C    X       IPOS(ILEV),IPOS(ILEV+1),
C    X       STDPP(ILEV),STDTV(ILEV),STDTC(ILEV),STDZ(ILEV),D(ILEV),
C    X       TTI1,TTI,DA(ILEV),DB(ILEV),E(ILEV)
      ENDIF
  265 CONTINUE
C
C    -------------------------------------------------------------------
C
C                  3.  COMPARE WITH TOLERANCE
C                ------------------------------
C
  300 CONTINUE
      LFLAG=.FALSE.
      DO 375 ILEV=1,NSTD-1
      IF(LFLAG) THEN
C
C           3.1  SKIP ALREADY FLAGGED LEVEL
C
           LFLAG=.FALSE.
           GO TO 375
      ENDIF
C
C           3.2  CALCULATE TOLERANCE
C
      IF(E(ILEV).NE.RMISS) THEN
           TOL=0.375*ABS(DA(ILEV)-DB(ILEV))
           IF(TOL.LT.20.0) TOL=20.0
           IF(STDPP(ILEV).GT.40000.0) THEN
                IF(TOL.GT.50.0) TOL=50.0
           ELSE
                IF(TOL.GT.80.0) TOL=80.0
           ENDIF
C          WRITE (*,'(1H ,''HYDRO'',3F10.2)') STDPP(ILEV),TOL,E(ILEV)
           IF(ABS(E(ILEV)).GT.TOL) THEN
C
C        3.3  E EXCEEDS TOLERANCE; COMPARE WITH ADJACENT LAYER
C
                IF(E(ILEV+1).NE.RMISS.AND.ABS(E(ILEV+1)).GT.0.001) THEN
                     F=E(ILEV)/E(ILEV+1)
C                    WRITE (*,'(1H ,''HYDRO FAIL'',5F10.2)')
C    X                  STDPP(ILEV),TOL,E(ILEV),E(ILEV+1),F
                     IF(F.GT.0.5.AND.F.LT.2.0) THEN
C
C        3.4  TEMPERATURE AT ILEV+1 IS SUSPECT
C
                          CALL SETGRS (ISTDTT(ILEV+1),3)
                          CALL SETGRS (ISTDTD(ILEV+1),3)
C                          WRITE (*,'(1H ,'' HYDROSTATIC CHECK   '',
C     X                    A,2X,I4,4I2.2,2(A,F10.2))')
C     X                    CALLSN,IY,IM,ID,IH,IN,
C     X                    ' PP:',STDPP(ILEV+1),', TV:',STDTV(ILEV+1)
                          LFLAG=.TRUE.
                     ELSEIF(F.GT.-2.0.AND.F.LT.-0.5) THEN
C
C        3.5  GEOPOTENTIAL AT ILEV+1 IS SUSPECT
C
                          CALL SETGRS (ISTDZ(ILEV+1),3)
C                          WRITE (*,'(1H ,'' HYDROSTATIC CHECK   '',
C     X                    A,2X,I4,4I2.2,2(A,F10.2))')
C     X                    CALLSN,IY,IM,ID,IH,IN,
C     X                    ' PP:',STDPP(ILEV+1),',  Z:',STDZ(ILEV+1)
                          LFLAG=.TRUE.
                     ELSEIF(ABS(F).GE.2.0) THEN
C
C        3.6  ALL HEIGHTS AT AND ABOVE ILEV+1 ARE SUSPECT
C
                          DO 362 I=ILEV+1,NSTD
                          CALL SETGRS (ISTDZ(I),3)
C                          WRITE (*,'(1H ,'' HYDROSTATIC CHECK   '',
C     X                    A,2X,I4,4I2.2,2(A,F10.2))')
C     X                    CALLSN,IY,IM,ID,IH,IN,
C     X                    ' PP:',STDPP(I),',  Z:',STDZ(I)
  362                     CONTINUE
                          GO TO 400
                     ENDIF
                ENDIF
C
C        3.7  NO DEFINITE DECISION
C
                IF(.NOT.LFLAG) THEN
                     CALL SETINT (IFAIL,
     X                             ISTDZ(ILEV), ISTDZ(ILEV+1),
     X                            ISTDTT(ILEV),ISTDTT(ILEV+1),
     X                            ISTDTD(ILEV),ISTDTD(ILEV+1),IEND)
C                     WRITE (*,'(1H ,'' HYDROSTATIC CHECK   '',
C     X               A,2X,I4,4I2.2,3(A,F10.2)/1H ,44X,3(A,F10.2))')
C     X               CALLSN,IY,IM,ID,IH,IN,
C     X  ' PP:',STDPP(ILEV),  ',  Z:',STDZ(ILEV),  ', TV:',STDTV(ILEV),
C     X  ' PP:',STDPP(ILEV+1),',  Z:',STDZ(ILEV+1),', TV:',STDTV(ILEV+1)
                ENDIF
           ENDIF
      ENDIF
  375 CONTINUE
C
C    ---------------------------------------------------------------------
C
C                       4.  EXIT
C                     -----------
C
  400 CONTINUE
      RETURN
      END
      SUBROUTINE INSHIZ (IERR,CALLSN,ZSTN)
C
C****  *INSHIZ*
C
C       PURPOSE.
C      ----------
C
C         RETURN HEIGHT OF SHIP'S PLATFORM
C
C       INTERFACE.
C      ------------
C
C         CALL INSHIZ (IERR,CALLSN,ZSTN)
C
C        INPUT
C           CALLSN   -  CALL SIGN OF CURRENT REPORT
C
C        OUTPUT
C           ZSTN     -  HEIGHT OF PLATFORM
C           IERR     -  RETURN CODE
C
C       METHOD.
C      ---------
C
C           MAKE SURE FILE OF SHIP CALL SIGNS AND HEIGHTS IS READ;
C           EXTRACT HEIGHT FOR CURRENT CALL SIGN.
C
C       EXTERNALS.
C      ------------
C
C        NONE
C
C       REFERENCE.
C      ------------
C
C        NONE
C
C       AUTHOR.
C      ---------
C
C        B. NORRIS,  ECMWF,  JANUARY 1991
C
C       MODIFICATIONS.
C      ----------------
C
C        NONE
C
      INCLUDE 'paramsh.f'
      INCLUDE 'comshiz.f'
      CHARACTER*(*) CALLSN
      IERR=0
C
C    ---------------------------------------------------------------------
C
C                    1. READ SHIP HEIGHT FILE
C                   --------------------------
C
  100 CONTINUE
      IF(IFSHIZ.NE.3H1ST) THEN
C
C              1.1  OPEN FILE
C
           OPEN(82,IOSTAT=IOS,ERR=122,
     X          FILE='ship_height.dat',
     X          STATUS='OLD',FORM='FORMATTED')
           NSHIZ=0
  111      CONTINUE
           IF(NSHIZ+1.GT.NMSHIZ) THEN
                WRITE(*,7293) nshiz
                IERR=1003
                GO TO 128
           ENDIF
7293  format(1h,' INSHIZ : TOO MANY SHIPS SPECIFIED :',i5)
           NSHIZ=NSHIZ+1
C
C              1.2  READ FILE
C
           READ (82,'(A9,F3.0)',ERR=124,END=128)
     X        CSHIZ(NSHIZ),SHIZ(NSHIZ)
           GO TO 111
  122      CONTINUE
           WRITE (*,7306)ios
           IERR=1001
           GO TO 300
  124      CONTINUE
           WRITE (*,7306)ios
           IERR=1002
           GO TO 300
  128      CONTINUE
7306  format(1h,' ERROR READING SHIP HEIGHT FILE.IOSTAT = ',i5)  
C
C              1.3  CLOSE FILE
C
           CLOSE (82)
           IFSHIZ=3H1ST
      ENDIF
C
C    ---------------------------------------------------------------------
C
C                    2. CHECK MATCH IN CALL SIGN
C                   -----------------------------
C
  200 CONTINUE
C
C         2.1  SEARCH ARRAY OF CALL SIGNS
C
      DO 215 I=1,NSHIZ
      IF(CALLSN.EQ.CSHIZ(I)) THEN
           ZSTN=SHIZ(I)
           WRITE (*,7335) CALLSN,ZSTN
           GO TO 300
      ENDIF
7335  format(1h,' HEIGHT FOR SHIP ',a9,' SET TO ',F5.1) 
  215 CONTINUE
C
C                2.2  NO MATCH FOUND, SET HEIGHT TO ZERO
C
      ZSTN=0.0
C
C    ---------------------------------------------------------------------
C
C                    3. EXIT
C                   ---------
C
  300 CONTINUE
      RETURN
      END
      SUBROUTINE BIASCOR (LBC,NST,SPP,SZ,SZNBC,
     X                    YSTIDBC,YSTYPBC,NSTBC,YSNAMBC,RCORBC,NSOBC, 
     X                    CIDENT,IM,ID,IH,RLAT,RLONG)
C
C**** *BIASCOR*
C
C       PURPOSE.
C      ----------
C
C          BIASCORRECTION FOR GEOPOTENTIAL HEIGHTS OF TEMPS
C
C       INTERFACE.
C      ------------
C
C          CALL BIASCOR (IERR,NST,SPP,SZ,SZNBC,
C     X                  YSTIDBC,YSTYPBC,NSTBC,YSNAMBC,RCORBC,NSOBC, 
C     X                  CIDENT,IM,ID,IH,IN,RLAT,RLONG)
C
C        INPUT
C         NST       NUMBER OF LEVELS
C         SPP       ARRAY WITH PRESSURE VALUES
C         SZ        ARRAY WITH Z VALUES
C         YSTIDBC   ARRAY WITH TEMP IDENTIFIER
C         YSTYPBC   ARRAY WITH SONDE TYPES
C         NSTBC     NUMBER OF ENTRIES IN T1ID AND T1TYP
C         YSNAMBC   ARRAY WITH SONDE TYPES
C         RCORBC    ARRAY (3DIM) WITH BIAS CORRECTION VALUES
C         NSOBC     NUMBER OF ENTRIES IN T2TYP
C         CIDENT    STATION IDENTIFIER
C         IM        MONTH
C         ID        DAY
C         IH        HOUR
C         RLAT      LATITUDE
C         RLONG     LONGITUDE
C         
C
C        OUTPUT
C         LBC       LOGICAL TO STATE IF BIAS COR. WAS SUCCESSFUL
C         SZNBC     ARRAY WITH BIAS CORRECTED Z VALUES
C
C       METHOD.
C      ---------
C
C        
C
C       EXTERNALS.
C      ------------
C
C        DIURNAL
C        PNTERP
C
C       REFERENCE.
C      ------------
C
C       AUTHOR.
C      ---------
C
C
C
C       MODIFICATIONS.
C      ----------------
C
C-------------------------------------------------------------------
      IMPLICIT LOGICAL (L,O,G)
      INCLUDE 'parameter.f'
      INCLUDE 'paramq.f'
      INCLUDE 'paramsh.f'
      PARAMETER (IUNIQ=12)
      PARAMETER (JTEMP=100)
      INCLUDE 'comwt.f'
      INCLUDE 'comkey.f'
      INCLUDE 'const.f'
      INCLUDE 'conf.f'
      DIMENSION SPP(JTEMP)
      DIMENSION SZ(JTEMP)
      CHARACTER*9 CIDENT
      INCLUDE 'parabias.f'
C-------------------------------------------------------------------
C
C          1.BC  DETERMINE SONDE TYPE
C
      DO 222 I=1,NSTBC
      IF(CIDENT.EQ.YSTIDBC(I)) THEN
           KSOBC=I
           GO TO 224
      ENDIF
  222 CONTINUE
      KSOBC=0
  224 CONTINUE
C
C
      IF(KSOBC.EQ.0.OR.YSTYPBC(KSOBC).EQ.'unknown'.OR.
     X                 YSTYPBC(KSOBC).EQ.'Vais/Gr'.OR.
     X                 YSTYPBC(KSOBC).EQ.'Mesei'.OR.
     X                 YSTYPBC(KSOBC).EQ.'few obs') THEN
            LBC=.FALSE.
            GO TO 400
      ENDIF
C
C              DETERMINE SECTION OF TABLE 2
C
      DO 228 I=1,NSOBC
      IF(YSTYPBC(KSOBC).EQ.YSNAMBC(I)) THEN
            ISON=I
            GO TO 229
      ENDIF
  228 CONTINUE
      LBC=.FALSE.
      GO TO 230
  229 CONTINUE
C
C      WRITE (*,*) ' ISON ',ISON
C
C              CALCULATE SOLAR ELEVATION
C
C     DLATBC=FLOAT(IAOF(8))/100000.0 - 90.0
C     DLONBC=FLOAT(IAOF(9))/100000.0 - 180.0
C     HH=FLOAT(IAOF(11))/100.0
C
      CALL DIURNAL ( IM, ID, IH, RLAT,RLONG, ANGLEBC )
C      WRITE (*,*) ' LAT,LON,HH,ANGLE ',RLAT,RLONG,IH,ANGLEBC
C
C              CALCULATE CORRECT COLUMN OF TABLE 2
C
      NELEVBC=NINT(ANGLEBC)
      IF(NELEVBC.LT.0) NELEVBC=NELEVBC-4
      ICOL=(NELEVBC+2)/5+3
      IF(ICOL.LT.1) ICOL=1
      IF(ICOL.GT.NCORBC) ICOL=NCORBC
C      WRITE (*,*) ' ICOL,NELEVBC,NCORBC ',ICOL,NELEVBC,NCORBC
  230 CONTINUE
C
C#########################################################
      DO 240 I=1,NST
      IF(SPP(I).EQ.RMISS) GO TO 240
      IPP=NINT(SPP(I))
      CALL PNTERP (ILEVBC,NLEVBC,IPP,POS)
C
      ILOW=IFIX(POS)
      IHGH=ILOW+1
      RADD=RCORBC(ICOL,ILOW,ISON)*(FLOAT(IHGH)-POS)+
     X          RCORBC(ICOL,IHGH,ISON)*(POS-FLOAT(ILOW))
C
      IF(SZ(I).NE.RMISS) THEN
         SZNBC(I)=SZ(I)+RADD
      ENDIF

C      WRITE (*,*) 'NST, PP,Z,ZB,POS,RCOR,RADD,IADD ',
C     X         NST,IPP,SZ(I),SZNBC(I),POS,
C     X         RCORBC(ICOL,ILOW,ISON),RADD,IADD
C
  240 CONTINUE
C#############################################################
C--------------------------------------------------------------------
C    
C                   4.  EXIT
C                  -----------
C
  400 CONTINUE
      RETURN
      END
      SUBROUTINE RBCFILES (IERR,YSTIDBC,YSTYPBC,NSTBC,
     X                     YSNAMBC,RCORBC,NSOBC)
C
C**** *RBCFILES*
C
C       PURPOSE.
C      ----------
C
C          READ BIASCORRECTION REFERENCE FILES
C
C       INTERFACE.
C      ------------
C
C        CALL BIASCOR (IERR,YSTIDBC,YSTYPBC,NSTBC,YSNAMBC,RCORBC,NSOBC)
C
C        INPUT
C         NONE
C         
C
C        OUTPUT
C         IERR    ERROR CODE
C         YSTID   ARRAY WITH STATION IDENTIFIERS
C         YSTYP   ARRAY WITH SONDE TYPES
C         NST     NUMBER OF ENTRIES IN YSTIDBC, YSTYPBC
C         YSNAM   ARRAY WITH STATION IDENTIFIERS
C         RCOR    ARRAY (3DIM) WITH BIAS CORRECTION VALUES
C         NSO     NUMBER OF SONDE TYPES IN YSNAMBC
C
C       METHOD.
C      ---------
C
C        
C
C       EXTERNALS.
C      ------------
C
C        NONE
C
C       REFERENCE.
C      ------------
C
C       AUTHOR.
C      ---------
C
C
C       MODIFICATIONS.
C      ----------------
C
C-----------------------------------------------------------------
      IMPLICIT LOGICAL (L,O,G)
      INCLUDE 'parameter.f'
      INCLUDE 'paramq.f'
      INCLUDE 'paramsh.f'
      PARAMETER (IUNIQ=12)
      PARAMETER (JTEMP=100)
      INCLUDE 'comwt.f'
      INCLUDE 'comkey.f'
      INCLUDE 'comkeyc.f'
      INCLUDE 'const.f'
      INCLUDE 'conf.f'
      DIMENSION SPP(JTEMP)
      DIMENSION SZ(JTEMP)
      CHARACTER*9 CIDENT
      INCLUDE 'parabias.f'
c
C-------------------------------------------------------------------
C
C
C              OPEN FILES
C
      OPEN(UNIT=65, FILE='biascor1.tab',
     X     IOSTAT=IOS,ERR=55,STATUS='OLD')
      GO TO 56
   55 CONTINUE
      WRITE(*,7588)IOS
   56 CONTINUE
7588  format(1h,'BIASCOR ERROR:OPENING OF BIASCOR1 FAILED.iOSTAT = ',i5)
C
      OPEN(UNIT=66, FILE='biascor2.tab',
     X     IOSTAT=IOS,ERR=65,STATUS='OLD')
      GO TO 66
   65 CONTINUE
      WRITE(*,7596)ios
7596  format(1h,'BIASCOR ERROR:OPENING OF BIASCOR2 FAILED.iOSTAT = ',i5)
   66 CONTINUE
C
C
C              READ TABLE 1 : STATION LIST & SONDE TYPE
C
      NSTBC=0
  102 CONTINUE
      IF (NSTBC.LT.NSTNBC) THEN
           NSTBC=NSTBC+1
           READ (65,'(A,1X,A)',END=104) YSTIDBC(NSTBC),YSTYPBC(NSTBC)
           GO TO 102
      ELSE
           WRITE (*,'(1X,A,I5)') ' DIMENSION NSTN TOO SMALL :',NSTNBC
           IERR=1
           GO TO 400
      ENDIF
  104 CONTINUE
      NSTBC=NSTBC-1
      WRITE (*,*) ' NUMBER OF STATIONS IN TABLE 1 ',NSTBC
C
C              READ TABLE 2 : SONDE CORRECTIONS
C
      NSOBC=0
  112 CONTINUE
      IF(NSOBC.LT.NSONBC) THEN
           NSOBC=NSOBC+1
           READ (66,'(2X,A)',END=118) YSNAMBC(NSOBC)
           READ (66,'(A)',END=118) YDUMMYBC
           DO 116 J=1,NLEVBC
           READ (66,'(5X,8F8.2/5X,8F8.2)',END=118)
     X            (RCORBC(K,J,NSOBC),K=1,NCORBC)
  116      CONTINUE
           READ (66,'(A)',END=118) YDUMMYBC
           GO TO 112
      ELSE
           WRITE (*,'(1X,A,I5)') ' DIMENSION NSON TOO SMALL :',NSONBC
           IERR=2
           GO TO 400
      ENDIF
  118 CONTINUE
      WRITE (*,*) ' NUMBER OF SONDES IN TABLE 2 ',NSOBC
C
C                CLOSE FILES
C
      CLOSE(65)
      CLOSE(66)

C
C-----------------------------------------------------------------
C                   4.  EXIT
C                  -----------
  400 CONTINUE
C
C
C
      RETURN
      END
      SUBROUTINE PNTERP (ILEV,NLEV,IPP,POS)
C
C**** *PNTERP*
C
C       PURPOSE.
C      ----------
C
C           INTERPOLATE POSITION OF PRESSURE OR HEIGHT
C               WITHIN TABLE OF UPPER AIR LEVELS
C
C       INTERFACE.
C      ------------
C
C         CALL PNTERP (ILEV,NLEV,IPP,POS)
C
C        INPUT
C         ILEV   -  TABLE OF STANDARD PRESSURE OR HEIGHT LEVELS
C         NLEV   -  DIMENSION OF ILEV
C         IPP    -  PRESSURE LEVEL
C
C        OUTPUT
C         IERR   -  ERROR CODE
C         POS    -  INTERPOLATED POSITION WITHIN TABLE
C
C       METHOD.
C      ---------
C
C           IF VALUES OF ILEV INCREASE, HEIGHT ASSUMED, LINEAR IN Z
C           IF VALUES OF ILEV DECREASE, PRESSURE ASSUMED, LINEAR IN LOGP
C
C       EXTERNALS.
C      -----------
C
C         NONE
C
C       REFERENCE.
C      ------------
C
C         NONE
C
C       AUTHOR.
C      ---------
C
C         B. NORRIS,  ECMWF,  JANUARY 1989.
C
C       MODIFICATIONS.A
C      ----------------
C
C         NONE
C
      DIMENSION ILEV(NLEV)
      LOGICAL LDOWN
      DATA EPS/0.000001/
C
C    -------------------------------------------------------------------
C
C                       1.  SETUP
C                     ------------
C
  100 CONTINUE
      IERR=0
      LDOWN=.FALSE.
      IF(ILEV(1).GT.ILEV(NLEV)) LDOWN=.TRUE.
C
C    -------------------------------------------------------------------
C
C           2.  CHECK WHETHER VALUE IS OUTSIDE TABLE LIMITS
C         --------------------------------------------------
C
  200 CONTINUE
      IF(LDOWN) THEN
           IF(IPP.GE.ILEV(1)) THEN
                POS=1.0+EPS
                GO TO 400
           ENDIF
           IF(IPP.LE.ILEV(NLEV)) THEN
                POS=FLOAT(NLEV)-EPS
                GO TO 400
           ENDIF
      ELSE
           IF(IPP.LE.ILEV(1)) THEN
                POS=1.0+EPS
                GO TO 400
           ENDIF
           IF(IPP.GE.ILEV(NLEV)) THEN
                POS=FLOAT(NLEV)-EPS
                GO TO 400
           ENDIF
      ENDIF
C
C    ---------------------------------------------------------------------------------------
C
C          3.   VALUE IS BETWEEN MAX AND MIN TABLE VALUES
C         ------------------------------------------------
C
  300 CONTINUE
      DO 312 JLEV=2,NLEV-1
C
C         3.1   CHECK IF VALUE  = TABLE VALUE
C
      IF(IPP.EQ.ILEV(JLEV)) THEN
           POS=FLOAT(JLEV)
           GO TO 400
      ENDIF
  312 CONTINUE
C
C         3.2   VALUE IS BETWEEN 2 TABLE VALUES
C
      DO 322 JLEV=1,NLEV-1
      IF(LDOWN) THEN
           IF(IPP.LE.ILEV(JLEV).AND.IPP.GE.ILEV(JLEV+1)) THEN
                JPOS=JLEV
                GO TO 324
           ENDIF
      ELSE
           IF(IPP.GE.ILEV(JLEV).AND.IPP.LE.ILEV(JLEV+1)) THEN
                JPOS=JLEV
                GO TO 324
           ENDIF
      ENDIF
  322 CONTINUE
  324 CONTINUE
C
C           3.3   INTERPOLATE
C
      IF(LDOWN) THEN
           POS=FLOAT(JPOS)+
     X         (ALOG(FLOAT(ILEV(JPOS)))-ALOG(FLOAT(IPP)))/
     X         (ALOG(FLOAT(ILEV(JPOS)))-ALOG(FLOAT(ILEV(JPOS+1))))
      ELSE
           POS=FLOAT(JPOS)+
     X  (FLOAT(ILEV(JPOS))-FLOAT(IPP))/
     X  (FLOAT(ILEV(JPOS))-FLOAT(ILEV(JPOS+1)))
      ENDIF
C
C    --------------------------------------------------------------------
C
C                   4.   EXIT
C                  -----------
C
  400 CONTINUE
      RETURN
      END
      SUBROUTINE DIURNAL(MM,DD,HH,DLAT,DLON,ANGLE)
C
C
      INTEGER MM,DD,HH
      PI=2.0*ASIN(1.0)
      TWOPI=2.0*PI
      RADCON=TWOPI/360.0
C
C*** SINUS OF SOLAR ELEVATION ANGLE WILL BE APPROXIMATED ************
C*** ANTTI LANGE, ECMWF, 25 JULY 1984.                   ************
C
C      YY=YYMMDD/10000
C      MMDD=YYMMDD-YY*10000
C      MM=MMDD/100
C      DD=MMDD-MM*100
      DAY=FLOAT(MM)*30.44+FLOAT(DD)-31.44
      SEASON=TWOPI*(DAY+9.5)/365.25
      COSSSN=COS(SEASON)
      DECL=-RADCON*23.5*COSSSN
      RLAT=RADCON*DLAT
      SINLAT=SIN(RLAT)
      COSLAT=COS(RLAT)
      RLON=RADCON*DLON
      RLHH=TWOPI*HH/24.00+RLON-PI
      COSLHH=COS(RLHH)
      SINE=SIN(DECL)*SINLAT+COS(DECL)*COSLAT*COSLHH
C
      ANGLE = ASIN(SINE) * 180. / PI
C
      RETURN
      END
c
      subroutine readseq(iunit,ierr,ios,irecnum,key,yfilename,rec)
c
      integer iunit,ierr,ios,irecnum
      character*9 key
      character*19 yfilename
c
      structure/track/
      character*9 yident
      integer*4   kslen
      integer*4   ksent
      integer*4   kspar
      integer*4   kinnt
      integer*4   krec(512)
      end structure
      record /track/ rec
c
      character*9 key_table(5000)
c
      common /com_krecords/ krecords ! p/ contornar erro oriundo do
                                     ! enderecamento de variavel 
                                     ! local na area de dados.
C
      krecords = krecords + 1
      do 11 i=1,krecords
         if(key_table(i)(1:9).eq.key(1:9)) then
            irec = i
            go to 12
         end if
 11   continue
      irec = krecords
 12   irecnum = irec
      if(irec.lt.krecords) then
        krecords = krecords - 1
      else
        key_table(irec)(1:9) = key(1:9)
        go to 100
      end if
c
      ios = 0
 10   read(unit=iunit,iostat=ios,err=100,rec=irec) rec 
      ios = 0
      ierr = 0
      return
 100  continue
      if(ios.ne.0) go to 200
      ios = 142
      ierr = 1
      return
 200  write(6,220) iunit,ios
 220  format(' ********** ERROR DURING READING FILE ',i2,
     1       ' IOSTATUS = ',i3)
      ierr = -1
      return
      end
